@@ -5,12 +5,12 @@
 (*                                                                     *)
 (*        Daniel de Rauglaudre, projet Cristal, INRIA Rocquencourt     *)
 (*                                                                     *)
-(*  Copyright 1998 Institut National de Recherche en Informatique et   *)
+(*  Copyright 2001 Institut National de Recherche en Informatique et   *)
 (*  Automatique.  Distributed only by permission.                      *)
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: ast2pt.ml,v 2.10 2000/04/17 20:23:56 ddr Exp $ *)
+(* $Id: ast2pt.ml,v 3.13 2001/07/13 13:56:50 ddr Exp $ *)
 
 open Stdpp;
 open MLast;
@@ -27,6 +27,16 @@ value get_tag x =
 
 value error loc str = raise_with_loc loc (Failure str);
 
+value char_of_char_token loc s =
+  try Token.eval_char s with
+  [ Failure _ as exn -> raise_with_loc loc exn ]
+;
+
+value string_of_string_token loc s =
+  try Token.eval_string s with
+  [ Failure _ as exn -> raise_with_loc loc exn ]
+;
+
 value mkloc (bp, ep) =
   {Location.loc_start = bp; Location.loc_end = ep; Location.loc_ghost = False}
 ;
@@ -47,7 +57,7 @@ value shd =
   let tab = Hashtbl.create 701 in
   fun s ->
     try Hashtbl.find tab s with
-    [ Not_found -> do Hashtbl.add tab s s; return s ]
+    [ Not_found -> do { Hashtbl.add tab s s; s } ]
 ;
 ... but no more implemented because semantic problems with Ocaml:
 let x = "bar" and y = "bar"; x.[0] <- 'c'; print_string y;;
@@ -59,15 +69,19 @@ value ldot l s = Ldot l (shd s);
 
 value conv_con =
   let t = Hashtbl.create 73 in
-  do List.iter (fun (s, s') -> Hashtbl.add t s s')
-       [("True", "true"); ("False", "false")];
-  return fun s -> try Hashtbl.find t s with [ Not_found -> s ]
+  do {
+    List.iter (fun (s, s') -> Hashtbl.add t s s')
+      [("True", "true"); ("False", "false")];
+    fun s -> try Hashtbl.find t s with [ Not_found -> s ]
+  }
 ;
 
 value conv_lab =
   let t = Hashtbl.create 73 in
-  do List.iter (fun (s, s') -> Hashtbl.add t s s') [("val", "contents")];
-  return fun s -> try Hashtbl.find t s with [ Not_found -> s ]
+  do {
+    List.iter (fun (s, s') -> Hashtbl.add t s s') [("val", "contents")];
+    fun s -> try Hashtbl.find t s with [ Not_found -> s ]
+  }
 ;
 
 value array_function str name =
@@ -102,9 +116,11 @@ value rec ctyp_fa al =
 value rec ctyp_long_id =
   fun
   [ TyAcc _ m (TyLid _ s) ->
-      let (is_cls, li) = ctyp_long_id m in (is_cls, ldot li s)
+      let (is_cls, li) = ctyp_long_id m in
+      (is_cls, ldot li s)
   | TyAcc _ m (TyUid _ s) ->
-      let (is_cls, li) = ctyp_long_id m in (is_cls, ldot li s)
+      let (is_cls, li) = ctyp_long_id m in
+      (is_cls, ldot li s)
   | TyApp _ m1 m2 ->
       let (is_cls, li1) = ctyp_long_id m1 in
       let (_, li2) = ctyp_long_id m2 in
@@ -135,9 +151,9 @@ value rec ctyp =
       let (is_cls, li) = ctyp_long_id f in
       if is_cls then mktyp loc (Ptyp_class li (List.map ctyp al) [])
       else mktyp loc (Ptyp_constr li (List.map ctyp al))
-  | TyArr loc (TyLab loc1 lab t1) t2  ->
+  | TyArr loc (TyLab loc1 lab t1) t2 ->
       mktyp loc (Ptyp_arrow lab (ctyp t1) (ctyp t2))
-  | TyArr loc (TyOlb loc1 lab t1) t2  ->
+  | TyArr loc (TyOlb loc1 lab t1) t2 ->
       let t1 = TyApp loc1 (TyLid loc1 "option") t1 in
       mktyp loc (Ptyp_arrow ("?" ^ lab) (ctyp t1) (ctyp t2))
   | TyArr loc t1 t2 -> mktyp loc (Ptyp_arrow "" (ctyp t1) (ctyp t2))
@@ -172,8 +188,9 @@ and meth_list loc fl v =
 ;
 
 value mktype loc tl cl tk tm =
-  {ptype_params = tl; ptype_cstrs = cl; ptype_kind = tk; ptype_manifest = tm;
-   ptype_loc = mkloc loc}
+  let (params, variance) = List.split tl in
+  {ptype_params = params; ptype_cstrs = cl; ptype_kind = tk;
+   ptype_manifest = tm; ptype_loc = mkloc loc; ptype_variance = variance}
 ;
 value mkmutable m = if m then Mutable else Immutable;
 value mkprivate m = if m then Private else Public;
@@ -184,7 +201,8 @@ value type_decl tl cl =
   [ TyMan loc t (TyRec _ ltl) ->
       mktype loc tl cl (Ptype_record (List.map mktrecord ltl)) (Some (ctyp t))
   | TyMan loc t (TySum _ ctl) ->
-      mktype loc tl cl (Ptype_variant (List.map mkvariant ctl)) (Some (ctyp t))
+      mktype loc tl cl (Ptype_variant (List.map mkvariant ctl))
+        (Some (ctyp t))
   | TyRec loc ltl ->
       mktype loc tl cl (Ptype_record (List.map mktrecord ltl)) None
   | TySum loc ctl ->
@@ -192,7 +210,7 @@ value type_decl tl cl =
   | t ->
       let m =
         match t with
-        [ TyQuo _ s -> if List.mem s tl then Some (ctyp t) else None
+        [ TyQuo _ s -> if List.mem_assoc s tl then Some (ctyp t) else None
         | _ -> Some (ctyp t) ]
       in
       mktype (loc_of_ctyp t) tl cl Ptype_abstract m ]
@@ -254,12 +272,13 @@ value rec module_expr_long_id =
 
 value mkwithc =
   fun
-  [ WcTyp loc id tl ct ->
+  [ WcTyp loc id tpl ct ->
+      let (params, variance) = List.split tpl in
       (long_id_of_string_list loc id,
        Pwith_type
-         {ptype_params = List.map shd tl; ptype_cstrs = [];
+         {ptype_params = params; ptype_cstrs = [];
           ptype_kind = Ptype_abstract; ptype_manifest = Some (ctyp ct);
-          ptype_loc = mkloc loc})
+          ptype_loc = mkloc loc; ptype_variance = variance})
   | WcMod loc id m ->
       (long_id_of_string_list loc id, Pwith_module (module_type_long_id m)) ]
 ;
@@ -343,26 +362,31 @@ value rec patt =
           error (loc_of_patt f)
             "this is not a constructor, it cannot be applied in a pattern" ]
   | PaArr loc pl -> mkpat loc (Ppat_array (List.map patt pl))
-  | PaChr loc s -> mkpat loc (Ppat_constant (Const_char s))
+  | PaChr loc s ->
+      mkpat loc (Ppat_constant (Const_char (char_of_char_token loc s)))
   | PaInt loc s -> mkpat loc (Ppat_constant (Const_int (int_of_string s)))
+  | PaFlo loc s -> mkpat loc (Ppat_constant (Const_float s))
   | PaLab loc _ _ -> error loc "labeled pattern not allowed here"
   | PaLid loc s -> mkpat loc (Ppat_var (shd s))
   | PaOlb loc _ _ _ -> error loc "labeled pattern not allowed here"
   | PaOrp loc p1 p2 -> mkpat loc (Ppat_or (patt p1) (patt p2))
   | PaRng loc p1 p2 ->
       match (p1, p2) with
-      [ (PaChr _ c1, PaChr _ c2) -> mkrangepat loc c1 c2
+      [ (PaChr loc1 c1, PaChr loc2 c2) ->
+          let c1 = char_of_char_token loc1 c1 in
+          let c2 = char_of_char_token loc2 c2 in
+          mkrangepat loc c1 c2
       | _ -> error loc "range pattern allowed only for characters" ]
   | PaRec loc lpl -> mkpat loc (Ppat_record (List.map mklabpat lpl))
-  | PaStr loc s -> mkpat loc (Ppat_constant (Const_string (shd s)))
+  | PaStr loc s ->
+      mkpat loc (Ppat_constant (Const_string (string_of_string_token loc s)))
   | PaTup loc pl -> mkpat loc (Ppat_tuple (List.map patt pl))
   | PaTyc loc p t -> mkpat loc (Ppat_constraint (patt p) (ctyp t))
   | PaUid loc s -> mkpat loc (Ppat_construct (lident (conv_con s)) None True)
   | PaVrn loc s -> mkpat loc (Ppat_variant s None)
   | PaXnd loc c _ ->
       error loc ("pattern \"" ^ c ^ "\" (extension) not allowed here") ]
-and mklabpat (lab, p) = (patt_label_long_id lab, patt p)
-;
+and mklabpat (lab, p) = (patt_label_long_id lab, patt p);
 
 value rec expr_fa al =
   fun
@@ -380,16 +404,20 @@ value rec sep_expr_acc l =
   | e -> [(loc_of_expr e, [], e) :: l] ]
 ;
 
+(*
 value expr_label_long_id e =
   match sep_expr_acc [] e with
   [ [(_, ml, ExLid _ s)] -> mkli (conv_lab s) ml
   | _ -> error (loc_of_expr e) "invalid label" ]
 ;
+*)
 
 value class_info class_expr ci =
+  let (params, variance) = List.split (snd ci.ciPrm) in
   {pci_virt = if ci.ciVir then Virtual else Concrete;
-   pci_params = (snd ci.ciPrm, mkloc (fst ci.ciPrm)); pci_name = ci.ciNam;
-   pci_expr = class_expr ci.ciExp; pci_loc = mkloc ci.ciLoc}
+   pci_params = (params, mkloc (fst ci.ciPrm)); pci_name = ci.ciNam;
+   pci_expr = class_expr ci.ciExp; pci_loc = mkloc ci.ciLoc;
+   pci_variance = variance}
 ;
 
 value rec expr =
@@ -440,8 +468,7 @@ value rec expr =
             | _ -> mkexp loc (Pexp_tuple al) ]
           in
           mkexp loc (Pexp_variant s (Some a))
-      | _ ->
-          mkexp loc (Pexp_apply (expr f) al) ]
+      | _ -> mkexp loc (Pexp_apply (expr f) al) ]
   | ExAre loc e1 e2 ->
       mkexp loc
         (Pexp_apply (mkexp loc (Pexp_ident (array_function "Array" "get")))
@@ -465,7 +492,8 @@ value rec expr =
         | _ -> error loc "bad left part of assignment" ]
       in
       mkexp loc e
-  | ExChr loc s -> mkexp loc (Pexp_constant (Const_char s))
+  | ExChr loc s ->
+      mkexp loc (Pexp_constant (Const_char (char_of_char_token loc s)))
   | ExCoe loc e t ->
       let (e, tc) =
         match e with
@@ -475,16 +503,12 @@ value rec expr =
       mkexp loc (Pexp_constraint (expr e) tc (Some (ctyp t)))
   | ExFlo loc s -> mkexp loc (Pexp_constant (Const_float s))
   | ExFor loc i e1 e2 df el ->
-      let e3 =
-        match List.rev el with
-        [ [] -> ExUid loc "()"
-        | [e :: el] -> ExSeq loc (List.rev el) e ]
-      in
+      let e3 = ExSeq loc el in
       let df = if df then Upto else Downto in
       mkexp loc (Pexp_for (shd i) (expr e1) (expr e2) df (expr e3))
-  | ExFun loc [(PaLab _ lab p, w, e)]  ->
+  | ExFun loc [(PaLab _ lab p, w, e)] ->
       mkexp loc (Pexp_function lab None [(patt p, when_expr e w)])
-  | ExFun loc [(PaOlb _ lab p eo, w, e)]  ->
+  | ExFun loc [(PaOlb _ lab p eo, w, e)] ->
       mkexp loc
         (Pexp_function ("?" ^ lab) (option expr eo) [(patt p, when_expr e w)])
   | ExFun loc pel -> mkexp loc (Pexp_function "" None (List.map mkpwe pel))
@@ -507,26 +531,30 @@ value rec expr =
         | None -> None ]
       in
       mkexp loc (Pexp_record (List.map mklabexp lel) eo)
-  | ExSeq loc el e ->
-      List.fold_right (fun e1 e2 -> mkexp loc (Pexp_sequence (expr e1) e2)) el
-        (expr e)
+  | ExSeq loc el ->
+      let el = List.rev el in
+      let rec loop =
+        fun
+        [ [] -> expr (ExUid loc "()")
+        | [e] -> expr e
+        | [e :: el] ->
+            mkexp (loc_of_expr e) (Pexp_sequence (loop el) (expr e)) ]
+      in
+      loop el
   | ExSnd loc e s -> mkexp loc (Pexp_send (expr e) s)
   | ExSte loc e1 e2 ->
       mkexp loc
         (Pexp_apply (mkexp loc (Pexp_ident (array_function "String" "get")))
            [("", expr e1); ("", expr e2)])
-  | ExStr loc s -> mkexp loc (Pexp_constant (Const_string (shd s)))
+  | ExStr loc s ->
+      mkexp loc (Pexp_constant (Const_string (string_of_string_token loc s)))
   | ExTry loc e pel -> mkexp loc (Pexp_try (expr e) (List.map mkpwe pel))
   | ExTup loc el -> mkexp loc (Pexp_tuple (List.map expr el))
   | ExTyc loc e t -> mkexp loc (Pexp_constraint (expr e) (Some (ctyp t)) None)
   | ExUid loc s -> mkexp loc (Pexp_construct (lident (conv_con s)) None True)
   | ExVrn loc s -> mkexp loc (Pexp_variant s None)
   | ExWhi loc e1 el ->
-      let e2 =
-        match List.rev el with
-        [ [] -> ExUid loc "()"
-        | [e :: el] -> ExSeq loc (List.rev el) e ]
-      in
+      let e2 = ExSeq loc el in
       mkexp loc (Pexp_while (expr e1) (expr e2))
   | ExXnd loc c _ ->
       error loc ("expression \"" ^ c ^ "\" (extension) not allowed here") ]
@@ -541,7 +569,7 @@ and when_expr e =
   fun
   [ Some w -> mkexp (loc_of_expr e) (Pexp_when (expr w) (expr e))
   | None -> expr e ]
-and mklabexp (lab, e) = (expr_label_long_id lab, expr e)
+and mklabexp (lab, e) = (patt_label_long_id lab, expr e)
 and mkideexp (ide, e) = (shd ide, expr e)
 and mktype_decl (c, tl, td, cl) =
   let cl =
@@ -552,7 +580,6 @@ and mktype_decl (c, tl, td, cl) =
       cl
   in
   (shd c, type_decl tl cl td)
-
 and module_type =
   fun
   [ MtAcc loc _ _ as f -> mkmty loc (Pmty_ident (module_type_long_id f))
@@ -570,8 +597,10 @@ and sig_item s l =
   [ SgCls loc cd ->
       [mksig loc (Psig_class (List.map (class_info class_type) cd)) :: l]
   | SgClt loc ctd ->
-      [mksig loc (Psig_class_type (List.map (class_info class_type) ctd)) :: l]
+      [mksig loc (Psig_class_type (List.map (class_info class_type) ctd)) ::
+       l]
   | SgDcl loc sl -> List.fold_right sig_item sl l
+  | SgDir loc _ _ -> l
   | SgExc loc n tl ->
       [mksig loc (Psig_exception (shd n) (List.map ctyp tl)) :: l]
   | SgExt loc n t p ->
@@ -586,7 +615,6 @@ and sig_item s l =
       [mksig loc (Psig_open (long_id_of_string_list loc id)) :: l]
   | SgTyp loc tdl -> [mksig loc (Psig_type (List.map mktype_decl tdl)) :: l]
   | SgVal loc n t -> [mksig loc (Psig_value n (mkvalue_desc t [])) :: l] ]
-
 and module_expr =
   fun
   [ MeAcc loc _ _ as f -> mkmod loc (Pmod_ident (module_expr_long_id f))
@@ -604,13 +632,16 @@ and str_item s l =
   [ StCls loc cd ->
       [mkstr loc (Pstr_class (List.map (class_info class_expr) cd)) :: l]
   | StClt loc ctd ->
-      [mkstr loc (Pstr_class_type (List.map (class_info class_type) ctd)) :: l]
+      [mkstr loc (Pstr_class_type (List.map (class_info class_type) ctd)) ::
+       l]
   | StDcl loc sl -> List.fold_right str_item sl l
+  | StDir loc _ _ -> l
   | StExc loc n tl ->
       [mkstr loc (Pstr_exception (shd n) (List.map ctyp tl)) :: l]
   | StExp loc e -> [mkstr loc (Pstr_eval (expr e)) :: l]
   | StExt loc n t p ->
       [mkstr loc (Pstr_primitive (shd n) (mkvalue_desc t p)) :: l]
+  | StInc loc me -> [mkstr loc (Pstr_include (module_expr me)) :: l]
   | StMod loc n me -> [mkstr loc (Pstr_module (shd n) (module_expr me)) :: l]
   | StMty loc n mt -> [mkstr loc (Pstr_modtype (shd n) (module_type mt)) :: l]
   | StOpn loc id ->
@@ -618,7 +649,6 @@ and str_item s l =
   | StTyp loc tdl -> [mkstr loc (Pstr_type (List.map mktype_decl tdl)) :: l]
   | StVal loc rf pel ->
       [mkstr loc (Pstr_value (mkrf rf) (List.map mkpe pel)) :: l] ]
-
 and class_type =
   fun
   [ CtCon loc id tl ->
@@ -627,10 +657,9 @@ and class_type =
   | CtFun loc (TyLab _ lab t) ct ->
       mkcty loc (Pcty_fun lab (ctyp t) (class_type ct))
   | CtFun loc (TyOlb loc1 lab t) ct ->
-      let t = TyApp loc1 (TyLid loc1 "option") t in    
+      let t = TyApp loc1 (TyLid loc1 "option") t in
       mkcty loc (Pcty_fun ("?" ^ lab) (ctyp t) (class_type ct))
-  | CtFun loc t ct ->
-      mkcty loc (Pcty_fun "" (ctyp t) (class_type ct))
+  | CtFun loc t ct -> mkcty loc (Pcty_fun "" (ctyp t) (class_type ct))
   | CtSig loc t_o ctfl ->
       let t =
         match t_o with
@@ -644,23 +673,25 @@ and class_sig_item =
   fun
   [ CgCtr loc t1 t2 -> Pctf_cstr (ctyp t1, ctyp t2, mkloc loc)
   | CgInh loc ct -> Pctf_inher (class_type ct)
-  | CgMth loc s pf t ->
-      Pctf_meth (s, mkprivate pf, ctyp t, mkloc loc)
-  | CgVal loc s b t ->
-      Pctf_val (s, mkmutable b, Some (ctyp t), mkloc loc)
-  | CgVir loc s b t ->
-      Pctf_virt (s, mkprivate b, ctyp t, mkloc loc) ]
+  | CgMth loc s pf t -> Pctf_meth (s, mkprivate pf, ctyp t, mkloc loc)
+  | CgVal loc s b t -> Pctf_val (s, mkmutable b, Some (ctyp t), mkloc loc)
+  | CgVir loc s b t -> Pctf_virt (s, mkprivate b, ctyp t, mkloc loc) ]
 and class_expr =
   fun
   [ CeApp loc ce el ->
       let el = List.map label_expr el in
       mkpcl loc (Pcl_apply (class_expr ce) el)
   | CeCon loc id tl ->
-      mkpcl loc (Pcl_constr (long_id_of_string_list loc id) (List.map ctyp tl))
+      mkpcl loc
+        (Pcl_constr (long_id_of_string_list loc id) (List.map ctyp tl))
   | CeFun loc (PaLab _ lab p) ce ->
       mkpcl loc (Pcl_fun lab None (patt p) (class_expr ce))
   | CeFun loc (PaOlb _ lab p eo) ce ->
-      let eo = match eo with [ Some e -> Some (expr e) | None -> None ] in
+      let eo =
+        match eo with
+        [ Some e -> Some (expr e)
+        | None -> None ]
+      in
       mkpcl loc (Pcl_fun ("?" ^ lab) eo (patt p) (class_expr ce))
   | CeFun loc p ce -> mkpcl loc (Pcl_fun "" None (patt p) (class_expr ce))
   | CeLet loc rf pel ce ->
