@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: quotation.ml,v 2.0 1998/12/01 08:19:41 ddr Exp $ *)
+(* $Id: quotation.ml,v 3.1 2001/05/21 08:36:20 ddr Exp $ *)
 
 type expander =
   [ ExStr of bool -> string -> string
@@ -28,10 +28,6 @@ value expander_name name =
   | name -> name ]
 ;
 
-value find name =
-  List.assoc (expander_name name) expanders_table.val
-;
+value find name = List.assoc (expander_name name) expanders_table.val;
 
-value add name f =
-  expanders_table.val := [(name, f) :: expanders_table.val]
-;
+value add name f = expanders_table.val := [(name, f) :: expanders_table.val];
