@@ -1,5 +1,5 @@
 (* camlp4r pa_extend.cmo q_MLast.cmo *)
-(* $Id: pa_format.ml,v 2.0 1998/12/01 08:20:42 ddr Exp $ *)
+(* $Id: pa_format.ml,v 3.0 2001/04/19 07:51:28 ddr Exp $ *)
 
 open Pcaml;
 
@@ -9,14 +9,16 @@ EXTEND
     [ [ n = box_type; d = SELF; "begin";
         el = LIST0 [ e = box_expr; ";" -> e ]; "end" ->
           let el = [<:expr< Format.$lid:"open_" ^ n$ $d$ >> :: el] in
-          <:expr< do $list:el$ return Format.close_box () >>
+          let el = el @ [<:expr< Format.close_box () >>] in
+          <:expr< do { $list:el$ } >>
       | "hbox"; "begin"; el = LIST0 [ e = box_expr; ";" -> e ]; "end" ->
           let el = [<:expr< Format.open_hbox () >> :: el] in
-          <:expr< do $list:el$ return Format.close_box () >>
+          let el = el @ [<:expr< Format.close_box () >>] in
+          <:expr< do { $list:el$ } >>
       | "nobox"; "begin"; el = LIST0 [ e = box_expr; ";" -> e ]; "end" ->
           match el with
           [ [e] -> e
-          | _ -> <:expr< do $list:el$ return () >> ] ] ]
+          | _ -> <:expr< do { $list:el$ } >> ] ] ]
   ;
   box_type:
     [ [ n = "hovbox" -> n
