@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: pa_oop.ml,v 2.2 1999/07/09 09:29:37 ddr Exp $ *)
+(* $Id: pa_oop.ml,v 3.0 2001/04/19 07:51:31 ddr Exp $ *)
 
 open Pcaml;
 
@@ -33,7 +33,7 @@ value stream_pattern_component skont =
   fun
   [ SpTrm loc p wo ->
       (<:expr< $peek_fun loc$ $lid:strm_n$ >>, p, wo,
-       <:expr< do $junk_fun loc$ $lid:strm_n$; return $skont$ >>)
+       <:expr< do { $junk_fun loc$ $lid:strm_n$; $skont$ } >>)
   | SpNtr loc p e ->
       (<:expr< try Some ($e$ $lid:strm_n$) with
                [ Stream.Failure -> None ] >>,
