@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: pa_op.ml,v 2.2 2000/04/12 22:39:43 ddr Exp $ *)
+(* $Id: pa_op.ml,v 3.1 2001/07/14 04:06:34 ddr Exp $ *)
 
 open Pcaml;
 
@@ -109,7 +109,7 @@ value stream_pattern_component skont ckont =
   [ SpTrm loc p wo ->
       <:expr< match $peek_fun loc$ $lid:strm_n$ with
               [ Some $p$ $when:wo$ ->
-                  do $junk_fun loc$ $lid:strm_n$; return $skont$
+                  do { $junk_fun loc$ $lid:strm_n$; $skont$ }
               | _ -> $ckont$ ] >>
   | SpNtr loc p e ->
       let e =
@@ -180,7 +180,7 @@ value stream_patterns_term loc ekont tspel =
              <:expr< raise (Stream.Error $str$) >>
            in
            let skont = stream_pattern loc epo e ekont spcl in
-           <:expr< do $junk_fun loc$ $lid:strm_n$; return $skont$ >>
+           <:expr< do { $junk_fun loc$ $lid:strm_n$; $skont$ } >>
          in
          (p, w, e))
       tspel
@@ -226,7 +226,7 @@ value cparser_match loc me bpo pc =
     [ Some bp -> <:expr< let $bp$ = Stream.count $lid:strm_n$ in $pc$ >>
     | None -> pc ]
   in
-  <:expr< let $lid:strm_n$ = $me$ in $e$ >>
+  <:expr< let ($lid:strm_n$ : Stream.t _) = $me$ in $e$ >>
 ;
 
 (* streams *)
