@@ -10,14 +10,37 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: pa_ru.ml,v 2.0 1998/12/01 08:20:46 ddr Exp $ *)
+(* $Id: pa_ru.ml,v 3.0 2001/04/19 07:51:31 ddr Exp $ *)
 
 open Pcaml;
 
+value o2b =
+  fun
+  [ Some _ -> True
+  | None -> False ]
+;
+
 EXTEND
+  GLOBAL: expr;
   expr: LEVEL "top"
-    [ [ "do"; seq = LIST0 [ e = expr; ";" -> <:expr< ($e$ : unit) >> ];
-        "return"; e = expr ->
-          <:expr< do $list:seq$ return $e$ >> ] ]
+    [ [ "do"; "{"; seq = sequence; "}" ->
+          match seq with
+          [ [e] -> e
+          | _ -> <:expr< do { $list:seq$ } >> ] ] ]
+  ;
+  sequence:
+    [ [ "let"; o = OPT "rec"; l = LIST1 let_binding SEP "and"; "in";
+        el = SELF ->
+          let e =
+            match el with
+            [ [e] -> e
+            | _ -> <:expr< do { $list:el$ } >> ]
+          in
+          [<:expr< let $rec:o2b o$ $list:l$ in $e$ >>]
+      | e = expr; ";"; el = SELF ->
+          let e = let loc = MLast.loc_of_expr e in <:expr< ($e$ : unit) >> in
+          [e :: el]
+      | e = expr; ";" -> [e]
+      | e = expr -> [e] ] ]
   ;
 END;
