@@ -10,54 +10,49 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: pa_sml.ml,v 2.1 1999/06/24 12:54:19 ddr Exp $ *)
+(* $Id: pa_sml.ml,v 3.6 2001/06/10 00:19:26 ddr Exp $ *)
 
 open Stdpp;
 open Pcaml;
 
 Pcaml.no_constructors_arity.val := False;
 
-do Grammar.Unsafe.reinit_gram gram (Plexer.make ());
-   Grammar.Unsafe.clear_entry interf;
-   Grammar.Unsafe.clear_entry implem;
-   Grammar.Unsafe.clear_entry top_phrase;
-   Grammar.Unsafe.clear_entry use_file;
-   Grammar.Unsafe.clear_entry module_type;
-   Grammar.Unsafe.clear_entry module_expr;
-   Grammar.Unsafe.clear_entry sig_item;
-   Grammar.Unsafe.clear_entry str_item;
-   Grammar.Unsafe.clear_entry expr;
-   Grammar.Unsafe.clear_entry patt;
-   Grammar.Unsafe.clear_entry ctyp;
-   Grammar.Unsafe.clear_entry let_binding;
-return ();
+do {
+  Grammar.Unsafe.reinit_gram gram (Plexer.make ());
+  Grammar.Unsafe.clear_entry interf;
+  Grammar.Unsafe.clear_entry implem;
+  Grammar.Unsafe.clear_entry top_phrase;
+  Grammar.Unsafe.clear_entry use_file;
+  Grammar.Unsafe.clear_entry module_type;
+  Grammar.Unsafe.clear_entry module_expr;
+  Grammar.Unsafe.clear_entry sig_item;
+  Grammar.Unsafe.clear_entry str_item;
+  Grammar.Unsafe.clear_entry expr;
+  Grammar.Unsafe.clear_entry patt;
+  Grammar.Unsafe.clear_entry ctyp;
+  Grammar.Unsafe.clear_entry let_binding;
+};
 
 value not_impl loc s =
   raise_with_loc loc (Stream.Error ("not implemented feature [" ^ s ^ "]"))
 ;
 
-value rec get_seq =
+value get_seq =
   fun
-  [ <:expr< do $list:el$ return $e$ >> ->
-      let (el2, e) = get_seq e in
-      let el =
-        List.fold_right
-          (fun e el2 -> let (el, e) = get_seq e in el @ [e :: el2]) el el2
-      in
-      (el, e)
-  | e -> ([], e) ]
+  [ <:expr< do { $list:el$ } >> -> el
+  | e -> [e] ]
 ;
 
 value choose_tvar tpl =
   let rec find_alpha v =
     let s = String.make 1 v in
-    if List.mem s tpl then
+    if List.mem_assoc s tpl then
       if v = 'z' then None else find_alpha (Char.chr (Char.code v + 1))
     else Some (String.make 1 v)
   in
   let rec make_n n =
     let v = "a" ^ string_of_int n in
-    if List.mem v tpl then make_n (succ n) else v
+    if List.mem_assoc v tpl then make_n (succ n) else v
   in
   match find_alpha 'a' with
   [ Some x -> x
@@ -117,10 +112,12 @@ value extract_label_types loc tn tal cdol =
 ;
 
 value special x =
-  do assert (String.length x > 0); return
-  match x.[0] with
-  [ '+' | '<' -> True
-  | _ -> False ]
+  do {
+    assert (String.length x > 0);
+    match x.[0] with
+    [ '+' | '<' -> True
+    | _ -> False ]
+  }
 ;
 
 value idd =
@@ -149,24 +146,16 @@ EXTEND
       | EOI -> None ] ]
   ;
   use_file:
-    [ [ l = LIST0 phrase; EOI -> l ] ]
+    [ [ l = LIST0 phrase; EOI -> (l, False) ] ]
   ;
   phrase:
-    [ [ x = str_item -> MLast.PhStr loc x
-      | x = expr -> MLast.PhStr loc <:str_item< $exp:x$ >>
-      | "#"; n = LIDENT; dp = dir_param -> MLast.PhDir loc n dp ] ]
+    [ [ x = str_item -> x
+      | x = expr -> <:str_item< $exp:x$ >>
+      | "#"; n = LIDENT; dp = dir_param -> MLast.StDir loc n dp ] ]
   ;
   dir_param:
-    [ [ -> MLast.DpNon
-      | s = STRING -> MLast.DpStr s
-      | i = INT -> MLast.DpInt i
-      | sl = mod_ident -> MLast.DpIde sl ] ]
-  ;
-  mod_ident:
-    [ RIGHTA
-      [ i = UIDENT -> [i]
-      | i = LIDENT -> [i]
-      | m = UIDENT; "."; i = mod_ident -> [m :: i] ] ]
+    [ [ -> None
+      | e = expr -> Some e ] ]
   ;
 
   sdecs: [ [ -> not_impl loc "sdecs" ] ];
@@ -253,13 +242,12 @@ EXTEND
     [ [ x1 = patt; "=>"; x2 = expr -> (x1, None, x2) ] ]
   ;
   elabel:
-    [ [ x1 = selector; "="; x2 = expr -> (<:expr< $lid:x1$ >>, x2) ] ]
+    [ [ x1 = selector; "="; x2 = expr -> (<:patt< $lid:x1$ >>, x2) ] ]
   ;
   exp_ps:
     [ [ x1 = expr -> x1
       | x1 = expr; ";"; x2 = expr ->
-          let (el, e) = get_seq <:expr< do $x1$; return $x2$ >> in
-          <:expr< do $list:el$ return $e$ >> ] ]
+          <:expr< do { $list:[x1 :: get_seq x2]$ } >> ] ]
   ;
   expr:
     [ [ "if"; x1 = expr; "then"; x2 = expr; "else"; x3 = expr ->
@@ -409,7 +397,7 @@ EXTEND
     [ [ x1 = tyvars; x2 = idd; "="; x3 = ctyp -> (uncap x2, x1, x3, []) ] ]
   ;
   tyvars:
-    [ [ "'"; x1 = LIDENT -> [x1]
+    [ [ "'"; x1 = LIDENT -> [(x1, (False, False))]
       | "("; x1 = tyvar_pc; ")" -> x1
       | -> [] ] ]
   ;
@@ -573,16 +561,18 @@ EXTEND
       | x1 = fixity; x2 = idd ->
           match x1 with
           [ ("infixr", Some n) ->
-              do List.iter
-                   (fun s ->
-                      EXTEND
-                        expr: LEVEL $n$
-                          [ [ x1 = expr; $s$; x2 = expr ->
-                                <:expr< $lid:s$ ($x1$, $x2$) >> ] ]
-                        ;
-                      END)
-                   [x2];
-              return str_declare loc []
+              do {
+                List.iter
+                  (fun s ->
+                     EXTEND
+                       expr: LEVEL $n$
+                         [ [ x1 = expr; $s$; x2 = expr ->
+                               <:expr< $lid:s$ ($x1$, $x2$) >> ] ]
+                       ;
+                     END)
+                  [x2];
+                  str_declare loc []
+              }
           | _ -> not_impl loc "ldec 14" ]
       | "overload"; x1 = ident; ":"; x2 = ctyp; "as"; x3 = exp_pa ->
           not_impl loc "ldec 15" ] ]
@@ -625,7 +615,7 @@ EXTEND
           not_impl loc "fctb 2" ] ]
   ;
   interdec:
-    [ [ x = LIST1 [ s = str_item -> (s, loc) ] -> x
+    [ [ x = LIST1 [ s = str_item -> (s, loc) ] -> (x, False)
       | x = expr -> not_impl loc "interdec 2" ] ]
   ;
 END;
