@@ -1,18 +1,22 @@
-(* camlp4r q_MLast.cmo *)
+(* camlp4r q_MLast.cmo ./pa_extfun.cmo *)
 (***********************************************************************)
 (*                                                                     *)
 (*                             Camlp4                                  *)
 (*                                                                     *)
 (*        Daniel de Rauglaudre, projet Cristal, INRIA Rocquencourt     *)
 (*                                                                     *)
-(*  Copyright 1998 Institut National de Recherche en Informatique et   *)
+(*  Copyright 2001 Institut National de Recherche en Informatique et   *)
 (*  Automatique.  Distributed only by permission.                      *)
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: pr_extend.ml,v 2.0 1998/12/01 08:20:48 ddr Exp $ *)
+(* $Id: pr_extend.ml,v 3.7 2001/06/25 09:02:45 ddr Exp $ *)
 
-open Pretty;
+open Pcaml;
+open Spretty;
+
+value expr e dg k = pr_expr.pr_fun "top" e dg k;
+value patt e dg k = pr_patt.pr_fun "top" e dg k;
 
 (* Utilities *)
 
@@ -91,7 +95,14 @@ value unassoc =
 
 value rec unaction =
   fun
-  [ <:expr< fun (loc : (int * int)) -> ($a$ : $_$) >> -> ([], a)
+  [ <:expr< fun ($lid:locp$ : (int * int)) -> ($a$ : $_$) >>
+    when locp = Stdpp.loc_name.val ->
+      let ao =
+        match a with
+        [ <:expr< () >> -> None
+        | _ -> Some a ]
+      in
+      ([], ao)
   | <:expr< fun ($p$ : $_$) -> $e$ >> ->
       let (pl, a) = unaction e in ([p :: pl], a)
   | <:expr< fun _ -> $e$ >> ->
@@ -117,7 +128,7 @@ type symbol =
   | Sself
   | Snext
   | Stoken of Token.pattern
-  | Srules of list (list (MLast.patt * symbol) * MLast.expr) ]
+  | Srules of list (list (option MLast.patt * symbol) * option MLast.expr) ]
 ;
 
 value rec unsymbol =
@@ -147,12 +158,21 @@ and unpsymbol_list pl e =
   match (pl, e) with
   [ ([], <:expr< [] >>) -> []
   | ([p :: pl], <:expr< [$e$ :: $el$] >>) ->
-      [(p, unsymbol e) :: unpsymbol_list pl el]
+      let op =
+        match p with
+        [ <:patt< _ >> -> None
+        | _ -> Some p ]
+      in
+      [(op, unsymbol e) :: unpsymbol_list pl el]
   | _ -> raise Not_found ]
 and unrule =
   fun
   [ <:expr< ($e1$, Gramext.action $e2$) >> ->
-      let (pl, a) = unaction e2 in
+      let (pl, a) =
+        match unaction e2 with
+        [ ([], None) -> let loc = (0, 0) in ([], Some <:expr< () >>)
+        | x -> x ]
+      in
       let sl = unpsymbol_list (List.rev pl) e1 in
       (sl, a)
   | _ -> raise Not_found ]
@@ -223,7 +243,7 @@ value unextend_body e =
 (* Printing *)
 
 value ident s k = HVbox [: `S LR s; k :];
-value string s k = HVbox [: `S LR ("\"" ^ String.escaped s ^ "\""); k :];
+value string s k = HVbox [: `S LR ("\"" ^ s ^ "\""); k :];
 
 value position =
   fun
@@ -245,47 +265,89 @@ value token (con, prm) k =
   else HVbox [: `S LR con; `string prm k :]
 ;
 
-value rec symbol (expr, patt) s k =
+value simplify_rules rl =
+  try
+    List.map
+      (fun
+       [ ([(Some <:patt< $lid:x$ >>, s)], Some <:expr< $lid:y$ >>) ->
+           if x = y then ([(None, s)], None) else raise Exit
+       | ([], _) as r -> r
+       | _ -> raise Exit ])
+      rl
+  with
+  [ Exit -> rl ]
+;
+
+value rec symbol s k =
   match s with
   [ Snterm e -> expr e "" k
   | Snterml e s -> HVbox [: `expr e "" [: :]; `S LR "LEVEL"; `string s k :]
-  | Slist0 s -> HVbox [: `S LR "LIST0"; `symbol (expr, patt) s k :]
+  | Slist0 s -> HVbox [: `S LR "LIST0"; `symbol s k :]
   | Slist0sep s sep ->
       HVbox
-        [: `S LR "LIST0"; `symbol (expr, patt) s [: :]; `S LR "SEP";
-           `symbol (expr, patt) sep k :]
-  | Slist1 s -> HVbox [: `S LR "LIST1"; `symbol (expr, patt) s k :]
+        [: `S LR "LIST0"; `symbol s [: :]; `S LR "SEP";
+           `symbol sep k :]
+  | Slist1 s -> HVbox [: `S LR "LIST1"; `symbol s k :]
   | Slist1sep s sep ->
       HVbox
-        [: `S LR "LIST1"; `symbol (expr, patt) s [: :]; `S LR "SEP";
-           `symbol (expr, patt) sep k :]
-  | Sopt s -> HVbox [: `S LR "OPT"; `symbol (expr, patt) s k :]
+        [: `S LR "LIST1"; `symbol  s [: :]; `S LR "SEP";
+           `symbol  sep k :]
+  | Sopt s -> HVbox [: `S LR "OPT"; `symbol  s k :]
   | Sself -> HVbox [: `S LR "SELF"; k :]
   | Snext -> HVbox [: `S LR "NEXT"; k :]
   | Stoken tok -> token tok k
-  | Srules rl -> HVbox [: `HVbox [: :]; rule_list (expr, patt) rl k :] ]
-and psymbol (expr, patt) (p, s) k =
+(**)
+  | Srules
+      [([(Some <:patt< a >>, Snterm <:expr< anti_list >>)], Some <:expr< a >>);
+       ([(Some <:patt< l >>,
+          ((Slist0 _ | Slist1 _ | Slist0sep _ _ | Slist1sep _ _) as s))],
+          Some <:expr< list l >>)]
+    ->
+      match s with
+      [ Slist0 s -> HVbox [: `S LR "SLIST0"; `simple_symbol s k :]
+      | Slist1 s -> HVbox [: `S LR "SLIST1"; `simple_symbol s k :]
+      | Slist0sep s sep ->
+          HVbox
+            [: `S LR "SLIST0"; `simple_symbol s [: :]; `S LR "SEP";
+               `symbol sep k :]
+      | Slist1sep s sep ->
+          HVbox
+            [: `S LR "SLIST1"; `simple_symbol s [: :]; `S LR "SEP";
+               `simple_symbol sep k :]
+      | _ -> assert False ]
+(**)
+  | Srules rl ->
+      let rl = simplify_rules rl in
+      HVbox [: `HVbox [: :]; rule_list  rl k :] ]
+and simple_symbol s k =
+  match s with
+  [ Snterml _ _ -> HVbox [: `S LO "("; `symbol s [: `S RO ")"; k :] :]
+  | s -> symbol s k ]
+and psymbol (p, s) k =
   match p with
-  [ <:patt< _ >> -> symbol (expr, patt) s k
-  | _ -> HVbox [: `patt p "" [: `S LR "=" :]; `symbol (expr, patt) s k :] ]
-and psymbol_list expr_patt sl k =
-  listws (psymbol expr_patt) (S RO ";") sl k
-and rule (expr, patt) b (sl, a) dg k =
-  HVbox
-    [: b;
-       `HOVbox
-          [: `HOVbox
-               [: `HVbox [: :];
-                  psymbol_list (expr, patt) sl [: `S LR "->" :] :];
-             `action expr a dg k :] :]
-and rule_list expr_patt ll k =
-  listwbws (rule expr_patt) [: `S LR "[" :] (S LR "|") ll ""
+  [ None -> symbol s k
+  | Some p -> HVbox [: `patt p "" [: `S LR "=" :]; `symbol  s k :] ]
+and psymbol_list sl k =
+  listws psymbol (S RO ";") sl k
+and rule  b (sl, a) dg k =
+  match a with
+  [ None -> HVbox [: b; `HOVbox [: psymbol_list sl k :] :]
+  | Some a ->
+      HVbox
+        [: b;
+           `HOVbox
+              [: `HOVbox
+                   [: `HVbox [: :];
+                      psymbol_list  sl [: `S LR "->" :] :];
+                 `action expr a dg k :] :] ]
+and rule_list ll k =
+  listwbws rule [: `S LR "[" :] (S LR "|") ll ""
     [: `S LR "]"; k :]
 ;
 
 value label =
   fun
-  [ Some s -> [: `S LR ("\"" ^ String.escaped s ^ "\"") :]
+  [ Some s -> [: `S LR ("\"" ^ s ^ "\"") :]
   | None -> [: :] ]
 ;
 
@@ -297,8 +359,11 @@ value assoc =
   | None -> [: :] ]
 ;
 
-value level expr_patt b (lab, ass, ll) dg k =
-  let s = rule_list expr_patt ll k in
+value level b (lab, ass, rl) dg k =
+  let s =
+    if rl = [] then [: `S LR "[ ]"; k :]
+    else [: `Vbox [: `HVbox [: :]; rule_list rl k :] :]
+  in
   match (lab, ass) with
   [ (None, None) -> HVbox [: b; s :]
   | _ ->
@@ -307,26 +372,26 @@ value level expr_patt b (lab, ass, ll) dg k =
            `HVbox [: `HVbox [: :]; s :] :] ]
 ;
 
-value level_list expr_patt ll k =
+value level_list ll k =
   Vbox
     [: `HVbox [: :];
-       listwbws (level expr_patt) [: `S LR "[" :] (S LR "|") ll ""
+       listwbws level [: `S LR "[" :] (S LR "|") ll ""
          [: `S LR "]"; k :] :]
 ;
 
-value entry (expr, patt) (e, pos, ll) k =
+value entry  (e, pos, ll) k =
   BEbox
     [: `HVbox [: `expr e "" [: `S RO ":" :]; position pos :];
-       `level_list (expr, patt) ll [: :];
+       `level_list  ll [: :];
        `HVbox [: `S RO ";"; k :] :]
 ;
 
-value entry_list expr_patt el k =
-  Vbox [: `HVbox [: :]; list (entry expr_patt) el k :]
+value entry_list el k =
+  Vbox [: `HVbox [: :]; list entry el k :]
 ;
 
-value extend_body expr_patt (globals, e) k =
-  let s = entry_list expr_patt e k in
+value extend_body (globals, e) k =
+  let s = entry_list e k in
   match globals with
   [ [] -> s
   | sl ->
@@ -337,13 +402,13 @@ value extend_body expr_patt (globals, e) k =
            `s :] ]
 ;
 
-value extend expr patt e dg k =
+value extend e dg k =
   match e with
   [ <:expr< Grammar.extend $e$ >> ->
       try
         let ex = unextend_body e in
         BEbox
-          [: `S LR "EXTEND"; `extend_body (expr, patt) ex [: :];
+          [: `S LR "EXTEND"; `extend_body ex [: :];
              `HVbox [: `S LR "END"; k :] :]
       with
       [ Not_found ->
@@ -355,4 +420,16 @@ value extend expr patt e dg k =
   | _ -> expr e "" k ]
 ;
 
-Pretty.add_pr_fun "extend" extend;
+(* Printer extensions *)
+
+let lev = find_pr_level "apply" pr_expr.pr_levels in
+lev.pr_rules :=
+  extfun lev.pr_rules with
+  [ <:expr< Grammar.extend $_$ >> as e ->
+      fun curr next _ k -> [: `next e "" k :] ];
+
+let lev = find_pr_level "simple" pr_expr.pr_levels in
+lev.pr_rules :=
+  extfun lev.pr_rules with
+  [ <:expr< Grammar.extend $_$ >> as e ->
+      fun curr next _ k -> [: `extend e "" k :] ];
