@@ -1,4 +1,4 @@
-(* camlp4r q_MLast.cmo *)
+(* camlp4r q_MLast.cmo ./pa_extfun.cmo *)
 (***********************************************************************)
 (*                                                                     *)
 (*                             Camlp4                                  *)
@@ -10,13 +10,17 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: pr_op.ml,v 2.0 1998/12/01 08:20:51 ddr Exp $ *)
+(* $Id: pr_op.ml,v 3.4 2001/06/22 06:51:18 ddr Exp $ *)
 
-open Pretty;
+open Pcaml;
+open Spretty;
 
 value loc = (0, 0);
 
-value spatt patt p dg k =
+value expr e dg k = pr_expr.pr_fun "top" e dg k;
+value patt e dg k = pr_patt.pr_fun "top" e dg k;
+
+value spatt p dg k =
   match p with
   [ <:patt< $lid:s$ >> ->
       if String.length s >= 2 && s.[1] == ''' then
@@ -27,7 +31,7 @@ value spatt patt p dg k =
 
 (* Streams *)
 
-value stream expr patt e _ k =
+value stream e _ k =
   let rec get =
     fun
     [ <:expr< Stream.iapp $x$ $y$ >> -> [(False, x) :: get y]
@@ -228,21 +232,21 @@ value rewrite_parser =
         let rec iter pel =
           match pel with
           [ [(<:patt< Some $p$ >>, eo,
-              <:expr< do Stream.junk strm__; return $sp_kont$ >>);
+              <:expr< do { Stream.junk strm__; $sp_kont$ } >>);
              (<:patt< _ >>, None, p_kont) :: _] ->
                <:expr<
                  match Stream.peek strm__ with
                  [ Some $p$ $when:eo$ ->
-                     do Stream.junk strm__; return $rewrite False sp_kont$
+                     do { Stream.junk strm__; $rewrite False sp_kont$ }
                  | _ -> $rewrite top p_kont$ ]
                >>
           | [(<:patt< Some $p$ >>, eo,
-              <:expr< do Stream.junk strm__; return $sp_kont$ >>) :: pel] ->
+              <:expr< do { Stream.junk strm__; $sp_kont$ } >>) :: pel] ->
                let p_kont = iter pel in
                <:expr<
                  match Stream.peek strm__ with
                  [ Some $p$ $when:eo$ ->
-                     do Stream.junk strm__; return $rewrite False sp_kont$
+                     do { Stream.junk strm__; $rewrite False sp_kont$ }
                  | _ -> $p_kont$ ]
                >>
           | _ ->
@@ -278,7 +282,7 @@ value rewrite_parser =
           >>
         else
           let v = free_var_in_expr 's' f in
-          <:expr< let s = strm__ in $subst v f$ s >>
+          <:expr< let $lid:v$ = strm__ in $f$ $lid:v$ >>
     | e -> semantic e ]
 ;
 
@@ -295,7 +299,7 @@ value parser_of_expr =
         [([spc :: sp], epo, e) :: parser_cases p_kont]
     | <:expr<
         match Stream.peek strm__ with
-        [ Some $p$ $when:wo$ -> do Stream.junk strm__; return $sp_kont$
+        [ Some $p$ $when:wo$ -> do { Stream.junk strm__; $sp_kont$ }
         | _ -> $p_kont$ ]
       >> ->
         let spc = (SPCterm (p, wo), None) in
@@ -325,7 +329,7 @@ value parser_of_expr =
         ([spc :: sp], epo, e)
     | <:expr<
         match Stream.peek strm__ with
-        [ Some $p$ $when:wo$ -> do Stream.junk strm__; return $sp_kont$
+        [ Some $p$ $when:wo$ -> do { Stream.junk strm__; $sp_kont$ }
         | _ -> raise (Stream.Error $err$) ]
       >> ->
         let err =
@@ -346,7 +350,7 @@ value parser_of_expr =
   parser_cases
 ;
 
-value parser_cases expr patt b spel dg k =
+value parser_cases b spel dg k =
   let rec parser_cases b spel dg k =
     match spel with
     [ [] -> [: `HVbox [: b; k :] :]
@@ -386,7 +390,7 @@ value parser_cases expr patt b spel dg k =
   and stream_patt_comp b spc dg k =
     match spc with
     [ SPCterm (p, w) ->
-        HVbox [: b; `S LO "'"; `spatt patt p "" (when_opt w k) :]
+        HVbox [: b; `S LO "'"; `spatt p "" (when_opt w k) :]
     | SPCnterm p e ->
         HVbox [: b; `HVbox [: `patt p "" [: `S LR "=" :]; `expr e dg k :] :]
     | SPCsterm p -> HVbox [: b; `patt p "" k :] ]
@@ -398,7 +402,7 @@ value parser_cases expr patt b spel dg k =
   parser_cases b spel dg k
 ;
 
-value parser_body expr patt e dg k =
+value parser_body e dg k =
   let (bp, e) =
     match e with
     [ <:expr< let $bp$ = Stream.count strm__ in $e$ >> -> (Some bp, e)
@@ -414,7 +418,7 @@ value parser_body expr patt e dg k =
                  match bp with
                  [ Some p -> [: `patt p "" [: :] :]
                  | _ -> [: :] ] :];
-           parser_cases expr patt [: :] [spe] dg k :]
+           parser_cases [: :] [spe] dg k :]
   | [spe] ->
       HVbox
         [: `HVbox
@@ -422,9 +426,9 @@ value parser_body expr patt e dg k =
                  match bp with
                  [ Some p -> [: `patt p "" [: :] :]
                  | _ -> [: :] ] :];
-           parser_cases expr patt [: :] [spe] dg k :]
+           parser_cases [: :] [spe] dg k :]
   | spel ->
-      HVbox
+      Vbox
         [: `HVbox [: :];
            `HVbox
               [: `S LR "parser";
@@ -432,13 +436,13 @@ value parser_body expr patt e dg k =
                  [ Some p -> [: `patt p "" [: :] :]
                  | _ -> [: :] ] :];
            `BEbox
-              [: `HVbox [: :]; parser_cases expr patt [: :] spel dg k :] :] ]
+              [: `HVbox [: :]; parser_cases [: :] spel dg k :] :] ]
 ;
 
-value pmatch expr patt e dg k =
+value pmatch e dg k =
   let (me, e) =
     match e with
-    [ <:expr< let strm__ = $me$ in $e$ >> -> (me, e)
+    [ <:expr< let (strm__ : Stream.t _) = $me$ in $e$ >> -> (me, e)
     | _ -> failwith "Pr_op.pmatch" ]
   in
   let (bp, e) =
@@ -448,16 +452,62 @@ value pmatch expr patt e dg k =
   in
   let e = rewrite_parser e in
   let spel = parser_of_expr e in
-  HVbox
+  Vbox
     [: `HVbox [: :];
        `HVbox
           [: `S LR "match"; `expr me "" [: `S LR "with" :]; `S LR "parser";
              match bp with
              [ Some p -> [: `patt p "" [: :] :]
              | _ -> [: :] ] :];
-       `BEbox [: `HVbox [: :]; parser_cases expr patt [: :] spel dg k :] :]
+       `BEbox [: `HVbox [: :]; parser_cases [: :] spel dg k :] :]
 ;
 
-Pretty.add_pr_fun "stream" stream;
-Pretty.add_pr_fun "parser" parser_body;
-Pretty.add_pr_fun "parser_m" pmatch;
+(* Printer extensions *)
+
+pr_expr_fun_args.val :=
+  extfun pr_expr_fun_args.val with
+  [ <:expr< fun strm__ -> $_$ >> as ge -> ([], ge)
+  | <:expr< fun [(strm__ : $_$) -> $_$] >> as ge -> ([], ge) ];
+
+let lev = find_pr_level "expr1" pr_expr.pr_levels in
+lev.pr_rules :=
+  extfun lev.pr_rules with
+  [ <:expr< let (strm__ : Stream.t _) = $_$ in $_$ >> as e ->
+      fun curr next dg k ->
+        if not (List.mem dg ["|"; ";"]) then [: `pmatch e dg k :]
+        else [: `S LO "("; `pmatch e "" [: `S RO ")"; k :] :]
+  | <:expr< fun strm__ -> $x$ >> ->
+      fun curr next dg k ->
+        if not (List.mem dg ["|"; ";"]) then [: `parser_body x dg k :]
+        else [: `S LO "("; `parser_body x "" [: `S RO ")"; k :] :]
+  | <:expr< fun [ (strm__ : $_$) -> $x$ ] >> ->
+      fun curr next dg k ->
+        if not (List.mem dg ["|"; ";"]) then [: `parser_body x dg k :]
+        else [: `S LO "("; `parser_body x "" [: `S RO ")"; k :] :] ];
+
+let lev = find_pr_level "apply" pr_expr.pr_levels in
+lev.pr_rules :=
+  extfun lev.pr_rules with
+  [ <:expr< Stream.iapp $_$ $_$ >> | <:expr< Stream.icons $_$ $_$ >> |
+    <:expr< Stream.ising $_$ >> | <:expr< Stream.lapp (fun _ -> $_$) $_$ >> |
+    <:expr< Stream.lcons (fun _ -> $_$) $_$ >> |
+    <:expr< Stream.lsing (fun _ -> $_$) >> | <:expr< Stream.sempty >> |
+    <:expr< Stream.slazy $_$ >> as e ->
+      fun curr next dg k -> [: `next e "" k :] ];
+
+let lev = find_pr_level "dot" pr_expr.pr_levels in
+lev.pr_rules :=
+  extfun lev.pr_rules with
+  [ <:expr< Stream.sempty >> as e ->
+      fun curr next dg k -> [: `next e "" k :] ];
+
+let lev = find_pr_level "simple" pr_expr.pr_levels in
+lev.pr_rules :=
+  extfun lev.pr_rules with
+  [ <:expr< Stream.iapp $_$ $_$ >> | <:expr< Stream.icons $_$ $_$ >> |
+    <:expr< Stream.ising $_$ >> | <:expr< Stream.lapp (fun _ -> $_$) $_$ >> |
+    <:expr< Stream.lcons (fun _ -> $_$) $_$ >> |
+    <:expr< Stream.lsing (fun _ -> $_$) >> | <:expr< Stream.sempty >> |
+    <:expr< Stream.slazy $_$ >> as e ->
+      fun curr next dg k ->
+        [: `stream e "" k :] ];
