@@ -1,4 +1,4 @@
-(* camlp4r q_MLast.cmo *)
+(* camlp4r q_MLast.cmo ./pa_extfun.cmo *)
 (***********************************************************************)
 (*                                                                     *)
 (*                             Camlp4                                  *)
@@ -10,15 +10,19 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: pr_rp.ml,v 2.0 1998/12/01 08:20:55 ddr Exp $ *)
+(* $Id: pr_rp.ml,v 3.5 2001/06/22 06:51:19 ddr Exp $ *)
 
-open Pretty;
+open Pcaml;
+open Spretty;
 
 value loc = (0, 0);
 
+value expr e dg k = pr_expr.pr_fun "top" e dg k;
+value patt e dg k = pr_patt.pr_fun "top" e dg k;
+
 (* Streams *)
 
-value stream expr patt e dg k =
+value stream e dg k =
   let rec get =
     fun
     [ <:expr< Stream.iapp $x$ $y$ >> -> [(False, x) :: get y]
@@ -85,8 +89,7 @@ value rec is_free v =
   | <:expr< $x$ $y$ >> -> is_free v x && is_free v y
   | <:expr< let $lid:s1$ = $e1$ in $e2$ >> ->
       is_free v e1 && (s1 = v || is_free v e2)
-  | <:expr< let _ = $e1$ in $e2$ >> ->
-      is_free v e1 && is_free v e2
+  | <:expr< let _ = $e1$ in $e2$ >> -> is_free v e1 && is_free v e2
   | <:expr< ($list:el$) >> -> List.for_all (is_free v) el
   | _ -> raise NotImpl ]
 ;
@@ -220,21 +223,21 @@ value rewrite_parser =
         let rec iter pel =
           match pel with
           [ [(<:patt< Some $p$ >>, eo,
-              <:expr< do Stream.junk strm__; return $sp_kont$ >>);
+              <:expr< do { Stream.junk strm__; $sp_kont$ } >>);
              (<:patt< _ >>, None, p_kont) :: _] ->
                <:expr<
                  match Stream.peek strm__ with
                  [ Some $p$ $when:eo$ ->
-                     do Stream.junk strm__; return $rewrite False sp_kont$
+                     do { Stream.junk strm__; $rewrite False sp_kont$ }
                  | _ -> $rewrite top p_kont$ ]
                >>
           | [(<:patt< Some $p$ >>, eo,
-              <:expr< do Stream.junk strm__; return $sp_kont$ >>) :: pel] ->
+              <:expr< do { Stream.junk strm__; $sp_kont$ } >>) :: pel] ->
                let p_kont = iter pel in
                <:expr<
                  match Stream.peek strm__ with
                  [ Some $p$ $when:eo$ ->
-                     do Stream.junk strm__; return $rewrite False sp_kont$
+                     do { Stream.junk strm__; $rewrite False sp_kont$ }
                  | _ -> $p_kont$ ]
                >>
           | _ ->
@@ -270,7 +273,7 @@ value rewrite_parser =
           >>
         else
           let v = free_var_in_expr 's' f in
-          <:expr< let s = strm__ in $subst v f$ s >>
+          <:expr< let $lid:v$ = strm__ in $f$ $lid:v$ >>
     | e -> semantic e ]
 ;
 
@@ -287,7 +290,7 @@ value parser_of_expr =
         [([spc :: sp], epo, e) :: parser_cases p_kont]
     | <:expr<
         match Stream.peek strm__ with
-        [ Some $p$ $when:wo$ -> do Stream.junk strm__; return $sp_kont$
+        [ Some $p$ $when:wo$ -> do { Stream.junk strm__; $sp_kont$ }
         | _ -> $p_kont$ ]
       >> ->
         let spc = (SPCterm (p, wo), None) in
@@ -317,7 +320,7 @@ value parser_of_expr =
         ([spc :: sp], epo, e)
     | <:expr<
         match Stream.peek strm__ with
-        [ Some $p$ $when:wo$ -> do Stream.junk strm__; return $sp_kont$
+        [ Some $p$ $when:wo$ -> do { Stream.junk strm__; $sp_kont$ }
         | _ -> raise (Stream.Error $err$) ]
       >> ->
         let err =
@@ -338,7 +341,7 @@ value parser_of_expr =
   parser_cases
 ;
 
-value parser_cases expr patt b spel k =
+value parser_cases b spel k =
   let rec parser_cases b spel k =
     match spel with
     [ [] -> [: `HVbox [: b; k :] :]
@@ -356,8 +359,8 @@ value parser_cases expr patt b spel k =
       [: b;
          `HOVbox
             [: `HOVbox
-                 [: `S LR "[:";
-                    stream_patt [: :] sp [: `S LR ":]"; epo :] :];
+                  [: `S LR "[:";
+                     stream_patt [: :] sp [: `S LR ":]"; epo :] :];
                `expr e "" k :] :]
   and stream_patt b sp k =
     match sp with
@@ -377,7 +380,8 @@ value parser_cases expr patt b spel k =
            stream_patt [: :] spcl k :] ]
   and stream_patt_comp b spc k =
     match spc with
-    [ SPCterm (p, w) -> HVbox [: b; `S LO "`"; `patt p "" (when_opt w k) :]
+    [ SPCterm (p, w) ->
+        HVbox [: b; `S LO "`"; `patt p "" [: :]; `HVbox [: when_opt w k :] :]
     | SPCnterm p e ->
         HVbox [: b; `HVbox [: `patt p "" [: `S LR "=" :]; `expr e "" k :] :]
     | SPCsterm p -> HVbox [: b; `patt p "" k :] ]
@@ -389,7 +393,7 @@ value parser_cases expr patt b spel k =
   parser_cases b spel k
 ;
 
-value parser_body expr patt e dg k =
+value parser_body e dg k =
   let (bp, e) =
     match e with
     [ <:expr< let $bp$ = Stream.count strm__ in $e$ >> -> (Some bp, e)
@@ -412,23 +416,22 @@ value parser_body expr patt e dg k =
                  match bp with
                  [ Some p -> [: `patt p "" [: :] :]
                  | _ -> [: :] ] :];
-           parser_cases expr patt [: :] [spe] k :]
+           parser_cases [: :] [spe] k :]
   | spel ->
-      HVbox
+      Vbox
         [: `HVbox [: :];
            `HVbox
               [: `S LR "parser";
                  match bp with
                  [ Some p -> [: `patt p "" [: :] :]
                  | _ -> [: :] ] :];
-           parser_cases expr patt [: `S LR "[" :] spel
-             [: `S LR "]"; k :] :] ]
+           parser_cases [: `S LR "[" :] spel [: `S LR "]"; k :] :] ]
 ;
 
-value pmatch expr patt e dg k =
+value pmatch e dg k =
   let (me, e) =
     match e with
-    [ <:expr< let strm__ = $me$ in $e$ >> -> (me, e)
+    [ <:expr< let (strm__ : Stream.t _) = $me$ in $e$ >> -> (me, e)
     | _ -> failwith "Pr_rp.pmatch" ]
   in
   let (bp, e) =
@@ -438,16 +441,55 @@ value pmatch expr patt e dg k =
   in
   let e = rewrite_parser e in
   let spel = parser_of_expr e in
-  HVbox
+  Vbox
     [: `HVbox [: :];
        `HVbox
           [: `S LR "match"; `expr me "" [: `S LR "with" :]; `S LR "parser";
              match bp with
              [ Some p -> [: `patt p "" [: :] :]
              | _ -> [: :] ] :];
-       parser_cases expr patt [: `S LR "[" :] spel [: `S LR "]"; k :] :]
+       parser_cases [: `S LR "[" :] spel [: `S LR "]"; k :] :]
 ;
 
-Pretty.add_pr_fun "stream" stream;
-Pretty.add_pr_fun "parser" parser_body;
-Pretty.add_pr_fun "parser_m" pmatch;
+(* Printer extensions *)
+
+pr_expr_fun_args.val :=
+  extfun pr_expr_fun_args.val with
+  [ <:expr< fun strm__ -> $_$ >> as ge -> ([], ge)
+  | <:expr< fun [(strm__ : $_$) -> $_$] >> as ge -> ([], ge) ];
+
+let lev = find_pr_level "top" pr_expr.pr_levels in
+lev.pr_rules :=
+  extfun lev.pr_rules with
+  [ <:expr< let (strm__ : Stream.t _) = $_$ in $_$ >> as e ->
+      fun curr next _ k -> [: `pmatch e "" k :]
+  | <:expr< fun strm__ -> $x$ >> ->
+      fun curr next _ k -> [: `parser_body x "" k :]
+  | <:expr< fun (strm__ : $_$) -> $x$ >> ->
+      fun curr next _ k -> [: `parser_body x "" k :] ];
+
+let lev = find_pr_level "apply" pr_expr.pr_levels in
+lev.pr_rules :=
+  extfun lev.pr_rules with
+  [ <:expr< Stream.iapp $_$ $_$ >> | <:expr< Stream.icons $_$ $_$ >> |
+    <:expr< Stream.ising $_$ >> | <:expr< Stream.lapp (fun _ -> $_$) $_$ >> |
+    <:expr< Stream.lcons (fun _ -> $_$) $_$ >> |
+    <:expr< Stream.lsing (fun _ -> $_$) >> | <:expr< Stream.sempty >> |
+    <:expr< Stream.slazy $_$ >> as e ->
+      fun curr next _ k -> [: `next e "" k :] ];
+
+let lev = find_pr_level "dot" pr_expr.pr_levels in
+lev.pr_rules :=
+  extfun lev.pr_rules with
+  [ <:expr< Stream.sempty >> as e ->
+      fun curr next _ k -> [: `next e "" k :] ];
+
+let lev = find_pr_level "simple" pr_expr.pr_levels in
+lev.pr_rules :=
+  extfun lev.pr_rules with
+  [ <:expr< Stream.iapp $_$ $_$ >> | <:expr< Stream.icons $_$ $_$ >> |
+    <:expr< Stream.ising $_$ >> | <:expr< Stream.lapp (fun _ -> $_$) $_$ >> |
+    <:expr< Stream.lcons (fun _ -> $_$) $_$ >> |
+    <:expr< Stream.lsing (fun _ -> $_$) >> | <:expr< Stream.sempty >> |
+    <:expr< Stream.slazy $_$ >> as e ->
+      fun curr next _ k -> [: `stream e "" k :] ];
