@@ -5,12 +5,12 @@
 (*                                                                     *)
 (*        Daniel de Rauglaudre, projet Cristal, INRIA Rocquencourt     *)
 (*                                                                     *)
-(*  Copyright 1998 Institut National de Recherche en Informatique et   *)
+(*  Copyright 2001 Institut National de Recherche en Informatique et   *)
 (*  Automatique.  Distributed only by permission.                      *)
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: grammar.ml,v 2.3 2000/03/28 19:19:14 ddr Exp $ *)
+(* $Id: grammar.ml,v 3.10 2001/06/24 18:21:54 ddr Exp $ *)
 
 open Stdpp;
 open Gramext;
@@ -29,34 +29,35 @@ value print_str s = print_string ("\"" ^ String.escaped s ^ "\"");
 
 value rec print_symbol =
   fun
-  [ Slist0 s ->
-      do print_string "LIST0"; print_string " "; print_symbol1 s; return ()
+  [ Slist0 s -> do { print_string "LIST0"; print_string " "; print_symbol1 s }
   | Slist0sep s t ->
-      do print_string "LIST0";
-         print_string " ";
-         print_symbol1 s;
-         print_string " SEP ";
-         print_symbol1 t;
-      return ()
-  | Slist1 s ->
-      do print_string "LIST1"; print_string " "; print_symbol1 s; return ()
+      do {
+        print_string "LIST0";
+        print_string " ";
+        print_symbol1 s;
+        print_string " SEP ";
+        print_symbol1 t
+      }
+  | Slist1 s -> do { print_string "LIST1"; print_string " "; print_symbol1 s }
   | Slist1sep s t ->
-      do print_string "LIST1";
-         print_string " ";
-         print_symbol1 s;
-         print_string " SEP ";
-         print_symbol1 t;
-      return ()
-  | Sopt s -> do print_string "OPT "; print_symbol1 s; return ()
+      do {
+        print_string "LIST1";
+        print_string " ";
+        print_symbol1 s;
+        print_string " SEP ";
+        print_symbol1 t
+      }
+  | Sopt s -> do { print_string "OPT "; print_symbol1 s }
   | Stoken (con, prm) when con <> "" && prm <> "" ->
-      do print_string con; print_space (); print_str prm; return ()
+      do { print_string con; print_space (); print_str prm }
   | Snterml e l ->
-      do print_string e.ename;
-         print_space ();
-         print_string "LEVEL";
-         print_space ();
-         print_str l;
-      return ()
+      do {
+        print_string e.ename;
+        print_space ();
+        print_string "LEVEL";
+        print_space ();
+        print_str l
+      }
   | s -> print_symbol1 s ]
 and print_symbol1 =
   fun
@@ -66,27 +67,39 @@ and print_symbol1 =
   | Snext -> print_string "NEXT"
   | Stoken (con, "") -> print_string con
   | Stree t -> print_level print_space (flatten_tree t)
-  | s -> do print_string "("; print_symbol s; print_string ")"; return () ]
+  | s -> do { print_string "("; print_symbol s; print_string ")" } ]
 and print_rule symbols =
-  do open_hovbox 0; return
-  let _ =
-    List.fold_left
-      (fun sep symbol ->
-         do sep (); print_symbol symbol; return
-         fun () -> do print_string ";"; return print_space ())
-      (fun () -> ()) symbols
-  in
-  close_box ()
+  do {
+    open_hovbox 0;
+    let _ =
+      List.fold_left
+        (fun sep symbol ->
+           do {
+             sep ();
+             print_symbol symbol;
+             fun () -> do { print_string ";"; print_space () }
+           })
+        (fun () -> ()) symbols
+    in
+    close_box ()
+  }
 and print_level print_space rules =
-  do open_hovbox 0; print_string "[ "; return
-  let _ =
-    List.fold_left
-      (fun sep rule ->
-         do sep (); print_rule rule; return
-         fun () -> do print_space (); print_string "| "; return ())
-      (fun () -> ()) rules
-  in
-  do print_string " ]"; close_box (); return ()
+  do {
+    open_hovbox 0;
+    print_string "[ ";
+    let _ =
+      List.fold_left
+        (fun sep rule ->
+           do {
+             sep ();
+             print_rule rule;
+             fun () -> do { print_space (); print_string "| " }
+           })
+        (fun () -> ()) rules
+    in
+    print_string " ]";
+    close_box ()
+  }
 ;
 
 value print_levels elev =
@@ -97,37 +110,41 @@ value print_levels elev =
            List.map (fun t -> [Sself :: t]) (flatten_tree lev.lsuffix) @
              flatten_tree lev.lprefix
          in
-         do sep ();
-            open_hovbox 2;
-            match lev.lname with
-            [ Some n ->
-                do print_string ("\"" ^ String.escaped n ^ "\"");
-                   print_break 1 2;
-                return ()
-            | _ -> () ];
-            match lev.assoc with
-            [ LeftA -> print_string "LEFTA"
-            | RightA -> print_string "RIGHTA"
-            | NonA -> print_string "NONA" ];
-            close_box ();
-            print_break 1 2;
-            print_level force_newline rules;
-         return fun () -> do print_cut (); print_string "| "; return ())
+         do {
+           sep ();
+           open_hovbox 2;
+           match lev.lname with
+           [ Some n ->
+               do {
+                 print_string ("\"" ^ String.escaped n ^ "\"");
+                 print_break 1 2
+               }
+           | _ -> () ];
+           match lev.assoc with
+           [ LeftA -> print_string "LEFTA"
+           | RightA -> print_string "RIGHTA"
+           | NonA -> print_string "NONA" ];
+           close_box ();
+           print_break 1 2;
+           print_level force_newline rules;
+           fun () -> do { print_cut (); print_string "| " }
+         })
       (fun () -> ()) elev
   in
   ()
 ;
 
 value print_entry e =
-  do open_vbox 0;
-     print_string "[ ";
-     match e.edesc with
-     [ Dlevels elev -> print_levels elev
-     | Dparser _ -> print_string "<parser>" ];
-     print_string " ]";
-     close_box ();
-     print_newline ();
-  return ()
+  do {
+    open_vbox 0;
+    print_string "[ ";
+    match e.edesc with
+    [ Dlevels elev -> print_levels elev
+    | Dparser _ -> print_string "<parser>" ];
+    print_string " ]";
+    close_box ();
+    print_newline ()
+  }
 ;
 
 type g = Gramext.grammar;
@@ -138,7 +155,9 @@ value floc = ref (fun _ -> failwith "internal error when computing location");
 value loc_of_token_interval bp ep =
   if bp == ep then
     if bp == 0 then (0, 1)
-    else let a = snd (floc.val (bp-1)) in (a, a + 1)
+    else
+      let a = snd (floc.val (bp - 1)) in
+      (a, a + 1)
   else
     let (bp1, bp2) = floc.val bp in
     let (ep1, ep2) = floc.val (pred ep) in
@@ -181,6 +200,78 @@ and name_of_tree_failed entry =
   | DeadEnd | LocAct _ _ -> "???" ]
 ;
 
+value search_tree_in_entry prev_symb tree =
+  fun
+  [ Dlevels levels ->
+      let rec search_levels =
+        fun
+        [ [] -> tree
+        | [level :: levels] ->
+            match search_level level with
+            [ Some tree -> tree
+            | None -> search_levels levels ] ]
+      and search_level level =
+        match search_tree level.lsuffix with
+        [ Some t -> Some (Node {node = Sself; son = t; brother = DeadEnd})
+        | None -> search_tree level.lprefix ]
+      and search_tree t =
+        if tree <> DeadEnd && t == tree then Some t
+        else
+          match t with
+          [ Node n ->
+              match search_symbol n.node with
+              [ Some symb ->
+                  Some (Node {node = symb; son = n.son; brother = DeadEnd})
+              | None ->
+                  match search_tree n.son with
+                  [ Some t ->
+                      Some (Node {node = n.node; son = t; brother = DeadEnd})
+                  | None -> search_tree n.brother ] ]
+          | _ -> None ]
+      and search_symbol symb =
+        match symb with
+        [ Snterm _ | Snterml _ _ | Slist0 _ | Slist0sep _ _ | Slist1 _ |
+          Slist1sep _ _ | Sopt _ | Stoken _ | Stree _
+          when symb == prev_symb ->
+            Some symb
+        | Slist0 symb ->
+            match search_symbol symb with
+            [ Some symb -> Some (Slist0 symb)
+            | None -> None ]
+        | Slist0sep symb sep ->
+            match search_symbol symb with
+            [ Some symb -> Some (Slist0sep symb sep)
+            | None ->
+                match search_symbol sep with
+                [ Some sep -> Some (Slist0sep symb sep)
+                | None -> None ] ]
+        | Slist1 symb ->
+            match search_symbol symb with
+            [ Some symb -> Some (Slist1 symb)
+            | None -> None ]
+        | Slist1sep symb sep ->
+            match search_symbol symb with
+            [ Some symb -> Some (Slist1sep symb sep)
+            | None ->
+                match search_symbol sep with
+                [ Some sep -> Some (Slist1sep symb sep)
+                | None -> None ] ]
+        | Sopt symb ->
+            match search_symbol symb with
+            [ Some symb -> Some (Sopt symb)
+            | None -> None ]
+        | Stree t ->
+            match search_tree t with
+            [ Some t -> Some (Stree t)
+            | None -> None ]
+        | _ -> None ]
+      in
+      search_levels levels
+  | _ -> tree ]
+;
+
+value error_verbose = ref False;
+
 value tree_failed entry prev_symb_result prev_symb tree =
   let txt = name_of_tree_failed entry tree in
   let txt =
@@ -210,7 +301,28 @@ value tree_failed entry prev_symb_result prev_symb tree =
     | Sopt _ | Stree _ -> txt ^ " expected"
     | _ -> txt ^ " expected after " ^ name_of_symbol entry prev_symb ]
   in
-  txt ^ " (in [" ^ entry.ename ^ "])"
+  do {
+    if error_verbose.val then do {
+      let tree = search_tree_in_entry prev_symb tree entry.edesc in
+      set_formatter_out_channel stderr;
+      open_vbox 0;
+      print_newline ();
+      print_string "----------------------------------";
+      print_newline ();
+      printf "Parse error in entry [%s], rule:" entry.ename;
+      print_break 0 2;
+      open_vbox 0;
+      print_level force_newline (flatten_tree tree);
+      close_box ();
+      print_newline ();
+      print_string "----------------------------------";
+      print_newline ();
+      close_box ();
+      print_newline ()
+    }
+    else ();
+    txt ^ " (in [" ^ entry.ename ^ "])"
+  }
 ;
 
 value symb_failed entry prev_symb_result prev_symb symb =
@@ -231,8 +343,7 @@ value level_number entry lab =
     fun
     [ [] -> failwith ("unknown level " ^ lab)
     | [lev :: levs] ->
-        if is_level_labelled lab lev then levn
-        else lookup (succ levn) levs ]
+        if is_level_labelled lab lev then levn else lookup (succ levn) levs ]
   in
   match entry.edesc with
   [ Dlevels elev -> lookup 0 elev
@@ -293,19 +404,20 @@ value recover parser_of_tree entry nlevn alevn bp a s son strm =
 
 value std_token_parse =
   fun
-  [ (p_con, "") ->
-      parser [: `(con, prm) when con = p_con :] -> prm
+  [ (p_con, "") -> parser [: `(con, prm) when con = p_con :] -> prm
   | (p_con, p_prm) ->
       parser [: `(con, prm) when con = p_con && prm = p_prm :] -> prm ]
 ;
 
 value peek_nth n strm =
   let list = Stream.npeek n strm in
-  loop list n where rec loop list n =
+  let rec loop list n =
     match (list, n) with
     [ ([x :: _], 1) -> Some x
     | ([_ :: l], n) -> loop l (n - 1)
     | ([], _) -> None ]
+  in
+  loop list n
 ;
 
 value rec get_token_list entry tokl last_tok tree =
@@ -329,7 +441,9 @@ value rec parser_of_tree entry nlevn alevn =
       parser [: a = entry.estart alevn :] -> app act a
   | Node {node = Sself; son = LocAct act _; brother = bro} ->
       let p2 = parser_of_tree entry nlevn alevn bro in
-      parser [ [: a = entry.estart alevn :] -> app act a | [: a = p2 :] -> a ]
+      parser
+      [ [: a = entry.estart alevn :] -> app act a
+      | [: a = p2 :] -> a ]
   | Node {node = s; son = son; brother = DeadEnd} ->
       let tokl =
         match s with
@@ -380,52 +494,58 @@ and parser_of_token_list p1 tokl =
   loop 1 tokl where rec loop n =
     fun
     [ [(p_con, "")] ->
-        let ps (strm__ : Stream.t _) =
-          match peek_nth n strm__ with
-          [ Some (con, prm) when con = p_con ->
-              do for i = 1 to n do Stream.junk strm__; done; return
-              Obj.repr prm
+        let ps strm =
+          match peek_nth n strm with
+          [ Some (con, prm) when p_con = "ANY" || con = p_con ->
+              do {
+                for i = 1 to n do { Stream.junk strm }; Obj.repr prm
+              }
           | _ -> raise Stream.Failure ]
         in
         parser bp [: a = ps; act = p1 bp a :] -> app act a
     | [(p_con, p_prm)] ->
-        let ps (strm__ : Stream.t _) =
-          match peek_nth n strm__ with
-          [ Some (con, prm) when con = p_con && prm = p_prm ->
-              do for i = 1 to n do Stream.junk strm__; done; return
-              Obj.repr prm
+        let ps strm =
+          match peek_nth n strm with
+          [ Some (con, prm)
+            when (p_con = "ANY" || con = p_con) && prm = p_prm ->
+              do {
+                for i = 1 to n do { Stream.junk strm }; Obj.repr prm
+              }
           | _ -> raise Stream.Failure ]
         in
         parser bp [: a = ps; act = p1 bp a :] -> app act a
     | [(p_con, "") :: tokl] ->
-        let ps (strm__ : Stream.t _) =
-          match peek_nth n strm__ with
-          [ Some (con, prm) when con = p_con -> prm
+        let ps strm =
+          match peek_nth n strm with
+          [ Some (con, prm) when p_con = "ANY" || con = p_con -> prm
           | _ -> raise Stream.Failure ]
         in
         let p1 = loop (n + 1) tokl in
-        fun (strm__ : Stream.t _) ->
-          let a = ps strm__ in
-          let act = p1 strm__ in
-          app act a
+        parser
+          [: a = ps; s :] ->
+            let act = p1 s in
+            app act a
     | [(p_con, p_prm) :: tokl] ->
-        let ps (strm__ : Stream.t _) =
-          match peek_nth n strm__ with
-          [ Some (con, prm) when con = p_con && prm = p_prm -> prm
+        let ps strm =
+          match peek_nth n strm with
+          [ Some (con, prm)
+            when (p_con = "ANY" || con = p_con) && prm = p_prm -> prm
           | _ -> raise Stream.Failure ]
         in
         let p1 = loop (n + 1) tokl in
-        fun (strm__ : Stream.t _) ->
-          let a = ps strm__ in
-          let act = p1 strm__ in
-          app act a
+        parser
+          [: a = ps; s :] ->
+            let act = p1 s in
+            app act a
     | [] -> assert False ]
 and parser_of_symbol entry nlevn =
   fun
   [ Slist0 s ->
       let ps = parser_of_symbol entry nlevn s in
       let rec loop al =
-        parser [ [: a = ps; s :] -> loop [a :: al] s | [: :] -> al ]
+        parser
+        [ [: a = ps; s :] -> loop [a :: al] s
+        | [: :] -> al ]
       in
       parser [: a = loop [] :] -> Obj.repr (List.rev a)
   | Slist0sep symb sep ->
@@ -443,7 +563,9 @@ and parser_of_symbol entry nlevn =
   | Slist1 s ->
       let ps = parser_of_symbol entry nlevn s in
       let rec loop al =
-        parser [ [: a = ps; s :] -> loop [a :: al] s | [: :] -> al ]
+        parser
+        [ [: a = ps; s :] -> loop [a :: al] s
+        | [: :] -> al ]
       in
       parser [: a = ps; s :] -> Obj.repr (List.rev (loop [a] s))
   | Slist1sep symb sep ->
@@ -456,8 +578,8 @@ and parser_of_symbol entry nlevn =
                parser
                [ [: a = ps :] -> a
                | [: a =
-                      parser_of_symbol entry nlevn
-                        (top_symb entry symb) :] -> a
+                      parser_of_symbol entry nlevn (top_symb entry symb) :] ->
+                   a
                | [: :] ->
                    raise (Stream.Error (symb_failed entry v sep symb)) ];
              s :] ->
@@ -467,15 +589,22 @@ and parser_of_symbol entry nlevn =
       parser [: a = ps; s :] -> Obj.repr (List.rev (kont [a] s))
   | Sopt s ->
       let ps = parser_of_symbol entry nlevn s in
-      parser [ [: a = ps :] -> Obj.repr (Some a) | [: :] -> Obj.repr None ]
+      parser
+      [ [: a = ps :] -> Obj.repr (Some a)
+      | [: :] -> Obj.repr None ]
   | Stree t ->
       let pt = parser_of_tree entry 1 0 t in
       parser bp
-        [: a = pt :] ep -> let loc = loc_of_token_interval bp ep in app a loc
+        [: a = pt :] ep ->
+          let loc = loc_of_token_interval bp ep in
+          app a loc
   | Snterm e -> parser [: a = e.estart 0 :] -> a
   | Snterml e l -> parser [: a = e.estart (level_number e l) :] -> a
   | Sself -> parser [: a = entry.estart 0 :] -> a
   | Snext -> parser [: a = entry.estart nlevn :] -> a
+  | Stoken ("ANY", v) ->
+      if v = "" then parser [: `(_, x) :] -> Obj.repr x
+      else parser [: `(_, x) when x = v :] -> Obj.repr x
   | Stoken tok ->
       match entry.egram.glexer.Token.tparse tok with
       [ Some f -> (Obj.magic f : Stream.t Token.t -> Obj.t)
@@ -543,36 +672,55 @@ value continue_parser_of_entry entry =
   match entry.edesc with
   [ Dlevels elev ->
       let p = continue_parser_of_levels entry 0 elev in
-      fun levn bp a -> parser [ [: a = p levn bp a :] -> a | [: :] -> a ]
+      fun levn bp a ->
+        parser
+        [ [: a = p levn bp a :] -> a
+        | [: :] -> a ]
   | Dparser p -> fun levn bp a -> parser [] ]
 ;
 
+value empty_entry ename levn strm =
+  raise (Stream.Error ("entry [" ^ ename ^ "] is empty"))
+;
+
 value rec start_parser_of_entry entry =
   match entry.edesc with
-  [ Dlevels elev -> start_parser_of_levels entry 0 elev
+  [ Dlevels [] -> empty_entry entry.ename
+  | Dlevels elev -> start_parser_of_levels entry 0 elev
   | Dparser p -> fun levn strm -> p strm ]
 ;
 
 value parse_parsable entry efun (cs, (ts, fun_loc)) =
-  let restore = let old_floc = floc.val in fun () -> floc.val := old_floc in
-  do floc.val := fun_loc; return
-  try
-    let r = efun ts in
-    do restore (); return r
-  with exc ->
-    let (loc, exc) =
-      match exc with
-      [ Stream.Failure ->
-          (try fun_loc (Stream.count ts) with _ ->
-             (Stream.count cs, Stream.count cs + 1),
-           Stream.Error ("illegal begin of " ^ entry.ename))
-      | Stream.Error _ ->
-          (try fun_loc (Stream.count ts) with _ ->
-             (Stream.count cs, Stream.count cs + 1),
-           exc)
-      | exc -> ((Stream.count cs, Stream.count cs + 1), exc) ]
-    in
-    do restore (); return raise_with_loc loc exc
+  let restore =
+    let old_floc = floc.val in
+    fun () -> floc.val := old_floc
+  in
+  do {
+    floc.val := fun_loc;
+    try
+      let r = efun ts in
+      do { restore (); r }
+    with
+    [ Stream.Failure ->
+        let loc =
+          try fun_loc (Stream.count ts) with _ ->
+            (Stream.count cs, Stream.count cs + 1)
+        in
+        do {
+          restore ();
+          raise_with_loc loc
+            (Stream.Error ("illegal begin of " ^ entry.ename))
+        }
+    | Stream.Error _ as exc ->
+        let loc =
+          try fun_loc (Stream.count ts) with _ ->
+            (Stream.count cs, Stream.count cs + 1)
+        in
+        do { restore (); raise_with_loc loc exc }
+    | exc ->
+        let loc = (Stream.count cs, Stream.count cs + 1) in
+        do { restore (); raise_with_loc loc exc } ]
+  }
 ;
 
 value wrap_parse entry efun cs =
@@ -588,41 +736,44 @@ value create lexer = {gtokens = create_toktab (); glexer = lexer};
 value extend_entry entry position rules =
   try
     let elev = Gramext.levels_of_rules entry position rules in
-    do entry.edesc := Dlevels elev;
-       entry.estart :=
-         fun lev strm ->
-           let f = start_parser_of_entry entry in
-           do entry.estart := f; return f lev strm;
-       entry.econtinue :=
-         fun lev bp a strm ->
-           let f = continue_parser_of_entry entry in
-           do entry.econtinue := f; return f lev bp a strm;
-    return ()
+    do {
+      entry.edesc := Dlevels elev;
+      entry.estart :=
+        fun lev strm ->
+          let f = start_parser_of_entry entry in
+          do { entry.estart := f; f lev strm };
+      entry.econtinue :=
+        fun lev bp a strm ->
+          let f = continue_parser_of_entry entry in
+          do { entry.econtinue := f; f lev bp a strm }
+    }
   with
   [ Token.Error s ->
-      do Printf.eprintf "Lexer initialization error.\n%s\n"
-           (String.capitalize s);
-         flush stderr;
-      return failwith "Grammar.extend" ]
+      do {
+        Printf.eprintf "Lexer initialization error.\n%s\n"
+          (String.capitalize s);
+        flush stderr;
+        failwith "Grammar.extend"
+      } ]
 ;
 
 value extend entry_rules_list =
   let gram = ref None in
-  do List.iter
-       (fun (entry, position, rules) ->
-          do match gram.val with
-             [ Some g ->
-                 if g != entry.egram then
-                   do Printf.eprintf
-                        "Error: entries with different grammars\n";
-                      flush stderr;
-                   return failwith "Grammar.extend"
-                 else ()
-             | None -> gram.val := Some entry.egram ];
-             extend_entry entry position rules;
-          return ())
-       entry_rules_list;
-  return ()
+  List.iter
+    (fun (entry, position, rules) ->
+       do {
+         match gram.val with
+         [ Some g ->
+             if g != entry.egram then do {
+               Printf.eprintf "Error: entries with different grammars\n";
+               flush stderr;
+               failwith "Grammar.extend"
+             }
+             else ()
+         | None -> gram.val := Some entry.egram ];
+         extend_entry entry position rules
+       })
+    entry_rules_list
 ;
 
 (* Deleting a rule *)
@@ -631,50 +782,48 @@ value delete_rule entry sl =
   match entry.edesc with
   [ Dlevels levs ->
       let levs = Gramext.delete_rule_in_level_list entry sl levs in
-      do entry.edesc := Dlevels levs;
-         entry.estart :=
-           fun lev strm ->
-             let f = start_parser_of_entry entry in
-             do entry.estart := f; return f lev strm;
-         entry.econtinue :=
-           fun lev bp a strm ->
-             let f = continue_parser_of_entry entry in
-             do entry.econtinue := f; return f lev bp a strm;
-      return ()
+      do {
+        entry.edesc := Dlevels levs;
+        entry.estart :=
+          fun lev strm ->
+            let f = start_parser_of_entry entry in
+            do { entry.estart := f; f lev strm };
+        entry.econtinue :=
+          fun lev bp a strm ->
+            let f = continue_parser_of_entry entry in
+            do { entry.econtinue := f; f lev bp a strm }
+      }
   | _ -> () ]
 ;
 
 (* Unsafe *)
 
 value clear_entry e =
-  do e.estart := fun _ -> parser [];
-     e.econtinue := fun _ _ _ -> parser [];
-     match e.edesc with
-     [ Dlevels _ -> e.edesc := Dlevels []
-     | Dparser _ -> () ];
-  return ()
+  do {
+    e.estart := fun _ -> parser [];
+    e.econtinue := fun _ _ _ -> parser [];
+    match e.edesc with
+    [ Dlevels _ -> e.edesc := Dlevels []
+    | Dparser _ -> () ]
+  }
 ;
 
-value reinit_gram g lexer =
-  do Hashtbl.clear g.gtokens; g.glexer := lexer; return ()
-;
+value reinit_gram g lexer = do { Hashtbl.clear g.gtokens; g.glexer := lexer };
 
 module Unsafe =
-  struct
-    value clear_entry = clear_entry;
-    value reinit_gram = reinit_gram;
-  end
+  struct value clear_entry = clear_entry; value reinit_gram = reinit_gram; end
 ;
 
 exception EntryFound of g_entry;
 value find_entry e s =
   let rec find_levels levs =
     try
-      do List.iter
-           (fun lev ->
-              do find_tree lev.lsuffix; find_tree lev.lprefix; return ())
-           levs;
-      return raise Not_found
+      do {
+        List.iter
+          (fun lev -> do { find_tree lev.lsuffix; find_tree lev.lprefix })
+          levs;
+        raise Not_found
+      }
     with
     [ EntryFound e -> e
     | _ -> raise Not_found ]
@@ -692,7 +841,7 @@ value find_entry e s =
   and find_tree =
     fun
     [ Node {node = s; brother = bro; son = son} ->
-        do find_symbol s; find_tree bro; find_tree son; return ()
+        do { find_symbol s; find_tree bro; find_tree son }
     | _ -> () ]
   in
   match e.edesc with
@@ -706,20 +855,17 @@ module Entry =
   struct
     type e 'a = g_entry;
     value create g n =
-      {egram = g; ename = n; estart = fun _ -> parser [];
-       econtinue = fun _ _ _ -> parser []; edesc = Dlevels []}
+      {egram = g; ename = n; estart = empty_entry n;
+       econtinue _ _ _ = parser []; edesc = Dlevels []}
     ;
-    value parse (entry : e 'a) cs =
-      (Obj.magic (wrap_parse entry (entry.estart 0) cs) : 'a)
-    ;
-    value parse_token (entry : e 'a) ts =
-      (Obj.magic (entry.estart 0 ts) : 'a)
+    value parse (entry : e 'a) cs : 'a =
+      Obj.magic (wrap_parse entry (entry.estart 0) cs)
     ;
+    value parse_token (entry : e 'a) ts : 'a = Obj.magic (entry.estart 0 ts);
     value name e = e.ename;
-    value of_parser g n (p : Stream.t Token.t -> 'a) =
-      ({egram = g; ename = n; estart = fun _ -> (Obj.magic p);
-        econtinue = fun _ _ _ -> parser []; edesc = Dparser (Obj.magic p)} :
-       e 'a)
+    value of_parser g n (p : Stream.t Token.t -> 'a) : e 'a =
+      {egram = g; ename = n; estart _ = Obj.magic p;
+       econtinue _ _ _ = parser []; edesc = Dparser (Obj.magic p)}
     ;
     external obj : e 'a -> Gramext.g_entry = "%identity";
     value print e = print_entry (obj e);
@@ -730,23 +876,20 @@ module Entry =
 value tokens g con =
   let g = grammar_obj g in
   let list = ref [] in
-  do Hashtbl.iter
-       (fun (p_con, p_prm) c ->
-          if p_con = con then list.val := [(p_prm, c.val) :: list.val]
-          else ())
-       g.gtokens;
-  return list.val
+  do {
+    Hashtbl.iter
+      (fun (p_con, p_prm) c ->
+         if p_con = con then list.val := [(p_prm, c.val) :: list.val] else ())
+      g.gtokens;
+    list.val
+  }
 ;
 
 value warning_verbose = Gramext.warning_verbose;
 
 (* Functorial interface *)
 
-module type LexerType =
-  sig
-    value lexer : Token.lexer;
-  end
-;
+module type LexerType = sig value lexer : Token.lexer; end;
 
 module type S =
   sig
@@ -763,12 +906,14 @@ module type S =
         value of_parser : string -> (Stream.t Token.t -> 'a) -> e 'a;
         value print : e 'a -> unit;
         external obj : e 'a -> Gramext.g_entry = "%identity";
-      end;
+      end
+    ;
     module Unsafe :
       sig
         value reinit_gram : Token.lexer -> unit;
         value clear_entry : Entry.e 'a -> unit;
-      end;
+      end
+    ;
     value extend :
       Entry.e 'a -> option Gramext.position ->
         list
@@ -791,30 +936,28 @@ module Make (L : LexerType) : S =
       struct
         type e 'a = g_entry;
         value create n =
-          {egram = gram; ename = n; estart = fun _ -> parser [];
-           econtinue = fun _ _ _ -> parser []; edesc = Dlevels []}
+          {egram = gram; ename = n; estart = empty_entry n;
+           econtinue _ _ _ = parser []; edesc = Dlevels []}
         ;
-        value parse (e : e 'a) p =
-          (Obj.magic (parse_parsable e (e.estart 0) p) : 'a)
-        ;
-        value parse_token (e : e 'a) ts =
-          (Obj.magic (e.estart 0 ts) : 'a)
+        value parse (e : e 'a) p : 'a =
+          Obj.magic (parse_parsable e (e.estart 0) p)
         ;
+        value parse_token (e : e 'a) ts : 'a = Obj.magic (e.estart 0 ts);
         value name e = e.ename;
-        value of_parser n (p : Stream.t Token.t -> 'a) =
-          ({egram = gram; ename = n; estart = fun _ -> (Obj.magic p);
-            econtinue = fun _ _ _ -> parser [];
-            edesc = Dparser (Obj.magic p)} :
-           e 'a)
+        value of_parser n (p : Stream.t Token.t -> 'a) : e 'a =
+          {egram = gram; ename = n; estart _ = Obj.magic p;
+           econtinue _ _ _ = parser []; edesc = Dparser (Obj.magic p)}
         ;
         external obj : e 'a -> Gramext.g_entry = "%identity";
         value print e = print_entry (obj e);
-      end;
+      end
+    ;
     module Unsafe =
       struct
         value reinit_gram = Unsafe.reinit_gram gram;
         value clear_entry = Unsafe.clear_entry;
-      end;
+      end
+    ;
     value extend = extend_entry;
     value delete_rule e r = delete_rule (Entry.obj e) r;
   end
