@@ -5,12 +5,12 @@
 (*                                                                     *)
 (*        Daniel de Rauglaudre, projet Cristal, INRIA Rocquencourt     *)
 (*                                                                     *)
-(*  Copyright 1998 Institut National de Recherche en Informatique et   *)
+(*  Copyright 2001 Institut National de Recherche en Informatique et   *)
 (*  Automatique.  Distributed only by permission.                      *)
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: plexer.ml,v 2.5 2000/04/13 18:23:44 ddr Exp $ *)
+(* $Id: plexer.ml,v 3.7 2001/07/03 02:46:44 ddr Exp $ *)
 
 open Stdpp;
 open Token;
@@ -19,11 +19,13 @@ open Token;
 
 value buff = ref (String.create 80);
 value store len x =
-  do if len >= String.length buff.val then
-       buff.val := buff.val ^ String.create (String.length buff.val)
-     else ();
-     buff.val.[len] := x;
-  return succ len
+  do {
+    if len >= String.length buff.val then
+      buff.val := buff.val ^ String.create (String.length buff.val)
+    else ();
+    buff.val.[len] := x;
+    succ len
+  }
 ;
 value mstore len s =
   add_rec len 0 where rec add_rec len i =
@@ -33,29 +35,32 @@ value get_buff len = String.sub buff.val 0 len;
 
 (* The lexer *)
 
-value valch x = Char.code x - Char.code '0';
-
 value rec ident len =
   parser
-  [ [: `('A'..'Z' | 'a'..'z' | '\192'..'\214' | '\216'..'\246'
-      | '\248'..'\255' | '0'..'9' | '_' | ''' as c); s :] ->
+  [ [: `('A'..'Z' | 'a'..'z' | '\192'..'\214' | '\216'..'\246' |
+         '\248'..'\255' | '0'..'9' | '_' | ''' as
+         c)
+        ;
+       s :] ->
       ident (store len c) s
   | [: :] -> len ]
 and ident2 len =
   parser
   [ [: `('!' | '?' | '~' | '=' | '@' | '^' | '&' | '+' | '-' | '*' | '/' |
          '%' | '.' | ':' | '<' | '>' | '|' as
-         c);
+         c)
+        ;
        s :] ->
       ident2 (store len c) s
   | [: :] -> len ]
 and ident3 len =
   parser
-  [ [: `('0'..'9' | 'A'..'Z' | 'a'..'z' | '\192'..'\214' | '\216'..'\246'
-      | '\248'..'\255' | '_' | '!' | '%' | '&' | '*' | '+' |
-         '-' | '.' | '/' | ':' | '<' | '=' | '>' | '?' | '@' | '^' | '|' |
-         '~' | ''' | '$' as
-         c);
+  [ [: `('0'..'9' | 'A'..'Z' | 'a'..'z' | '\192'..'\214' | '\216'..'\246' |
+         '\248'..'\255' | '_' | '!' | '%' | '&' | '*' | '+' | '-' | '.' |
+         '/' | ':' | '<' | '=' | '>' | '?' | '@' | '^' | '|' | '~' | ''' |
+         '$' as
+         c)
+        ;
        s :] ->
       ident3 (store len c) s
   | [: :] -> len ]
@@ -99,121 +104,104 @@ and end_exponent_part len =
   | [: :] -> ("FLOAT", get_buff len) ]
 ;
 
-value rec escape =
-  parser
-  [ [: `'n' :] -> '\n'
-  | [: `'r' :] -> '\r'
-  | [: `'t' :] -> '\t'
-  | [: `'b' :] -> '\b'
-  | [: `'\\' :] -> '\\'
-  | [: `('0'..'9' as c); cod = escape1 (valch c) :] -> Char.chr cod ]
-and escape1 cod =
-  parser
-  [ [: `('0'..'9' as c); a = escape2 (10 * cod + valch c) :] -> a
-  | [: :] -> cod ]
-and escape2 cod =
-  parser [ [: `('0'..'9' as c) :] -> 10 * cod + valch c | [: :] -> cod ]
-;
-
-value escape_string =
-  parser
-  [ [: c = escape :] -> c
-  | [: `'"' :] -> '"' ]
-;
-value escape_char =
-  parser
-  [ [: c = escape :] -> c
-  | [: `''' :] -> ''' ]
-;
-
 value rec skip_spaces =
   parser
   [ [: `' ' | '\010' | '\013' | '\t' | '\026' | '\012'; s :] -> skip_spaces s
   | [: :] -> () ]
 ;
 
-value rec skip_indent =
-  parser
-  [ [: `' ' | '\t'; s :] -> skip_indent s
-  | [: :] -> () ]
-;
-
-value skip_opt_linefeed =
-  parser
-  [ [: `'\010' :] -> ()
-  | [: :] -> () ]
-;
-
 value error_on_unknown_keywords = ref False;
+value err loc msg = raise_with_loc loc (Token.Error msg);
 
-value next_token_fun find_id_kwd find_spe_kwd =
-  let err bp ep msg = raise_with_loc (bp, ep) (Token.Error msg) in
-  let keyword_or_error (bp, ep) s =
-    try ("", find_spe_kwd s) with
+value next_token_fun find_kwd =
+  let keyword_or_error loc s =
+    try (("", find_kwd s), loc) with
     [ Not_found ->
-        if error_on_unknown_keywords.val then
-          err bp ep ("illegal token: " ^ s)
-        else ("", s) ]
+        if error_on_unknown_keywords.val then err loc ("illegal token: " ^ s)
+        else (("", s), loc) ]
   in
   let rec next_token =
     parser bp
-    [ [: `('A'..'Z' | '\192'..'\214' | '\216'..'\222' as c); s :] ->
+    [ [: `' ' | '\010' | '\013' | '\t' | '\026' | '\012'; s :] ->
+        next_token s
+    | [: `'('; s :] -> left_paren bp s
+    | [: `'#'; s :] -> do { spaces_tabs s; linenum bp s }
+    | [: `('A'..'Z' | '\192'..'\214' | '\216'..'\222' as c); s :] ->
         let id = get_buff (ident (store 0 c) s) in
-        try ("", find_id_kwd id) with [ Not_found -> ("UIDENT", id) ]
+        let loc = (bp, Stream.count s) in
+        (try ("", find_kwd id) with [ Not_found -> ("UIDENT", id) ], loc)
     | [: `('a'..'z' | '\223'..'\246' | '\248'..'\255' | '_' as c); s :] ->
         let id = get_buff (ident (store 0 c) s) in
-        try ("", find_id_kwd id) with [ Not_found -> ("LIDENT", id) ]
-    | [: `('1'..'9' as c); s :] -> number (store 0 c) s
-    | [: `'0'; s :] -> base_number (store 0 '0') s
-    | [: `'''; s :] ep ->
+        let loc = (bp, Stream.count s) in
+        (try ("", find_kwd id) with [ Not_found -> ("LIDENT", id) ], loc)
+    | [: `('1'..'9' as c); s :] ->
+        let tok = number (store 0 c) s in
+        let loc = (bp, Stream.count s) in
+        (tok, loc)
+    | [: `'0'; s :] ->
+        let tok = base_number (store 0 '0') s in
+        let loc = (bp, Stream.count s) in
+        (tok, loc)
+    | [: `'''; s :] ->
         match Stream.npeek 2 s with
-        [ [_; '''] | ['\\'; _] -> ("CHAR", String.make 1 (char bp s))
-        | _ -> keyword_or_error (bp, ep) "'" ]
-    | [: `'"'; s :] -> ("STRING", string bp 0 s)
-    | [: `'$'; s :] -> locate_or_antiquot bp 0 s
-    | [: `('!' | '=' | '@' | '^' | '&' | '+' | '-' | '*' | '/' |
-           '%' as
-           c);
+        [ [_; '''] | ['\\'; _] ->
+            let tok = ("CHAR", char bp 0 s) in
+            let loc = (bp, Stream.count s) in
+            (tok, loc)
+        | _ -> keyword_or_error (bp, Stream.count s) "'" ]
+    | [: `'"'; s :] ->
+        let tok = ("STRING", string bp 0 s) in
+        let loc = (bp, Stream.count s) in
+        (tok, loc)
+    | [: `'$'; s :] ->
+        let tok = dollar bp 0 s in
+        let loc = (bp, Stream.count s) in
+        (tok, loc)
+    | [: `('!' | '=' | '@' | '^' | '&' | '+' | '-' | '*' | '/' | '%' as c);
          s :] ->
         let id = get_buff (ident2 (store 0 c) s) in
         keyword_or_error (bp, Stream.count s) id
     | [: `('~' as c);
-         r =
+         a =
            parser
            [ [: `('a'..'z' as c); len = ident (store 0 c);
                 t =
                   parser
-                    [ [: `':' :] -> "TILDEIDENTCOLON"
-                    | [: :] -> "TILDEIDENT" ] :] ->
-               (t, get_buff len)
+                  [ [: `':' :] -> "TILDEIDENTCOLON"
+                  | [: :] -> "TILDEIDENT" ] :] ep ->
+               ((t, get_buff len), (bp, ep))
            | [: s :] ->
                let id = get_buff (ident2 (store 0 c) s) in
-               keyword_or_error (bp, Stream.count s) id ] :] -> r
+               keyword_or_error (bp, Stream.count s) id ] :] ->
+        a
     | [: `('?' as c);
-         r =
+         a =
            parser
            [ [: `('a'..'z' as c); len = ident (store 0 c);
                 t =
                   parser
-                    [ [: `':' :] -> "QUESTIONIDENTCOLON"
-                    | [: :] -> "QUESTIONIDENT" ] :] ->
-               (t, get_buff len)
+                  [ [: `':' :] -> "QUESTIONIDENTCOLON"
+                  | [: :] -> "QUESTIONIDENT" ] :] ep ->
+               ((t, get_buff len), (bp, ep))
            | [: s :] ->
                let id = get_buff (ident2 (store 0 c) s) in
-               keyword_or_error (bp, Stream.count s) id ] :] -> r
+               keyword_or_error (bp, Stream.count s) id ] :] ->
+        a
     | [: `'<'; s :] -> less bp s
     | [: `(':' as c1);
          len =
            parser
            [ [: `(']' | ':' | '=' | '>' as c2) :] -> store (store 0 c1) c2
            | [: :] -> store 0 c1 ] :] ep ->
-        let id = get_buff len in keyword_or_error (bp, ep) id
+        let id = get_buff len in
+        keyword_or_error (bp, ep) id
     | [: `('>' | '|' as c1);
          len =
            parser
            [ [: `(']' | '}' as c2) :] -> store (store 0 c1) c2
            | [: a = ident2 (store 0 c1) :] -> a ] :] ep ->
-        let id = get_buff len in keyword_or_error (bp, ep) id
+        let id = get_buff len in
+        keyword_or_error (bp, ep) id
     | [: `('[' | '{' as c1); s :] ->
         let len =
           match Stream.npeek 2 s with
@@ -224,43 +212,47 @@ value next_token_fun find_id_kwd find_spe_kwd =
               | [: :] -> store 0 c1 ] ]
         in
         let ep = Stream.count s in
-        let id = get_buff len in keyword_or_error (bp, ep) id
-    | [: `'.'; id = parser [ [: `'.' :] -> ".." | [: :] -> "." ] :] ep ->
+        let id = get_buff len in
         keyword_or_error (bp, ep) id
-    | [: `';'; id = parser [ [: `';' :] -> ";;" | [: :] -> ";" ] :] ep ->
+    | [: `'.';
+         id =
+           parser
+           [ [: `'.' :] -> ".."
+           | [: :] -> "." ] :] ep ->
         keyword_or_error (bp, ep) id
-    | [: `'\\'; s :] -> ("LIDENT", get_buff (ident3 0 s))
-    | [: `c :] ep -> keyword_or_error (bp, ep) (String.make 1 c) ]
+    | [: `';';
+         id =
+           parser
+           [ [: `';' :] -> ";;"
+           | [: :] -> ";" ] :] ep ->
+        keyword_or_error (bp, ep) id
+    | [: `'\\'; s :] ep -> (("LIDENT", get_buff (ident3 0 s)), (bp, ep))
+    | [: `c :] ep -> keyword_or_error (bp, ep) (String.make 1 c)
+    | [: _ = Stream.empty :] -> (("EOI", ""), (bp, succ bp)) ]
   and less bp =
     parser
-    [ [: `'<'; s :] -> ("QUOTATION", (":" ^ get_buff (quotation bp 0 s)))
+    [ [: `'<'; len = quotation bp 0 :] ep ->
+        (("QUOTATION", ":" ^ get_buff len), (bp, ep))
     | [: `':'; i = parser [: len = ident 0 :] -> get_buff len;
-         `'<' ? "character '<' expected"; s :] ->
-        ("QUOTATION", i ^ ":" ^ get_buff (quotation bp 0 s))
-    | [: s :] ep ->
-        let id = get_buff (ident2 (store 0 '<') s) in
+         `'<' ? "character '<' expected"; len = quotation bp 0 :] ep ->
+        (("QUOTATION", i ^ ":" ^ get_buff len), (bp, ep))
+    | [: len = ident2 (store 0 '<') :] ep ->
+        let id = get_buff len in
         keyword_or_error (bp, ep) id ]
   and string bp len =
     parser
     [ [: `'"' :] -> get_buff len
-    | [: `'\\';
-         len =
-           parser
-           [ [: `'\010'; _ = skip_indent :] -> len
-           | [: `'\013'; _ = skip_opt_linefeed; _ = skip_indent :] -> len
-           | [: c = escape_string :] -> store len c
-           | [: `c :] -> store (store len '\\') c ];
-         s :] ->
-        string bp len s
+    | [: `'\\'; `c; s :] -> string bp (store (store len '\\') c) s
     | [: `c; s :] -> string bp (store len c) s
-    | [: :] ep -> err bp ep "string not terminated" ]
-  and char bp =
+    | [: :] ep -> err (bp, ep) "string not terminated" ]
+  and char bp len =
     parser
-    [ [: `'\\'; c = escape_char ? "escape character expected";
-         `''' ? "quote expected" :] -> c
-    | [: `c; `''' ? "quote expected" :] -> c
-    | [: :] ep -> err bp ep "char not terminated" ]
-  and locate_or_antiquot bp len =
+    [ [: `'''; s :] ->
+        if len = 0 then char bp (store len ''') s else get_buff len
+    | [: `'\\'; `c; s :] -> char bp (store (store len '\\') c) s
+    | [: `c; s :] -> char bp (store len c) s
+    | [: :] ep -> err (bp, ep) "char not terminated" ]
+  and dollar bp len =
     parser
     [ [: `'$' :] -> ("ANTIQUOT", ":" ^ get_buff len)
     | [: `('a'..'z' | 'A'..'Z' as c); s :] -> antiquot bp (store len c) s
@@ -271,8 +263,8 @@ value next_token_fun find_id_kwd find_spe_kwd =
     | [: `'\\'; `c; s :] ->
         ("ANTIQUOT", ":" ^ locate_or_antiquot_rest bp (store len c) s)
     | [: `c; s :] ->
-        ("ANTIQUOT", ":" ^  locate_or_antiquot_rest bp (store len c) s)
-    | [: :] ep -> err bp ep "antiquotation not terminated" ]
+        ("ANTIQUOT", ":" ^ locate_or_antiquot_rest bp (store len c) s)
+    | [: :] ep -> err (bp, ep) "antiquotation not terminated" ]
   and maybe_locate bp len =
     parser
     [ [: `'$' :] -> ("ANTIQUOT", ":" ^ get_buff len)
@@ -283,7 +275,7 @@ value next_token_fun find_id_kwd find_spe_kwd =
         ("ANTIQUOT", ":" ^ locate_or_antiquot_rest bp (store len c) s)
     | [: `c; s :] ->
         ("ANTIQUOT", ":" ^ locate_or_antiquot_rest bp (store len c) s)
-    | [: :] ep -> err bp ep "antiquotation not terminated" ]
+    | [: :] ep -> err (bp, ep) "antiquotation not terminated" ]
   and antiquot bp len =
     parser
     [ [: `'$' :] -> ("ANTIQUOT", ":" ^ get_buff len)
@@ -296,18 +288,18 @@ value next_token_fun find_id_kwd find_spe_kwd =
         ("ANTIQUOT", ":" ^ locate_or_antiquot_rest bp (store len c) s)
     | [: `c; s :] ->
         ("ANTIQUOT", ":" ^ locate_or_antiquot_rest bp (store len c) s)
-    | [: :] ep -> err bp ep "antiquotation not terminated" ]
+    | [: :] ep -> err (bp, ep) "antiquotation not terminated" ]
   and locate_or_antiquot_rest bp len =
     parser
     [ [: `'$' :] -> get_buff len
     | [: `'\\'; `c; s :] -> locate_or_antiquot_rest bp (store len c) s
     | [: `c; s :] -> locate_or_antiquot_rest bp (store len c) s
-    | [: :] ep -> err bp ep "antiquotation not terminated" ]
+    | [: :] ep -> err (bp, ep) "antiquotation not terminated" ]
   and quotation bp len =
     parser
     [ [: `'>'; s :] -> maybe_end_quotation bp len s
     | [: `'<'; s :] ->
-        quotation bp (maybe_nested_quotation bp (store len '<') s) s
+        quotation bp (maybe_nested_quotation bp (store len '<') strm__) s
     | [: `'\\';
          len =
            parser
@@ -316,7 +308,7 @@ value next_token_fun find_id_kwd find_spe_kwd =
          s :] ->
         quotation bp len s
     | [: `c; s :] -> quotation bp (store len c) s
-    | [: :] ep -> err bp ep "quotation not terminated" ]
+    | [: :] ep -> err (bp, ep) "quotation not terminated" ]
   and maybe_nested_quotation bp len =
     parser
     [ [: `'<'; s :] -> mstore (quotation bp (store len '<') s) ">>"
@@ -328,36 +320,32 @@ value next_token_fun find_id_kwd find_spe_kwd =
         a
     | [: :] -> len ]
   and maybe_end_quotation bp len =
-    parser [ [: `'>' :] -> len | [: a = quotation bp (store len '>') :] -> a ]
-  in
-  let rec next_token_loc =
-    parser bp
-    [ [: `' ' | '\010' | '\013' | '\t' | '\026' | '\012'; s :] -> next_token_loc s
-    | [: `'('; s :] -> maybe_comment bp s
-    | [: `'#'; _ = spaces_tabs; t = linenum bp :] -> t
-    | [: tok = next_token :] ep -> (tok, (bp, ep))
-    | [: _ = Stream.empty :] -> (("EOI", ""), (bp, succ bp)) ]
-  and maybe_comment bp =
     parser
-    [ [: `'*'; s :] -> do comment bp s; return next_token_loc s
-    | [: :] ep -> let tok = keyword_or_error (bp, ep) "(" in (tok, (bp, ep)) ]
+    [ [: `'>' :] -> len
+    | [: a = quotation bp (store len '>') :] -> a ]
+  and left_paren bp =
+    parser
+    [ [: `'*'; _ = comment bp; a = next_token :] -> a
+    | [: :] ep -> keyword_or_error (bp, ep) "(" ]
   and comment bp =
     parser
-    [ [: `'('; s :] -> maybe_nested_comment bp s
-    | [: `'*'; s :] -> maybe_end_comment bp s
+    [ [: `'('; s :] -> left_paren_in_comment bp s
+    | [: `'*'; s :] -> star_in_comment bp s
     | [: `c; s :] -> comment bp s
-    | [: :] ep -> err bp ep "comment not terminated" ]
-  and maybe_nested_comment bp =
+    | [: :] ep -> err (bp, ep) "comment not terminated" ]
+  and left_paren_in_comment bp =
     parser
-    [ [: `'*'; s :] -> do comment bp s; return comment bp s
+    [ [: `'*'; s :] -> do { comment bp s; comment bp s }
+    | [: a = comment bp :] -> a ]
+  and star_in_comment bp =
+    parser
+    [ [: `')' :] -> ()
     | [: a = comment bp :] -> a ]
-  and maybe_end_comment bp =
-    parser [ [: `')' :] -> () | [: a = comment bp :] -> a ]
   and linenum bp =
     parser
     [ [: `'0'..'9'; _ = digits; _ = spaces_tabs; `'"'; _ = any_to_nl; s :] ->
-        next_token_loc s
-    | [: :] -> (keyword_or_error (bp, bp + 1) "#", (bp, bp + 1)) ]
+        next_token s
+    | [: :] -> keyword_or_error (bp, bp + 1) "#" ]
   and spaces_tabs =
     parser
     [ [: `' ' | '\t'; s :] -> spaces_tabs s
@@ -373,88 +361,72 @@ value next_token_fun find_id_kwd find_spe_kwd =
     | [: :] -> () ]
   in
   fun cstrm ->
-    try next_token_loc cstrm with
+    try next_token cstrm with
     [ Stream.Error str ->
-        err (Stream.count cstrm) (Stream.count cstrm + 1) str ]
-;
-
-value locerr () = invalid_arg "Lexer: location function";
-value loct_create () = ref (Array.create 1024 None);
-value loct_func loct i =
-  match
-    if i < 0 || i >= Array.length loct.val then None
-    else Array.unsafe_get loct.val i
-  with
-  [ Some loc -> loc
-  | _ -> locerr () ]
-;
-value loct_add loct i loc =
-  do if i >= Array.length loct.val then
-       let new_tmax = Array.length loct.val * 2 in
-       let new_loct = Array.create new_tmax None in
-       do Array.blit loct.val 0 new_loct 0 (Array.length loct.val);
-          loct.val := new_loct;
-       return ()
-     else ();
-     loct.val.(i) := Some loc;
-  return ()
+        err (Stream.count cstrm, Stream.count cstrm + 1) str ]
 ;
 
 value func kwd_table =
   let find = Hashtbl.find kwd_table in
-  let lex cstrm =
-    let next_token_loc = next_token_fun find find in
-    let loct = loct_create () in
-    let ts =
-      Stream.from
-        (fun i ->
-           let (tok, loc) = next_token_loc cstrm in
-           do loct_add loct i loc; return Some tok)
-    in
-    let locf = loct_func loct in
-    (ts, locf)
-  in
-  lex
+  Token.lexer_func_of_parser (next_token_fun find)
 ;
 
 value rec check_keyword_stream =
   parser [: _ = check; _ = Stream.empty :] -> True
 and check =
   parser
-  [ [: `'A'..'Z' | 'a'..'z' | '\192'..'\214' | '\216'..'\246'
-       | '\248'..'\255'; s :] -> check_ident s
+  [ [: `'A'..'Z' | 'a'..'z' | '\192'..'\214' | '\216'..'\246' | '\248'..'\255'
+        ;
+       s :] ->
+      check_ident s
   | [: `'!' | '?' | '~' | '=' | '@' | '^' | '&' | '+' | '-' | '*' | '/' |
-        '%' | '.';
+        '%' | '.'
+        ;
        s :] ->
       check_ident2 s
   | [: `'<'; s :] ->
       match Stream.npeek 1 s with
-      [ [':' | '<']  -> ()
+      [ [':' | '<'] -> ()
       | _ -> check_ident2 s ]
   | [: `':';
        _ =
-         parser [ [: `']' | ':' | '=' | '>' :] -> () | [: :] -> () ] :] ep ->
+         parser
+         [ [: `']' | ':' | '=' | '>' :] -> ()
+         | [: :] -> () ] :] ep ->
       ()
   | [: `'>' | '|';
        _ =
-         parser [ [: `']' | '}' :] -> () | [: a = check_ident2 :] -> a ] :] ->
+         parser
+         [ [: `']' | '}' :] -> ()
+         | [: a = check_ident2 :] -> a ] :] ->
       ()
   | [: `'[' | '{'; s :] ->
       match Stream.npeek 2 s with
       [ ['<'; '<' | ':'] -> ()
       | _ ->
-          match s with parser [ [: `'|' | '<' | ':' :] -> () | [: :] -> () ] ]
-  | [: `';'; _ = parser [ [: `';' :] -> () | [: :] -> () ] :] -> ()
+          match s with parser
+          [ [: `'|' | '<' | ':' :] -> ()
+          | [: :] -> () ] ]
+  | [: `';';
+       _ =
+         parser
+         [ [: `';' :] -> ()
+         | [: :] -> () ] :] ->
+      ()
   | [: `_ :] -> () ]
 and check_ident =
   parser
   [ [: `'A'..'Z' | 'a'..'z' | '\192'..'\214' | '\216'..'\246' |
-        '\248'..'\255' | '0'..'9' | '_' | '''; s :] -> check_ident s
+        '\248'..'\255' | '0'..'9' | '_' | '''
+        ;
+       s :] ->
+      check_ident s
   | [: :] -> () ]
 and check_ident2 =
   parser
   [ [: `'!' | '?' | '~' | '=' | '@' | '^' | '&' | '+' | '-' | '*' | '/' |
-        '%' | '.' | ':' | '<' | '>' | '|';
+        '%' | '.' | ':' | '<' | '>' | '|'
+        ;
        s :] ->
       check_ident2 s
   | [: :] -> () ]
@@ -468,19 +440,27 @@ value using_token kwd_table (p_con, p_prm) =
   match p_con with
   [ "" ->
       try
-        let _ = Hashtbl.find kwd_table p_prm in ()
+        let _ = Hashtbl.find kwd_table p_prm in
+        ()
       with
       [ Not_found ->
           if check_keyword p_prm then Hashtbl.add kwd_table p_prm p_prm
           else
-            raise (Token.Error ("\
-the token \"" ^ p_prm ^ "\" does not respect Plexer rules")) ]
-  | "LIDENT" | "UIDENT" | "TILDEIDENT" | "TILDEIDENTCOLON"
-  | "QUESTIONIDENT" | "QUESTIONIDENTCOLON" | "INT" | "FLOAT"
-  | "CHAR" | "STRING" | "QUOTATION" | "ANTIQUOT" | "LOCATE" | "EOI" -> ()
+            raise
+              (Token.Error
+                 ("\
+the token \"" ^ p_prm ^
+                    "\" does not respect Plexer rules")) ]
+  | "LIDENT" | "UIDENT" | "TILDEIDENT" | "TILDEIDENTCOLON" | "QUESTIONIDENT" |
+    "QUESTIONIDENTCOLON" | "INT" | "FLOAT" | "CHAR" | "STRING" | "QUOTATION" |
+    "ANTIQUOT" | "LOCATE" | "EOI" ->
+      ()
   | _ ->
-      raise (Token.Error ("\
-the constructor \"" ^ p_con ^ "\" is not recognized by Plexer")) ]
+      raise
+        (Token.Error
+           ("\
+the constructor \"" ^ p_con ^
+              "\" is not recognized by Plexer")) ]
 ;
 
 value removing_token kwd_table (p_con, p_prm) =
@@ -520,15 +500,17 @@ value after_colon e =
   try
     let i = String.index e ':' in
     String.sub e (i + 1) (String.length e - i - 1)
-  with [ Not_found -> "" ]
+  with
+  [ Not_found -> "" ]
 ;
 
 value tparse =
   fun
   [ ("ANTIQUOT", p_prm) ->
       let p =
-        parser [: `("ANTIQUOT", prm) when eq_before_colon p_prm prm :] ->
-          after_colon prm
+        parser
+          [: `("ANTIQUOT", prm) when eq_before_colon p_prm prm :] ->
+            after_colon prm
       in
       Some p
   | _ -> None ]
