@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: stdpp.mli,v 2.0 1998/12/01 08:21:07 ddr Exp $ *)
+(* $Id: stdpp.mli,v 3.1 2001/05/29 04:50:55 ddr Exp $ *)
 
 (* Module [Stdpp]: standard definitions *)
 
@@ -24,3 +24,8 @@ exception Exc_located of (int * int) and exn;
 value raise_with_loc : (int * int) -> exn -> 'a;
     (* [raise_with_loc loc e], if [e] is already the exception [Exc_located],
        re-raise it, else raise the exception [Exc_located loc e]. *)
+
+value loc_name : ref string;
+    (* Name of the location variable used in grammars and in the predefined
+       quotations for OCaml syntax trees. Default: [loc] *)
+
