@@ -5,36 +5,57 @@
 (*                                                                     *)
 (*        Daniel de Rauglaudre, projet Cristal, INRIA Rocquencourt     *)
 (*                                                                     *)
-(*  Copyright 1998 Institut National de Recherche en Informatique et   *)
+(*  Copyright 2001 Institut National de Recherche en Informatique et   *)
 (*  Automatique.  Distributed only by permission.                      *)
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: pa_r.ml,v 2.5 2000/04/18 12:25:43 ddr Exp $ *)
+(* $Id: pa_r.ml,v 3.15 2001/07/13 13:56:53 ddr Exp $ *)
 
 open Stdpp;
 open Pcaml;
 
 Pcaml.no_constructors_arity.val := False;
 
-do Grammar.Unsafe.reinit_gram gram (Plexer.make ());
-   Grammar.Unsafe.clear_entry interf;
-   Grammar.Unsafe.clear_entry implem;
-   Grammar.Unsafe.clear_entry top_phrase;
-   Grammar.Unsafe.clear_entry use_file;
-   Grammar.Unsafe.clear_entry module_type;
-   Grammar.Unsafe.clear_entry module_expr;
-   Grammar.Unsafe.clear_entry sig_item;
-   Grammar.Unsafe.clear_entry str_item;
-   Grammar.Unsafe.clear_entry expr;
-   Grammar.Unsafe.clear_entry patt;
-   Grammar.Unsafe.clear_entry ctyp;
-   Grammar.Unsafe.clear_entry let_binding;
-   Grammar.Unsafe.clear_entry class_type;
-   Grammar.Unsafe.clear_entry class_expr;
-   Grammar.Unsafe.clear_entry class_sig_item;
-   Grammar.Unsafe.clear_entry class_str_item;
-return ();
+value help_sequences () =
+  do {
+    Printf.eprintf "\
+New syntax:
+     do {e1; e2; ... ; en}
+     while e do {e1; e2; ... ; en}
+     for v = v1 to/downto v2 do {e1; e2; ... ; en}
+Old (discouraged) syntax:
+     do e1; e2; ... ; en-1; return en
+     while e do e1; e2; ... ; en; done
+     for v = v1 to/downto v2 do e1; e2; ... ; en; done
+To avoid compilation warning use the new syntax.
+";
+    flush stderr;
+    exit 1
+  }
+;
+Pcaml.add_option "-help_seq" (Arg.Unit help_sequences)
+  "    Print explanations about new sequences and exit.";
+
+do {
+  Grammar.Unsafe.reinit_gram gram (Plexer.make ());
+  Grammar.Unsafe.clear_entry interf;
+  Grammar.Unsafe.clear_entry implem;
+  Grammar.Unsafe.clear_entry top_phrase;
+  Grammar.Unsafe.clear_entry use_file;
+  Grammar.Unsafe.clear_entry module_type;
+  Grammar.Unsafe.clear_entry module_expr;
+  Grammar.Unsafe.clear_entry sig_item;
+  Grammar.Unsafe.clear_entry str_item;
+  Grammar.Unsafe.clear_entry expr;
+  Grammar.Unsafe.clear_entry patt;
+  Grammar.Unsafe.clear_entry ctyp;
+  Grammar.Unsafe.clear_entry let_binding;
+  Grammar.Unsafe.clear_entry class_type;
+  Grammar.Unsafe.clear_entry class_expr;
+  Grammar.Unsafe.clear_entry class_sig_item;
+  Grammar.Unsafe.clear_entry class_str_item
+};
 
 value o2b =
   fun
@@ -45,14 +66,16 @@ value o2b =
 value mkumin loc f arg =
   match arg with
   [ <:expr< $int:n$ >> when int_of_string n > 0 ->
-      let n = "-" ^ n in <:expr< $int:n$ >>
+      let n = "-" ^ n in
+      <:expr< $int:n$ >>
   | <:expr< $flo:n$ >> when float_of_string n > 0.0 ->
-      let n = "-" ^ n in <:expr< $flo:n$ >>
-  | _ -> let f = "~" ^ f in <:expr< $lid:f$ $arg$ >> ]
+      let n = "-" ^ n in
+      <:expr< $flo:n$ >>
+  | _ ->
+      let f = "~" ^ f in
+      <:expr< $lid:f$ $arg$ >> ]
 ;
 
-external loc_of_node : 'a -> (int * int) = "%field0";
-
 value mklistexp loc last =
   loop True where rec loop top =
     fun
@@ -61,7 +84,9 @@ value mklistexp loc last =
         [ Some e -> e
         | None -> <:expr< [] >> ]
     | [e1 :: el] ->
-        let loc = if top then loc else (fst (loc_of_node e1), snd loc) in
+        let loc =
+          if top then loc else (fst (MLast.loc_of_expr e1), snd loc)
+        in
         <:expr< [$e1$ :: $loop False el$] >> ]
 ;
 
@@ -73,42 +98,71 @@ value mklistpat loc last =
         [ Some p -> p
         | None -> <:patt< [] >> ]
     | [p1 :: pl] ->
-        let loc = if top then loc else (fst (loc_of_node p1), snd loc) in
+        let loc =
+          if top then loc else (fst (MLast.loc_of_patt p1), snd loc)
+        in
         <:patt< [$p1$ :: $loop False pl$] >> ]
 ;
 
+(* ...suppose to flush the input in case of syntax error to avoid multiple
+   errors in case of cut-and-paste in the xterm, but work bad: for example
+   the input "for x = 1;" waits for another line before displaying the
+   error...
+value rec sync cs =
+  match cs with parser
+  [ [: `';' :] -> sync_semi cs
+  | [: `_ :] -> sync cs ]
+and sync_semi cs =
+  match Stream.peek cs with 
+  [ Some ('\010' | '\013') -> ()
+  | _ -> sync cs ]
+;
+Pcaml.sync.val := sync;
+*)
+
 value neg s = string_of_int (- int_of_string s);
 
 value type_parameter = Grammar.Entry.create gram "type_parameter";
 value fun_binding = Grammar.Entry.create gram "fun_binding";
 value ipatt = Grammar.Entry.create gram "ipatt";
+value direction_flag = Grammar.Entry.create gram "direction_flag";
 
 EXTEND
   GLOBAL: interf implem top_phrase use_file sig_item str_item ctyp patt expr
-    module_type module_expr let_binding
-    type_parameter fun_binding ipatt;
-
+    module_type module_expr let_binding type_parameter fun_binding ipatt
+    direction_flag;
   interf:
-    [ [ st = LIST0 [ s = sig_item; ";" -> (s, loc) ]; EOI -> st ] ]
+    [ [ si = sig_item_semi; (sil, stopped) = SELF -> ([si :: sil], stopped)
+      | "#"; n = LIDENT; dp = OPT expr; ";" ->
+          ([(<:sig_item< # $n$ $opt:dp$ >>, loc)], True)
+      | EOI -> ([], False) ] ]
+  ;
+  sig_item_semi:
+    [ [ si = sig_item; ";" -> (si, loc) ] ]
   ;
   implem:
-    [ [ st = LIST0 [ s = str_item; ";" -> (s, loc) ]; EOI -> st ] ]
+    [ [ si = str_item_semi; (sil, stopped) = SELF -> ([si :: sil], stopped)
+      | "#"; n = LIDENT; dp = OPT expr; ";" ->
+          ([(<:str_item< # $n$ $opt:dp$ >>, loc)], True)
+      | EOI -> ([], False) ] ]
+  ;
+  str_item_semi:
+    [ [ si = str_item; ";" -> (si, loc) ] ]
   ;
   top_phrase:
-    [ [ ph = phrase -> Some ph | EOI -> None ] ]
+    [ [ ph = phrase -> Some ph
+      | EOI -> None ] ]
   ;
   use_file:
-    [ [ l = LIST0 phrase; EOI -> l ] ]
+    [ [ si = str_item; ";"; (sil, stopped) = SELF -> ([si :: sil], stopped)
+      | "#"; n = LIDENT; dp = OPT expr; ";" ->
+          ([<:str_item< # $n$ $opt:dp$ >>], True)
+      | EOI -> ([], False) ] ]
   ;
   phrase:
-    [ [ sti = str_item; ";" -> MLast.PhStr loc sti
-      | "#"; n = LIDENT; dp = dir_param; ";" -> MLast.PhDir loc n dp ] ]
-  ;
-  dir_param:
-    [ [ -> MLast.DpNon
-      | s = STRING -> MLast.DpStr s
-      | i = INT -> MLast.DpInt i
-      | sl = mod_ident -> MLast.DpIde sl ] ]
+    [ [ sti = str_item; ";" -> sti
+      | "#"; n = LIDENT; dp = OPT expr; ";" ->
+          <:str_item< # $n$ $opt:dp$ >> ] ]
   ;
   module_expr:
     [ [ "functor"; "("; i = UIDENT; ":"; t = module_type; ")"; "->";
@@ -131,6 +185,7 @@ EXTEND
           <:str_item< exception $c$ of $list:tl$ >>
       | "external"; i = LIDENT; ":"; t = ctyp; "="; pd = LIST1 STRING ->
           <:str_item< external $i$ : $t$ = $list:pd$ >>
+      | "include"; me = module_expr -> <:str_item< include $me$ >>
       | "module"; i = UIDENT; mb = module_binding ->
           <:str_item< module $i$ = $mb$ >>
       | "module"; "type"; i = UIDENT; "="; mt = module_type ->
@@ -171,8 +226,7 @@ EXTEND
           <:sig_item< exception $c$ of $list:tl$ >>
       | "external"; i = LIDENT; ":"; t = ctyp; "="; pd = LIST1 STRING ->
           <:sig_item< external $i$ : $t$ = $list:pd$ >>
-      | "include"; mt = module_type ->
-          <:sig_item< include $mt$ >>
+      | "include"; mt = module_type -> <:sig_item< include $mt$ >>
       | "module"; i = UIDENT; mt = module_declaration ->
           <:sig_item< module $i$ : $mt$ >>
       | "module"; "type"; i = UIDENT; "="; mt = module_type ->
@@ -190,8 +244,8 @@ EXTEND
           <:module_type< functor ( $i$ : $t$ ) -> $mt$ >> ] ]
   ;
   with_constr:
-    [ [ "type"; i = mod_ident; tp = LIST0 type_parameter; "="; t = ctyp ->
-          MLast.WcTyp loc i tp t
+    [ [ "type"; i = mod_ident; tpl = LIST0 type_parameter; "="; t = ctyp ->
+          MLast.WcTyp loc i tpl t
       | "module"; i = mod_ident; "="; mt = module_type ->
           MLast.WcMod loc i mt ] ]
   ;
@@ -215,13 +269,15 @@ EXTEND
           <:expr< try $x$ with $p$ -> $e$ >>
       | "if"; e1 = SELF; "then"; e2 = SELF; "else"; e3 = SELF ->
           <:expr< if $e1$ then $e2$ else $e3$ >>
-      | "do"; seq = LIST0 [ e = expr; ";" -> e ]; "return"; e = SELF ->
-          <:expr< do $list:seq$ return $e$ >>
+      | "do"; "{"; seq = sequence; "}" ->
+          match seq with
+          [ [e] -> e
+          | _ -> <:expr< do { $list:seq$ } >> ]
       | "for"; i = LIDENT; "="; e1 = SELF; df = direction_flag; e2 = SELF;
-        "do"; seq = LIST0 [ e = expr; ";" -> e ]; "done" ->
-          <:expr< for $i$ = $e1$ $to:df$ $e2$ do $list:seq$ done >>
-      | "while"; e = SELF; "do"; seq = LIST0 [ e = expr; ";" -> e ]; "done" ->
-          <:expr< while $e$ do $list:seq$ done >> ]
+        "do"; "{"; el = sequence; "}" ->
+          <:expr< for $i$ = $e1$ $to:df$ $e2$ do { $list:el$ } >>
+      | "while"; e = SELF; "do"; "{"; el = sequence; "}" ->
+          <:expr< while $e$ do { $list:el$ } >> ]
     | "where"
       [ e = SELF; "where"; rf = OPT "rec"; lb = let_binding ->
           <:expr< let $rec:o2b rf$ $list:[lb]$ in $e$ >> ]
@@ -232,83 +288,50 @@ EXTEND
     | "&&" RIGHTA
       [ e1 = SELF; f = "&&"; e2 = SELF -> <:expr< $lid:f$ $e1$ $e2$ >> ]
     | "<" LEFTA
-      [ e1 = SELF;
-        f =
-          [ op = "<" -> op
-          | op = ">" -> op
-          | op = "<=" -> op
-          | op = ">=" -> op
-          | op = "=" -> op
-          | op = "<>" -> op
-          | op = "==" -> op
-          | op = "!=" -> op ];
+      [ e1 = SELF; f = [ "<" | ">" | "<=" | ">=" | "=" | "<>" | "==" | "!=" ];
         e2 = SELF ->
           <:expr< $lid:f$ $e1$ $e2$ >> ]
     | "^" RIGHTA
-      [ e1 = SELF; f = [ op = "^" -> op | op = "@" -> op ]; e2 = SELF ->
+      [ e1 = SELF; f = [ "^" | "@" ]; e2 = SELF ->
           <:expr< $lid:f$ $e1$ $e2$ >> ]
     | "+" LEFTA
-      [ e1 = SELF;
-        f =
-          [ op = "+" -> op
-          | op = "-" -> op
-          | op = "+." -> op
-          | op = "-." -> op ];
-        e2 = SELF ->
+      [ e1 = SELF; f = [ "+" | "-" | "+." | "-." ]; e2 = SELF ->
           <:expr< $lid:f$ $e1$ $e2$ >> ]
     | "*" LEFTA
       [ e1 = SELF;
-        f =
-          [ op = "*" -> op
-          | op = "/" -> op
-          | op = "*." -> op
-          | op = "/." -> op
-          | op = "land" -> op
-          | op = "lor" -> op
-          | op = "lxor" -> op
-          | op = "mod" -> op ];
+        f = [ "*" | "/" | "*." | "/." | "land" | "lor" | "lxor" | "mod" ];
         e2 = SELF ->
           <:expr< $lid:f$ $e1$ $e2$ >> ]
     | "**" RIGHTA
-      [ e1 = SELF;
-        f =
-          [ op = "**" -> op
-          | op = "asr" -> op
-          | op = "lsl" -> op
-          | op = "lsr" -> op ];
-        e2 = SELF ->
+      [ e1 = SELF; f = [ "**" | "asr" | "lsl" | "lsr" ]; e2 = SELF ->
           <:expr< $lid:f$ $e1$ $e2$ >> ]
     | "unary minus" NONA
-      [ f = [ op = "-" -> op | op = "-." -> op ]; e = SELF ->
-          <:expr< $mkumin loc f e$ >> ]
+      [ f = [ "-" | "-." ]; e = SELF -> <:expr< $mkumin loc f e$ >> ]
     | "apply" LEFTA
       [ e1 = SELF; e2 = SELF -> <:expr< $e1$ $e2$ >>
-      | "assert"; e = expr ->
+      | "assert"; e = SELF ->
           let f = <:expr< $str:input_file.val$ >> in
           let bp = <:expr< $int:string_of_int (fst loc)$ >> in
           let ep = <:expr< $int:string_of_int (snd loc)$ >> in
-          let raiser =
-            <:expr< raise (Pervasives.Assert_failure ($f$, $bp$, $ep$)) >>
-          in
+          let raiser = <:expr< raise (Assert_failure ($f$, $bp$, $ep$)) >> in
           match e with
           [ <:expr< False >> -> raiser
           | _ ->
               if no_assert.val then <:expr< () >>
               else <:expr< if $e$ then () else $raiser$ >> ]
-      | "lazy"; e = expr ->
+      | "lazy"; e = SELF ->
           <:expr< Pervasives.ref (Lazy.Delayed (fun () -> $e$)) >> ]
     | "." LEFTA
       [ e1 = SELF; "."; "("; e2 = SELF; ")" -> <:expr< $e1$ .( $e2$ ) >>
       | e1 = SELF; "."; "["; e2 = SELF; "]" -> <:expr< $e1$ .[ $e2$ ] >>
       | e1 = SELF; "."; e2 = SELF -> <:expr< $e1$ . $e2$ >> ]
     | "~-" NONA
-      [ f = [ op = "~-" -> op | op = "~-." -> op ]; e = SELF ->
-          <:expr< $lid:f$ $e$ >> ]
+      [ f = [ "~-" | "~-." ]; e = SELF -> <:expr< $lid:f$ $e$ >> ]
     | "simple"
       [ s = INT -> <:expr< $int:s$ >>
       | s = FLOAT -> <:expr< $flo:s$ >>
       | s = STRING -> <:expr< $str:s$ >>
-      | c = CHAR -> <:expr< $chr:c.[0]$ >>
+      | s = CHAR -> <:expr< $chr:s$ >>
       | i = expr_ident -> i
       | "["; "]" -> <:expr< [] >>
       | "["; el = LIST1 expr SEP ";"; last = OPT [ "::"; e = expr -> e ];
@@ -316,8 +339,9 @@ EXTEND
           <:expr< $mklistexp loc last el$ >>
       | "[|"; el = LIST0 expr SEP ";"; "|]" -> <:expr< [| $list:el$ |] >>
       | "{"; lel = LIST1 label_expr SEP ";"; "}" -> <:expr< { $list:lel$ } >>
-      | "{"; "("; e = expr; ")"; "with"; lel = LIST1 label_expr SEP ";";
-        "}" -> <:expr< { ($e$) with $list:lel$ } >>
+      | "{"; "("; e = SELF; ")"; "with"; lel = LIST1 label_expr SEP ";";
+        "}" ->
+          <:expr< { ($e$) with $list:lel$ } >>
       | "("; ")" -> <:expr< () >>
       | "("; e = SELF; ":"; t = ctyp; ")" -> <:expr< ($e$ : $t$) >>
       | "("; e = SELF; ","; el = LIST1 expr SEP ","; ")" ->
@@ -347,6 +371,19 @@ EXTEND
   dummy:
     [ [ -> () ] ]
   ;
+  sequence:
+    [ [ "let"; o = OPT "rec"; l = LIST1 let_binding SEP "and"; "in";
+        el = SELF ->
+          let e =
+            match el with
+            [ [e] -> e
+            | _ -> <:expr< do { $list:el$ } >> ]
+          in
+          [<:expr< let $rec:o2b o$ $list:l$ in $e$ >>]
+      | e = expr; ";"; el = SELF -> [e :: el]
+      | e = expr; ";" -> [e]
+      | e = expr -> [e] ] ]
+  ;
   let_binding:
     [ [ p = ipatt; e = fun_binding -> (p, e) ] ]
   ;
@@ -367,17 +404,19 @@ EXTEND
           (p, w, e) ] ]
   ;
   label_expr:
-    [ [ i = expr_ident; "="; e = expr -> (i, e) ] ]
+    [ [ i = patt_label_ident; e = fun_binding -> (i, e) ] ]
   ;
   expr_ident:
     [ RIGHTA
       [ i = LIDENT -> <:expr< $lid:i$ >>
       | i = UIDENT -> <:expr< $uid:i$ >>
-      | m = UIDENT; "."; i = expr_ident ->
-          loop <:expr< $uid:m$ >> i where rec loop m =
+      | m = UIDENT; "."; i = SELF ->
+          let rec loop m =
             fun
             [ <:expr< $x$ . $y$ >> -> loop <:expr< $m$ . $x$ >> y
-            | e -> <:expr< $m$ . $e$ >> ] ] ]
+            | e -> <:expr< $m$ . $e$ >> ]
+          in
+          loop <:expr< $uid:m$ >> i ] ]
   ;
   fun_def:
     [ RIGHTA
@@ -398,8 +437,9 @@ EXTEND
       | s = UIDENT -> <:patt< $uid:s$ >>
       | s = INT -> <:patt< $int:s$ >>
       | "-"; s = INT -> <:patt< $int:neg s$ >>
+      | s = FLOAT -> <:patt< $flo:s$ >>
       | s = STRING -> <:patt< $str:s$ >>
-      | s = CHAR -> <:patt< $chr:s.[0]$ >>
+      | s = CHAR -> <:patt< $chr:s$ >>
       | "["; "]" -> <:patt< [] >>
       | "["; pl = LIST1 patt SEP ";"; last = OPT [ "::"; p = patt -> p ];
         "]" ->
@@ -467,7 +507,9 @@ EXTEND
     [ [ "constraint"; t1 = ctyp; "="; t2 = ctyp -> (t1, t2) ] ]
   ;
   type_parameter:
-    [ [ "'"; i = ident -> i ] ]
+    [ [ "'"; i = ident -> (i, (False, False))
+      | "+"; "'"; i = ident -> (i, (True, False))
+      | "-"; "'"; i = ident -> (i, (False, True)) ] ]
   ;
   ctyp:
     [ LEFTA
@@ -501,7 +543,8 @@ EXTEND
     [ [ i = LIDENT; ":"; mf = OPT "mutable"; t = ctyp -> (i, o2b mf, t) ] ]
   ;
   ident:
-    [ [ i = LIDENT -> i | i = UIDENT -> i ] ]
+    [ [ i = LIDENT -> i
+      | i = UIDENT -> i ] ]
   ;
   mod_ident:
     [ RIGHTA
@@ -510,16 +553,16 @@ EXTEND
       | m = UIDENT; "."; i = SELF -> [m :: i] ] ]
   ;
   direction_flag:
-    [ [ "to" -> True | "downto" -> False ] ]
+    [ [ "to" -> True
+      | "downto" -> False ] ]
   ;
 END;
 
 (* Objects and Classes *)
 
 EXTEND
-  GLOBAL: str_item sig_item expr ctyp
-    class_sig_item class_str_item class_type class_expr;
-
+  GLOBAL: str_item sig_item expr ctyp class_sig_item class_str_item class_type
+    class_expr;
   str_item:
     [ [ "class"; cd = LIST1 class_declaration SEP "and" ->
           <:str_item< class $list:cd$ >>
@@ -532,9 +575,6 @@ EXTEND
       | "class"; "type"; ctd = LIST1 class_type_declaration SEP "and" ->
           <:sig_item< class type $list:ctd$ >> ] ]
   ;
-
-  (* Class expressions *)
-
   class_declaration:
     [ [ vf = OPT "virtual"; i = LIDENT; ctp = class_type_parameters;
         cfb = class_fun_binding ->
@@ -545,8 +585,7 @@ EXTEND
     [ [ "="; ce = class_expr -> ce
       | ":"; ct = class_type; "="; ce = class_expr ->
           <:class_expr< ($ce$ : $ct$) >>
-      | p = ipatt; cfb = class_fun_binding ->
-          <:class_expr< fun $p$ -> $cfb$ >> ] ]
+      | p = ipatt; cfb = SELF -> <:class_expr< fun $p$ -> $cfb$ >> ] ]
   ;
   class_type_parameters:
     [ [ -> (loc, [])
@@ -555,32 +594,30 @@ EXTEND
   class_fun_def:
     [ [ p = patt LEVEL "simple"; "->"; ce = class_expr ->
           <:class_expr< fun $p$ -> $ce$ >>
-      | p = patt LEVEL "simple"; cfd = class_fun_def ->
+      | p = patt LEVEL "simple"; cfd = SELF ->
           <:class_expr< fun $p$ -> $cfd$ >> ] ]
-  ;      
+  ;
   class_expr:
     [ "top"
       [ "fun"; cfd = class_fun_def -> cfd
       | "let"; rf = OPT "rec"; lb = LIST1 let_binding SEP "and"; "in";
-        ce = class_expr ->
+        ce = SELF ->
           <:class_expr< let $rec:o2b rf$ $list:lb$ in $ce$ >> ]
     | "apply" NONA
-      [ ce = class_expr; sel = LIST1 (expr LEVEL "simple") ->
+      [ ce = SELF; sel = LIST1 expr LEVEL "simple" ->
           <:class_expr< $ce$ $list:sel$ >> ]
     | "simple"
-      [ ci = class_longident;
-        "["; ct = ctyp; ","; ctcl = LIST1 ctyp SEP ","; "]" ->
+      [ ci = class_longident; "["; ct = ctyp; ","; ctcl = LIST1 ctyp SEP ",";
+        "]" ->
           <:class_expr< $list:ci$ [ $ct$ , $list:ctcl$ ] >>
       | ci = class_longident; "["; ct = ctyp; "]" ->
           <:class_expr< $list:ci$ [ $ct$ ] >>
-      | ci = class_longident ->
-          <:class_expr< $list:ci$ >>
+      | ci = class_longident -> <:class_expr< $list:ci$ >>
       | "object"; cspo = OPT class_self_patt; cf = class_structure; "end" ->
           <:class_expr< object $cspo$ $list:cf$ end >>
-      | "("; ce = class_expr; ":"; ct = class_type; ")" ->
+      | "("; ce = SELF; ":"; ct = class_type; ")" ->
           <:class_expr< ($ce$ : $ct$) >>
-      | "("; ce = class_expr; ")" ->
-          ce ] ]
+      | "("; ce = SELF; ")" -> ce ] ]
   ;
   class_structure:
     [ [ cf = LIST0 [ cf = class_str_item; ";" -> cf ] -> cf ] ]
@@ -592,7 +629,7 @@ EXTEND
   class_str_item:
     [ [ "inherit"; ce = class_expr; pb = OPT [ "as"; i = LIDENT -> i ] ->
           <:class_str_item< inherit $ce$ $as:pb$ >>
-      | "value"; (lab, mf, e)  = cvalue ->
+      | "value"; (lab, mf, e) = cvalue ->
           <:class_str_item< value $mut:mf$ $lab$ = $e$ >>
       | "method"; "private"; "virtual"; l = label; ":"; t = ctyp ->
           <:class_str_item< method private virtual $l$ : $t$ >>
@@ -604,12 +641,10 @@ EXTEND
           <:class_str_item< method $l$ = $fb$ >>
       | "type"; t1 = ctyp; "="; t2 = ctyp ->
           <:class_str_item< type $t1$ = $t2$ >>
-      | "initializer"; se = expr ->
-          <:class_str_item< initializer $se$ >> ] ]
+      | "initializer"; se = expr -> <:class_str_item< initializer $se$ >> ] ]
   ;
   cvalue:
-    [ [ mf = OPT "mutable"; l = label; "="; e = expr ->
-          (l, o2b mf, e)
+    [ [ mf = OPT "mutable"; l = label; "="; e = expr -> (l, o2b mf, e)
       | mf = OPT "mutable"; l = label; ":"; t = ctyp; "="; e = expr ->
           (l, o2b mf, <:expr< ($e$ : $t$) >>)
       | mf = OPT "mutable"; l = label; ":"; t1 = ctyp; ":>"; t2 = ctyp; "=";
@@ -621,19 +656,15 @@ EXTEND
   label:
     [ [ i = LIDENT -> i ] ]
   ;
-
-  (* Class types *)
-
   class_type:
-    [ [ "["; t = ctyp; "]"; "->"; ct = class_type ->
+    [ [ "["; t = ctyp; "]"; "->"; ct = SELF ->
           <:class_type< [ $t$ ] -> $ct$ >>
       | "["; t = ctyp; ","; tl = LIST1 ctyp SEP ","; "]";
         id = clty_longident ->
           <:class_type< [ $t$ , $list:tl$ ] $list:id$ >>
       | "["; t = ctyp; "]"; id = clty_longident ->
           <:class_type< [ $t$ ] $list:id$ >>
-      | id = clty_longident ->
-          <:class_type< $list:id$ >>
+      | id = clty_longident -> <:class_type< $list:id$ >>
       | "object"; cst = OPT class_self_type;
         csf = LIST0 [ csf = class_sig_item; ";" -> csf ]; "end" ->
           <:class_type< object $cst$ $list:csf$ end >> ] ]
@@ -642,8 +673,7 @@ EXTEND
     [ [ "("; t = ctyp; ")" -> t ] ]
   ;
   class_sig_item:
-    [ [ "inherit"; cs = class_type ->
-          <:class_sig_item< inherit $cs$ >>
+    [ [ "inherit"; cs = class_type -> <:class_sig_item< inherit $cs$ >>
       | "value"; mf = OPT "mutable"; l = label; ":"; t = ctyp ->
           <:class_sig_item< value $mut:o2b mf$ $l$ : $t$ >>
       | "method"; "private"; "virtual"; l = label; ":"; t = ctyp ->
@@ -669,41 +699,32 @@ EXTEND
           {MLast.ciLoc = loc; MLast.ciVir = o2b vf; MLast.ciPrm = ctp;
            MLast.ciNam = n; MLast.ciExp = cs} ] ]
   ;
-
-  (* Expressions *)
-
   expr: LEVEL "apply"
     [ LEFTA
       [ "new"; i = class_longident -> <:expr< new $list:i$ >> ] ]
   ;
   expr: LEVEL "."
-    [ [ e = expr; "#"; lab = label -> <:expr< $e$ # $lab$ >> ] ]
+    [ [ e = SELF; "#"; lab = label -> <:expr< $e$ # $lab$ >> ] ]
   ;
   expr: LEVEL "simple"
-    [ [ "("; e = expr; ":"; t1 = ctyp; ":>"; t2 = ctyp; ")" ->
+    [ [ "("; e = SELF; ":"; t1 = ctyp; ":>"; t2 = ctyp; ")" ->
           <:expr< ($e$ : $t1$ :> $t2$ ) >>
-      | "("; e = expr; ":>"; t = ctyp; ")" ->
-          <:expr< ($e$ :> $t$) >>
-      | "{<"; ">}" ->
-          <:expr< {< >} >>
-      | "{<"; fel = field_expr_list; ">}" ->
-          <:expr< {< $list:fel$ >} >> ] ]
+      | "("; e = SELF; ":>"; t = ctyp; ")" -> <:expr< ($e$ :> $t$) >>
+      | "{<"; ">}" -> <:expr< {< >} >>
+      | "{<"; fel = field_expr_list; ">}" -> <:expr< {< $list:fel$ >} >> ] ]
   ;
   field_expr_list:
-    [ [ l = label; "="; e = expr; ";"; fel = field_expr_list -> [(l, e) :: fel]
+    [ [ l = label; "="; e = expr; ";"; fel = SELF -> [(l, e) :: fel]
       | l = label; "="; e = expr; ";" -> [(l, e)]
       | l = label; "="; e = expr -> [(l, e)] ] ]
   ;
-
-  (* Core types *)
-
   ctyp: LEVEL "simple"
     [ [ "#"; id = class_longident -> <:ctyp< # $list:id$ >>
       | "<"; (ml, v) = meth_list; ">" -> <:ctyp< < $list:ml$ $v$ > >>
       | "<"; ">" -> <:ctyp< < > >> ] ]
   ;
   meth_list:
-    [ [ f = field; ";"; (ml, v) = meth_list -> ([f :: ml], v)
+    [ [ f = field; ";"; (ml, v) = SELF -> ([f :: ml], v)
       | f = field; ";" -> ([f], False)
       | f = field -> ([f], False)
       | ".." -> ([], True) ] ]
@@ -711,15 +732,12 @@ EXTEND
   field:
     [ [ lab = LIDENT; ":"; t = ctyp -> (lab, t) ] ]
   ;
-
-  (* Identifiers *)
-
   clty_longident:
-    [ [ m = UIDENT; "."; l = clty_longident -> [m :: l]
+    [ [ m = UIDENT; "."; l = SELF -> [m :: l]
       | i = LIDENT -> [i] ] ]
   ;
   class_longident:
-    [ [ m = UIDENT; "."; l = class_longident -> [m :: l]
+    [ [ m = UIDENT; "."; l = SELF -> [m :: l]
       | i = LIDENT -> [i] ] ]
   ;
 END;
@@ -730,8 +748,8 @@ EXTEND
   GLOBAL: ctyp ipatt patt expr;
   ctyp: AFTER "arrow"
     [ NONA
-      [ i = TILDEIDENTCOLON; t = ctyp -> <:ctyp< ~ $i$ : $t$ >>
-      | i = QUESTIONIDENTCOLON; t = ctyp -> <:ctyp< ? $i$ : $t$ >>  ] ]
+      [ i = TILDEIDENTCOLON; t = SELF -> <:ctyp< ~ $i$ : $t$ >>
+      | i = QUESTIONIDENTCOLON; t = SELF -> <:ctyp< ? $i$ : $t$ >> ] ]
   ;
   ctyp: LEVEL "simple"
     [ [ "[|"; rfl = LIST0 row_field SEP "|"; "|]" ->
@@ -749,37 +767,65 @@ EXTEND
   row_field_list_dd:
     [ [ rf = row_field -> ([rf], True)
       | rf = row_field; "|"; ".." -> ([rf], False)
-      | rf = row_field; "|"; (rfl, dd) = row_field_list_dd ->
-          ([rf :: rfl], dd) ] ]
+      | rf = row_field; "|"; (rfl, dd) = SELF -> ([rf :: rfl], dd) ] ]
   ;
   patt: LEVEL "simple"
     [ [ "`"; s = ident -> <:patt< ` $s$ >> ] ]
   ;
   ipatt:
-    [ [ i = TILDEIDENTCOLON; p = ipatt ->
-          <:patt< ~ $i$ : $p$ >>
-      | i = TILDEIDENT ->
-          <:patt< ~ $i$ >>
-      | i = QUESTIONIDENTCOLON; j = LIDENT ->
-          <:patt< ? $i$ : $lid:j$ >>
+    [ [ i = TILDEIDENTCOLON; p = SELF -> <:patt< ~ $i$ : $p$ >>
+      | i = TILDEIDENT -> <:patt< ~ $i$ >>
+      | i = QUESTIONIDENTCOLON; j = LIDENT -> <:patt< ? $i$ : $lid:j$ >>
       | i = QUESTIONIDENTCOLON; "("; j = LIDENT; "="; e = expr; ")" ->
           <:patt< ? $i$ : ( $lid:j$ = $e$ ) >>
-      | i = QUESTIONIDENT ->
-          <:patt< ? $i$ : $lid:i$ >>
+      | i = QUESTIONIDENT -> <:patt< ? $i$ : $lid:i$ >>
       | "?"; "("; i = LIDENT; "="; e = expr; ")" ->
           <:patt< ? ( $i$ = $e$ ) >> ] ]
   ;
   expr: AFTER "apply"
     [ "label"
-      [ i = TILDEIDENTCOLON; e = expr -> <:expr< ~ $i$ : $e$ >>
+      [ i = TILDEIDENTCOLON; e = SELF -> <:expr< ~ $i$ : $e$ >>
       | i = TILDEIDENT -> <:expr< ~ $i$ >>
-      | i = QUESTIONIDENTCOLON; e = expr -> <:expr< ? $i$ : $e$ >>
+      | i = QUESTIONIDENTCOLON; e = SELF -> <:expr< ? $i$ : $e$ >>
       | i = QUESTIONIDENT -> <:expr< ? $i$ >> ] ]
   ;
   expr: LEVEL "simple"
     [ [ "`"; s = ident -> <:expr< ` $s$ >> ] ]
   ;
   ident:
-    [ [ i = LIDENT -> i | i = UIDENT -> i ] ]
+    [ [ i = LIDENT -> i
+      | i = UIDENT -> i ] ]
+  ;
+END;
+
+(* Old syntax for sequences *)
+
+value not_yet_warned = ref False;
+value warning_seq () =
+  if not_yet_warned.val then do {
+    not_yet_warned.val := False;
+    Printf.eprintf "\
+*** warning: use of old syntax; do \"camlp4r -help_seq\" for explanations
+";
+    flush stderr
+  }
+  else ()
+;
+Pcaml.add_option "-warn_seq" (Arg.Set not_yet_warned)
+  "    Warn when using old syntax for sequences.";
+
+EXTEND
+  GLOBAL: expr direction_flag;
+  expr: LEVEL "top"
+    [ [ "do"; seq = LIST0 [ e = expr; ";" -> e ]; "return"; e = SELF ->
+          do { warning_seq (); <:expr< do { $list:seq @ [e]$ } >> }
+      | "for"; i = LIDENT; "="; e1 = SELF; df = direction_flag; e2 = SELF;
+        "do"; seq = LIST0 [ e = expr; ";" -> e ]; "done" ->
+          do {
+            warning_seq ();
+            <:expr< for $i$ = $e1$ $to:df$ $e2$ do { $list:seq$ } >>
+          }
+      | "while"; e = SELF; "do"; seq = LIST0 [ e = expr; ";" -> e ]; "done" ->
+          do { warning_seq (); <:expr< while $e$ do { $list:seq$ } >> } ] ]
   ;
 END;
