@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: pa_rp.ml,v 2.2 2000/04/12 22:39:46 ddr Exp $ *)
+(* $Id: pa_rp.ml,v 3.0 2001/04/19 07:51:51 ddr Exp $ *)
 
 open Pcaml;
 
@@ -107,7 +107,7 @@ value stream_pattern_component skont ckont =
   [ SpTrm loc p wo ->
       <:expr< match $peek_fun loc$ $lid:strm_n$ with
               [ Some $p$ $when:wo$ ->
-                  do $junk_fun loc$ $lid:strm_n$; return $skont$
+                  do { $junk_fun loc$ $lid:strm_n$; $skont$ }
               | _ -> $ckont$ ] >>
   | SpNtr loc p e ->
       let e =
@@ -177,7 +177,7 @@ value stream_patterns_term loc ekont tspel =
              <:expr< raise (Stream.Error $str$) >>
            in
            let skont = stream_pattern loc epo e ekont spcl in
-           <:expr< do $junk_fun loc$ $lid:strm_n$; return $skont$ >>
+           <:expr< do { $junk_fun loc$ $lid:strm_n$; $skont$ } >>
          in
          (p, w, e))
       tspel
