@@ -5,12 +5,12 @@
 (*                                                                     *)
 (*        Daniel de Rauglaudre, projet Cristal, INRIA Rocquencourt     *)
 (*                                                                     *)
-(*  Copyright 1998 Institut National de Recherche en Informatique et   *)
+(*  Copyright 2001 Institut National de Recherche en Informatique et   *)
 (*  Automatique.  Distributed only by permission.                      *)
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: q_MLast.ml,v 2.5 2000/04/17 21:10:28 ddr Exp $ *)
+(* $Id: q_MLast.ml,v 3.16 2001/07/21 12:37:14 ddr Exp $ *)
 
 value gram = Grammar.create (Plexer.make ());
 
@@ -20,9 +20,10 @@ type ast =
   | Tuple of list ast
   | Option of option ast
   | Str of string
-  | Chr of char
+  | Chr of string
   | Bool of bool
   | Cons of ast and ast
+  | Append of ast and ast
   | Record of list (string * ast)
   | Loc
   | Antiquot of MLast.loc and string ]
@@ -55,10 +56,14 @@ value antiquot k (bp, ep) x =
 value mkumin f arg =
   match arg with
   [ Node "ExInt" [Str n] when int_of_string n > 0 ->
-      let n = "-" ^ n in Node "ExInt" [Str n]
+      let n = "-" ^ n in
+      Node "ExInt" [Str n]
   | Node "ExFlo" [Str n] when float_of_string n > 0.0 ->
-      let n = "-" ^ n in Node "ExFlo" [Str n]
-  | _ -> let f = "~" ^ f in Node "ExApp" [Node "ExLid" [Str f]; arg] ]
+      let n = "-" ^ n in
+      Node "ExFlo" [Str n]
+  | _ ->
+      let f = "~" ^ f in
+      Node "ExApp" [Node "ExLid" [Str f]; arg] ]
 ;
 
 value mklistexp last =
@@ -87,10 +92,20 @@ value mklistpat last =
 
 value neg s = string_of_int (- int_of_string s);
 
+value not_yet_warned = ref True;
+value warning_seq () =
+  if not_yet_warned.val then do {
+    not_yet_warned.val := False;
+    Printf.eprintf "\
+*** warning: use of old syntax for sequences in expr quotation\n";
+    flush stderr
+  }
+  else ()
+;
+
 EXTEND
   GLOBAL: sig_item str_item ctyp patt expr directive module_type module_expr
     class_type class_expr class_sig_item class_str_item;
-
   module_expr:
     [ [ "functor"; "("; i = uident; ":"; t = module_type; ")"; "->";
         me = SELF ->
@@ -108,12 +123,14 @@ EXTEND
   str_item:
     [ [ "declare"; st = SLIST0 [ s = str_item; ";" -> s ]; "end" ->
           Node "StDcl" [st]
+      | "#"; n = lident; dp = dir_param -> Node "StDir" [n; dp]
       | "exception"; ctl = constructor_declaration ->
           match ctl with
           [ Tuple ctl -> Node "StExc" ctl
           | _ -> match () with [] ]
       | "external"; i = lident; ":"; t = ctyp; "="; p = SLIST1 string ->
           Node "StExt" [i; t; p]
+      | "include"; me = module_expr -> Node "StInc" [me]
       | "module"; i = uident; mb = module_binding -> Node "StMod" [i; mb]
       | "module"; "type"; i = uident; "="; mt = module_type ->
           Node "StMty" [i; mt]
@@ -142,7 +159,7 @@ EXTEND
     | [ m1 = SELF; m2 = SELF -> Node "MtApp" [m1; m2] ]
     | [ m1 = SELF; "."; m2 = SELF -> Node "MtAcc" [m1; m2] ]
     | [ i = UIDENT -> Node "MtUid" [Str i]
-      | i = LIDENT -> Node "MuLid" [Str i]
+      | i = LIDENT -> Node "MtLid" [Str i]
       | a = anti_uid -> Node "MtUid" [a]
       | a = anti_lid -> Node "MtLid" [a]
       | a = anti_ -> a
@@ -151,6 +168,7 @@ EXTEND
   sig_item:
     [ [ "declare"; st = SLIST0 [ s = sig_item; ";" -> s ]; "end" ->
           Node "SgDcl" [st]
+      | "#"; n = lident; dp = dir_param -> Node "SgDir" [n; dp]
       | "exception"; ctl = constructor_declaration ->
           match ctl with
           [ Tuple ctl -> Node "SgExc" ctl
@@ -178,15 +196,19 @@ EXTEND
       | "module"; i = mod_ident; "="; mt = module_type ->
           Node "WcMod" [i; mt] ] ]
   ;
+  dir_param:
+    [ [ a = anti_opt -> a
+      | e = expr -> Option (Some e)
+      | -> Option None ] ]
+  ;
   expr:
     [ RIGHTA
       [ "let"; r = rec_flag; l = SLIST1 let_binding SEP "and"; "in";
-        x = expr ->
+        x = SELF ->
           Node "ExLet" [r; l; x]
-      | "let"; "module"; m = uident; mb = module_binding; "in"; x = expr ->
+      | "let"; "module"; m = uident; mb = module_binding; "in"; x = SELF ->
           Node "ExLmd" [m; mb; x]
-      | "fun"; "["; l = SLIST0 match_case SEP "|"; "]" ->
-          Node "ExFun" [l]
+      | "fun"; "["; l = SLIST0 match_case SEP "|"; "]" -> Node "ExFun" [l]
       | "fun"; p = ipatt; e = fun_def ->
           Node "ExFun" [List [Tuple [p; Option None; e]]]
       | "match"; e = SELF; "with"; "["; l = SLIST0 match_case SEP "|"; "]" ->
@@ -199,15 +221,13 @@ EXTEND
           Node "ExTry" [x; List [Tuple [p; Option None; e]]]
       | "if"; e1 = SELF; "then"; e2 = SELF; "else"; e3 = SELF ->
           Node "ExIfe" [e1; e2; e3]
-      | "do"; seq = SLIST0 [ e = expr; ";" -> e ]; "return"; e = SELF ->
-          Node "ExSeq" [seq; e]
+      | "do"; "{"; seq = SLIST0 expr SEP ";"; "}" -> Node "ExSeq" [seq]
       | "for"; i = lident; "="; e1 = SELF; df = direction_flag; e2 = SELF;
-        "do"; seq = SLIST0 [ e = expr; ";" -> e ]; "done" ->
+        "do"; "{"; seq = SLIST0 [ e = expr; ";" -> e ]; "}" ->
           Node "ExFor" [i; e1; e2; df; seq]
-      | "while"; e = SELF; "do"; seq = SLIST0 [ e = expr; ";" -> e ]; "done" ->
+      | "while"; e = SELF; "do"; "{"; seq = SLIST0 [ e = expr; ";" -> e ];
+        "}" ->
           Node "ExWhi" [e; seq] ]
-    | [ e = SELF; "where"; rf = rec_flag; lb = let_binding ->
-          Node "ExLet" [rf; List [lb]; e] ]
     | NONA
       [ e1 = SELF; ":="; e2 = SELF; dummy -> Node "ExAss" [e1; e2] ]
     | RIGHTA
@@ -287,7 +307,7 @@ EXTEND
       [ s = INT -> Node "ExInt" [Str s]
       | s = FLOAT -> Node "ExFlo" [Str s]
       | s = STRING -> Node "ExStr" [Str s]
-      | s = CHAR -> Node "ExChr" [Chr s.[0]]
+      | s = CHAR -> Node "ExChr" [Chr s]
       | s = UIDENT -> Node "ExUid" [Str s]
       | s = LIDENT -> Node "ExLid" [Str s]
       | "`"; s = ident -> Node "ExVrn" [s]
@@ -306,7 +326,7 @@ EXTEND
       | "[|"; el = SLIST0 expr SEP ";"; "|]" -> Node "ExArr" [el]
       | "{"; lel = SLIST1 label_expr SEP ";"; "}" ->
           Node "ExRec" [lel; Option None]
-      | "{"; "("; e = expr; ")"; "with"; lel = SLIST1 label_expr SEP ";";
+      | "{"; "("; e = SELF; ")"; "with"; lel = SLIST1 label_expr SEP ";";
         "}" ->
           Node "ExRec" [lel; Option (Some e)]
       | "("; ")" -> Node "ExUid" [Str "()"]
@@ -316,6 +336,19 @@ EXTEND
       | "("; el = anti_list; ")" -> Node "ExTup" [el]
       | "("; e = SELF; ")" -> e ] ]
   ;
+  expr:
+    [ [ "do"; seq = SLIST0 [ e = expr; ";" -> e ]; "return"; e = SELF ->
+          let _ = warning_seq () in
+          Node "ExSeq" [Append seq e]
+      | "for"; i = lident; "="; e1 = SELF; df = direction_flag; e2 = SELF;
+        "do"; seq = SLIST0 [ e = expr; ";" -> e ]; "done" ->
+          let _ = warning_seq () in
+          Node "ExFor" [i; e1; e2; df; seq]
+      | "while"; e = SELF; "do"; seq = SLIST0 [ e = expr; ";" -> e ];
+        "done" ->
+          let _ = warning_seq () in
+          Node "ExWhi" [e; seq] ] ]
+  ;
   dummy:
     [ [ -> () ] ]
   ;
@@ -339,13 +372,7 @@ EXTEND
           Tuple [p; w; e] ] ]
   ;
   label_expr:
-    [ [ i = expr_label_ident; "="; e = expr -> Tuple [i; e] ] ]
-  ;
-  expr_label_ident:
-    [ LEFTA
-      [ e1 = SELF; "."; e2 = SELF -> Node "ExAcc" [e1; e2] ]
-    | RIGHTA
-      [ i = uident -> Node "ExUid" [i] | i = lident -> Node "ExLid" [i] ] ]
+    [ [ i = patt_label_ident; "="; e = expr -> Tuple [i; e] ] ]
   ;
   fun_def:
     [ [ p = ipatt; e = SELF -> Node "ExFun" [List [Tuple [p; Option None; e]]]
@@ -357,29 +384,27 @@ EXTEND
     | [ p1 = SELF; p2 = SELF -> Node "PaApp" [p1; p2] ]
     | [ p1 = SELF; "."; p2 = SELF -> Node "PaAcc" [p1; p2] ]
     | NONA
-      [ "~"; i = lident; ":"; p = SELF ->
-          Node "PaLab" [i; p]
-      | "~"; i = lident ->
-          Node "PaLab" [i; Node "PaLid" [i]]
-      | "?"; i = lident; ":"; p = SELF ->
-          Node "PaOlb" [i; p; Option None]
+      [ "~"; i = lident; ":"; p = SELF -> Node "PaLab" [i; p]
+      | "~"; i = lident -> Node "PaLab" [i; Node "PaLid" [i]]
+      | "?"; i = lident; ":"; p = SELF -> Node "PaOlb" [i; p; Option None]
       | "?"; i = lident; ":"; "("; p = SELF; "="; e = expr; ")" ->
           Node "PaOlb" [i; p; Option (Some e)]
       | "?"; "("; j = lident; "="; e = expr; ")" ->
           Node "PaOlb" [j; Node "PaLid" [j]; Option (Some e)]
-      | "?"; i = lident ->
-          Node "PaOlb" [i; Node "PaLid" [i]; Option None] ]
+      | "?"; i = lident -> Node "PaOlb" [i; Node "PaLid" [i]; Option None] ]
     | "simple"
       [ v = LIDENT -> Node "PaLid" [Str v]
       | v = UIDENT -> Node "PaUid" [Str v]
       | s = INT -> Node "PaInt" [Str s]
       | "-"; s = INT -> Node "PaInt" [Str (neg s)]
+      | s = FLOAT -> Node "PaFlo" [Str s]
       | s = STRING -> Node "PaStr" [Str s]
-      | s = CHAR -> Node "PaChr" [Chr s.[0]]
+      | s = CHAR -> Node "PaChr" [Chr s]
       | "`"; s = ident -> Node "PaVrn" [s]
       | a = anti_lid -> Node "PaLid" [a]
       | a = anti_uid -> Node "PaUid" [a]
       | a = anti_int -> Node "PaInt" [a]
+      | a = anti_flo -> Node "PaFlo" [a]
       | a = anti_str -> Node "PaStr" [a]
       | a = anti_chr -> Node "PaChr" [a]
       | a = anti_anti -> Node "PaAnt" [a]
@@ -406,7 +431,11 @@ EXTEND
     [ LEFTA
       [ p1 = SELF; "."; p2 = SELF -> Node "PaAcc" [p1; p2] ]
     | RIGHTA
-      [ i = uident -> Node "PaUid" [i] | i = lident -> Node "PaLid" [i] ] ]
+      [ a = anti_ -> a
+      | a = anti_lid -> Node "PaLid" [a]
+      | a = anti_uid -> Node "PaUid" [a]
+      | i = UIDENT -> Node "PaUid" [Str i]
+      | i = LIDENT -> Node "PaLid" [Str i] ] ]
   ;
   ipatt:
     [ [ "{"; lpl = SLIST1 label_ipatt SEP ";"; "}" -> Node "PaRec" [lpl]
@@ -435,7 +464,9 @@ EXTEND
     [ [ "constraint"; t1 = ctyp; "="; t2 = ctyp -> Tuple [t1; t2] ] ]
   ;
   type_parameter:
-    [ [ "'"; i = ident -> i ] ]
+    [ [ "'"; i = ident -> Tuple [i; Tuple [Bool False; Bool False]]
+      | "_"; "'"; i = ident -> Tuple [i; Tuple [Bool True; Bool False]]
+      | "-"; "'"; i = ident -> Tuple [i; Tuple [Bool False; Bool True]] ] ]
   ;
   ctyp:
     [ LEFTA
@@ -466,10 +497,9 @@ EXTEND
       | "("; t = SELF; ")" -> t
       | "["; cdl = SLIST0 constructor_declaration SEP "|"; "]" ->
           Node "TySum" [cdl]
-      | "{"; ldl = SLIST1 label_declaration SEP ";"; "}" ->
-          Node "TyRec" [ldl]
+      | "{"; ldl = SLIST1 label_declaration SEP ";"; "}" -> Node "TyRec" [ldl]
       | "[|"; rfl = SLIST0 row_field SEP "|"; "|]" ->
-          Node "TyVrn" [rfl; Option None ]
+          Node "TyVrn" [rfl; Option None]
       | "[|"; ">"; rfl = SLIST1 row_field SEP "|"; "|]" ->
           Node "TyVrn" [rfl; Option (Some (Option None))]
       | "[|"; "<"; (rfl, clos) = row_field_list_dd; sl = opt_tag_list; "|]" ->
@@ -486,16 +516,14 @@ EXTEND
       | a = anti_list -> (a, Bool True)
       | rf = row_field; clos = clos -> (rf, clos)
       | rf = row_field -> (rf, Bool True)
-      | rf = row_field; "|"; (rfl, clos) = row_field_list_dd ->
-          (Cons rf rfl, clos) ] ]
+      | rf = row_field; "|"; (rfl, clos) = SELF -> (Cons rf rfl, clos) ] ]
   ;
   clos:
     [ [ a = anti_dd -> a
       | "|"; ".." -> Bool False ] ]
   ;
   opt_tag_list:
-    [ [ a = anti_gt -> a
-      | ">"; sl = LIST1 lident -> List sl
+    [ [ ">"; sl = SLIST1 lident -> sl
       | -> List [] ] ]
   ;
   constructor_declaration:
@@ -506,13 +534,17 @@ EXTEND
     [ [ i = lident; ":"; mf = mutable_flag; t = ctyp -> Tuple [i; mf; t] ] ]
   ;
   ident:
-    [ [ i = LIDENT -> Str i | i = UIDENT -> Str i | a = anti_ -> a ] ]
+    [ [ i = LIDENT -> Str i
+      | i = UIDENT -> Str i
+      | a = anti_ -> a ] ]
   ;
   lident:
-    [ [ i = LIDENT -> Str i | a = anti_ -> a ] ]
+    [ [ i = LIDENT -> Str i
+      | a = anti_ -> a ] ]
   ;
   uident:
-    [ [ i = UIDENT -> Str i | a = anti_ -> a ] ]
+    [ [ i = UIDENT -> Str i
+      | a = anti_ -> a ] ]
   ;
   mod_ident:
     [ RIGHTA
@@ -524,13 +556,18 @@ EXTEND
       | m = UIDENT; "."; i = SELF -> Cons (Str m) i ] ]
   ;
   direction_flag:
-    [ [ "to" -> Bool True | "downto" -> Bool False | a = anti_to -> a ] ]
+    [ [ "to" -> Bool True
+      | "downto" -> Bool False
+      | a = anti_to -> a ] ]
   ;
   string:
-    [ [ s = STRING -> Str s | a = anti_ -> a ] ]
+    [ [ s = STRING -> Str s
+      | a = anti_ -> a ] ]
   ;
   rec_flag:
-    [ [ a = anti_rec -> a | "rec" -> Bool True | -> Bool False ] ]
+    [ [ a = anti_rec -> a
+      | "rec" -> Bool True
+      | -> Bool False ] ]
   ;
   as_opt:
     [ [ "as"; p = patt -> Option (Some p)
@@ -543,10 +580,12 @@ EXTEND
       | -> Option None ] ]
   ;
   mutable_flag:
-    [ [ a = anti_mut -> a | "mutable" -> Bool True | -> Bool False ] ]
+    [ [ a = anti_mut -> a
+      | "mutable" -> Bool True
+      | -> Bool False ] ]
   ;
   anti_:
-    [ [ a = ANTIQUOT "" -> antiquot "" loc a ] ]
+    [ [ a = ANTIQUOT -> antiquot "" loc a ] ]
   ;
   anti_anti:
     [ [ a = ANTIQUOT "anti" -> antiquot "anti" loc a ] ]
@@ -566,9 +605,6 @@ EXTEND
   anti_flo:
     [ [ a = ANTIQUOT "flo" -> antiquot "flo" loc a ] ]
   ;
-  anti_gt:
-    [ [ a = ANTIQUOT "gt" -> antiquot "gt" loc a ] ]
-  ;
   anti_int:
     [ [ a = ANTIQUOT "int" -> antiquot "int" loc a ] ]
   ;
@@ -581,6 +617,9 @@ EXTEND
   anti_mut:
     [ [ a = ANTIQUOT "mut" -> antiquot "mut" loc a ] ]
   ;
+  anti_opt:
+    [ [ a = ANTIQUOT "opt" -> antiquot "mut" loc a ] ]
+  ;
   anti_rec:
     [ [ a = ANTIQUOT "rec" -> antiquot "rec" loc a ] ]
   ;
@@ -600,14 +639,12 @@ EXTEND
 (* Objects and Classes *)
 
   str_item:
-    [ [ "class"; cd = SLIST1 class_declaration SEP "and" ->
-          Node "StCls" [cd]
+    [ [ "class"; cd = SLIST1 class_declaration SEP "and" -> Node "StCls" [cd]
       | "class"; "type"; ctd = SLIST1 class_type_declaration SEP "and" ->
           Node "StClt" [ctd] ] ]
   ;
   sig_item:
-    [ [ "class"; cd = SLIST1 class_description SEP "and" ->
-          Node "SgCls" [cd]
+    [ [ "class"; cd = SLIST1 class_description SEP "and" -> Node "SgCls" [cd]
       | "class"; "type"; ctd = SLIST1 class_type_declaration SEP "and" ->
           Node "SgClt" [ctd] ] ]
   ;
@@ -618,50 +655,43 @@ EXTEND
     [ [ vf = virtual_flag; i = lident; ctp = class_type_parameters;
         cfb = class_fun_binding ->
           Record
-            [("ciLoc", Loc); ("ciVir", vf); ("ciPrm", ctp);
-             ("ciNam", i); ("ciExp", cfb)] ] ]
+            [("ciLoc", Loc); ("ciVir", vf); ("ciPrm", ctp); ("ciNam", i);
+             ("ciExp", cfb)] ] ]
   ;
   class_fun_binding:
     [ [ "="; ce = class_expr -> ce
-      | ":"; ct = class_type; "="; ce = class_expr ->
-          Node "CeTyc" [ce; ct]
-      | p = patt LEVEL "simple"; cfb = class_fun_binding ->
-          Node "CeFun" [p; cfb] ] ]
+      | ":"; ct = class_type; "="; ce = class_expr -> Node "CeTyc" [ce; ct]
+      | p = patt LEVEL "simple"; cfb = SELF -> Node "CeFun" [p; cfb] ] ]
   ;
   class_type_parameters:
     [ [ -> Tuple [Loc; List []]
       | "["; tpl = SLIST1 type_parameter SEP ","; "]" -> Tuple [Loc; tpl] ] ]
   ;
   class_fun_def:
-    [ [ p = patt LEVEL "simple"; "->"; ce = class_expr ->
-          Node "CeFun" [p; ce]
-      | p = patt LEVEL "simple"; cfd = class_fun_def ->
-          Node "CeFun" [p; cfd] ] ]
-  ;      
+    [ [ p = patt LEVEL "simple"; "->"; ce = class_expr -> Node "CeFun" [p; ce]
+      | p = patt LEVEL "simple"; cfd = SELF -> Node "CeFun" [p; cfd] ] ]
+  ;
   class_expr:
     [ "top"
       [ "fun"; cfd = class_fun_def -> cfd
       | "let"; rf = rec_flag; lb = SLIST1 let_binding SEP "and"; "in";
-        ce = class_expr ->
+        ce = SELF ->
           Node "CeLet" [rf; lb; ce] ]
     | "apply" NONA
-      [ ce = class_expr; sel = SLIST1 (expr LEVEL "simple") ->
+      [ ce = SELF; sel = SLIST1 (expr LEVEL "simple") ->
           Node "CeApp" [ce; sel] ]
     | "simple"
       [ a = anti_ -> a
-      | ci = class_longident;
-        "["; ct = ctyp; ","; ctcl = SLIST1 ctyp SEP ","; "]" ->
+      | ci = class_longident; "["; ct = ctyp; ","; ctcl = SLIST1 ctyp SEP ",";
+        "]" ->
           Node "CeCon" [ci; Cons ct ctcl]
       | ci = class_longident; "["; ct = ctyp; "]" ->
           Node "CeCon" [ci; List [ct]]
-      | ci = class_longident ->
-          Node "CeCon" [ci; List []]
+      | ci = class_longident -> Node "CeCon" [ci; List []]
       | "object"; csp = class_self_patt_opt; cf = class_structure; "end" ->
           Node "CeStr" [csp; cf]
-      | "("; ce = class_expr; ":"; ct = class_type; ")" ->
-          Node "CeTyc" [ce; ct]
-      | "("; ce = class_expr; ")" ->
-          ce ] ]
+      | "("; ce = SELF; ":"; ct = class_type; ")" -> Node "CeTyc" [ce; ct]
+      | "("; ce = SELF; ")" -> ce ] ]
   ;
   class_structure:
     [ [ cf = SLIST0 [ cf = class_str_item; ";" -> cf ] -> cf ] ]
@@ -673,10 +703,8 @@ EXTEND
           Option (Some (Node "PaTyc" [p; t])) ] ]
   ;
   class_str_item:
-    [ [ "inherit"; ce = class_expr; pb = as_ident_opt ->
-          Node "CrInh" [ce; pb]
-      | "value"; (lab, mf, e)  = cvalue ->
-          Node "CrVal" [lab; mf; e]
+    [ [ "inherit"; ce = class_expr; pb = as_ident_opt -> Node "CrInh" [ce; pb]
+      | "value"; (lab, mf, e) = cvalue -> Node "CrVal" [lab; mf; e]
       | "method"; "private"; "virtual"; l = label; ":"; t = ctyp ->
           Node "CrVir" [l; Bool True; t]
       | "method"; "virtual"; pf = private_flag; l = label; ":"; t = ctyp ->
@@ -685,14 +713,11 @@ EXTEND
           Node "CrMth" [l; Bool True; fb]
       | "method"; l = label; fb = fun_binding ->
           Node "CrMth" [l; Bool False; fb]
-      | "type"; t1 = ctyp; "="; t2 = ctyp ->
-          Node "CrCtr" [t1; t2]
-      | "initializer"; se = expr ->
-          Node "CrIni" [se] ] ]
+      | "type"; t1 = ctyp; "="; t2 = ctyp -> Node "CrCtr" [t1; t2]
+      | "initializer"; se = expr -> Node "CrIni" [se] ] ]
   ;
   cvalue:
-    [ [ mf = mutable_flag; l = label; "="; e = expr ->
-          (l, mf, e)
+    [ [ mf = mutable_flag; l = label; "="; e = expr -> (l, mf, e)
       | mf = mutable_flag; l = label; ":"; t = ctyp; "="; e = expr ->
           (l, mf, Node "ExTyc" [e; t])
       | mf = mutable_flag; l = label; ":"; t1 = ctyp; ":>"; t2 = ctyp; "=";
@@ -709,15 +734,12 @@ EXTEND
 
   class_type:
     [ [ a = anti_ -> a
-      | "["; t = ctyp; "]"; "->"; ct = class_type ->
-          Node "CtFun" [t; ct]
-      |  "["; t = ctyp; ","; tl = SLIST1 ctyp SEP ","; "]";
+      | "["; t = ctyp; "]"; "->"; ct = SELF -> Node "CtFun" [t; ct]
+      | "["; t = ctyp; ","; tl = SLIST1 ctyp SEP ","; "]";
         id = clty_longident ->
           Node "CtCon" [id; Cons t tl]
-      | "["; t = ctyp; "]"; id = clty_longident ->
-          Node "CtCon" [id; List [t]]
-      | id = clty_longident ->
-          Node "CtCon" [id; List []]
+      | "["; t = ctyp; "]"; id = clty_longident -> Node "CtCon" [id; List [t]]
+      | id = clty_longident -> Node "CtCon" [id; List []]
       | "object"; cst = class_self_type_opt;
         csf = SLIST0 [ csf = class_sig_item; ";" -> csf ]; "end" ->
           Node "CtSig" [cst; csf] ] ]
@@ -727,8 +749,7 @@ EXTEND
       | "("; t = ctyp; ")" -> Option (Some t) ] ]
   ;
   class_sig_item:
-    [ [ "inherit"; cs = class_type ->
-          Node "CgInh" [cs]
+    [ [ "inherit"; cs = class_type -> Node "CgInh" [cs]
       | "value"; mf = mutable_flag; l = label; ":"; t = ctyp ->
           Node "CgVal" [l; mf; t]
       | "method"; "private"; "virtual"; l = label; ":"; t = ctyp ->
@@ -737,10 +758,8 @@ EXTEND
           Node "CgVir" [l; pf; t]
       | "method"; "private"; l = label; ":"; t = ctyp ->
           Node "CgMth" [l; Bool True; t]
-      | "method"; l = label; ":"; t = ctyp ->
-          Node "CgMth" [l; Bool False; t]
-      | "type"; t1 = ctyp; "="; t2 = ctyp ->
-          Node "CgCtr" [t1; t2] ] ]
+      | "method"; l = label; ":"; t = ctyp -> Node "CgMth" [l; Bool False; t]
+      | "type"; t1 = ctyp; "="; t2 = ctyp -> Node "CgCtr" [t1; t2] ] ]
   ;
   class_description:
     [ [ vf = virtual_flag; n = lident; ctp = class_type_parameters; ":";
@@ -764,23 +783,18 @@ EXTEND
       [ "new"; i = class_longident -> Node "ExNew" [i] ] ]
   ;
   expr: LEVEL "."
-    [ [ e = expr; "#"; lab = label -> Node "ExSnd" [e; lab] ] ]
+    [ [ e = SELF; "#"; lab = label -> Node "ExSnd" [e; lab] ] ]
   ;
   expr: LEVEL "simple"
-    [ [ "("; e = expr; ":"; t1 = ctyp; ":>"; t2 = ctyp; ")" ->
+    [ [ "("; e = SELF; ":"; t1 = ctyp; ":>"; t2 = ctyp; ")" ->
           Node "ExCoe" [Node "ExTyc" [e; t1]; t2]
-      | "("; e = expr; ":>"; t = ctyp; ")" ->
-          Node "ExCoe" [e; t]
-      | "{<"; ">}" ->
-          Node "ExOvr" [List []]
-      | "{<"; fel = field_expr_list; ">}" ->
-          Node "ExOvr" [List fel]
-      | "{<"; fel = anti_list; ">}" ->
-          Node "ExOvr" [fel] ] ]
+      | "("; e = SELF; ":>"; t = ctyp; ")" -> Node "ExCoe" [e; t]
+      | "{<"; ">}" -> Node "ExOvr" [List []]
+      | "{<"; fel = field_expr_list; ">}" -> Node "ExOvr" [List fel]
+      | "{<"; fel = anti_list; ">}" -> Node "ExOvr" [fel] ] ]
   ;
   field_expr_list:
-    [ [ l = label; "="; e = expr; ";"; fel = field_expr_list ->
-          [Tuple [l; e] :: fel]
+    [ [ l = label; "="; e = expr; ";"; fel = SELF -> [Tuple [l; e] :: fel]
       | l = label; "="; e = expr; ";" -> [Tuple [l; e]]
       | l = label; "="; e = expr -> [Tuple [l; e]] ] ]
   ;
@@ -795,7 +809,7 @@ EXTEND
   meth_list:
     [ [ a = anti_list -> (a, Bool False)
       | a = anti_list; b = anti_ -> (a, b)
-      | f = field; ";"; (ml, v) = meth_list -> (Cons f ml, v)
+      | f = field; ";"; (ml, v) = SELF -> (Cons f ml, v)
       | f = field; ";" -> (List [f], Bool False)
       | f = field -> (List [f], Bool False)
       | ".." -> (List [], Bool True) ] ]
@@ -807,7 +821,7 @@ EXTEND
   (* Identifiers *)
 
   longid:
-    [ [ m = uident; "."; l = longid -> [m :: l]
+    [ [ m = uident; "."; l = SELF -> [m :: l]
       | i = lident -> [i] ] ]
   ;
   clty_longident:
@@ -818,12 +832,15 @@ EXTEND
     [ [ l = longid -> List l
       | a = anti_list -> a ] ]
   ;
-
   virtual_flag:
-    [ [ a = anti_virt -> a | "virtual" -> Bool True | -> Bool False ] ]
+    [ [ a = anti_virt -> a
+      | "virtual" -> Bool True
+      | -> Bool False ] ]
   ;
   private_flag:
-    [ [ a = anti_priv -> a | "private" -> Bool True | -> Bool False ] ]
+    [ [ a = anti_priv -> a
+      | "private" -> Bool True
+      | -> Bool False ] ]
   ;
   as_ident_opt:
     [ [ "as"; p = lident -> Option (Some p)
@@ -844,7 +861,7 @@ value rec expr_of_ast =
   fun
   [ Node n al ->
       List.fold_left (fun e a -> <:expr< $e$ $expr_of_ast a$ >>)
-        <:expr< MLast.$uid:n$ loc >> al
+        <:expr< MLast.$uid:n$ $lid:Stdpp.loc_name.val$ >> al
   | List al ->
       List.fold_right (fun a e -> <:expr< [$expr_of_ast a$ :: $e$] >>) al
         <:expr< [] >>
@@ -856,18 +873,17 @@ value rec expr_of_ast =
   | Bool True -> <:expr< True >>
   | Bool False -> <:expr< False >>
   | Cons a1 a2 -> <:expr< [$expr_of_ast a1$ :: $expr_of_ast a2$] >>
+  | Append a1 a2 -> <:expr< $expr_of_ast a1$ @ [$expr_of_ast a2$] >>
   | Record lal -> <:expr< {$list:List.map label_expr_of_ast lal$} >>
-  | Loc -> <:expr< loc >>
+  | Loc -> <:expr< $lid:Stdpp.loc_name.val$ >>
   | Antiquot loc s ->
       let e =
         try Grammar.Entry.parse Pcaml.expr_eoi (Stream.of_string s) with
         [ Stdpp.Exc_located (bp, ep) exc ->
             raise (Stdpp.Exc_located (fst loc + bp, fst loc + ep) exc) ]
       in
-      MLast.ExAnt loc e ]
-and label_expr_of_ast (l, a) =
-  (<:expr< MLast.$lid:l$ >>, expr_of_ast a)
-;
+      <:expr< $anti:e$ >> ]
+and label_expr_of_ast (l, a) = (<:patt< MLast.$lid:l$ >>, expr_of_ast a);
 
 value rec patt_of_ast =
   fun
@@ -885,129 +901,141 @@ value rec patt_of_ast =
   | Bool True -> <:patt< True >>
   | Bool False -> <:patt< False >>
   | Cons a1 a2 -> <:patt< [$patt_of_ast a1$ :: $patt_of_ast a2$] >>
+  | Append _ _ -> failwith "bad pattern"
   | Record lal -> <:patt< {$list:List.map label_patt_of_ast lal$} >>
-  | Loc -> <:patt< loc >>
+  | Loc -> <:patt< $lid:Stdpp.loc_name.val$ >>
   | Antiquot loc s ->
       let p =
         try Grammar.Entry.parse Pcaml.patt_eoi (Stream.of_string s) with
         [ Stdpp.Exc_located (bp, ep) exc ->
             raise (Stdpp.Exc_located (fst loc + bp, fst loc + ep) exc) ]
       in
-      MLast.PaAnt loc p ]
-and label_patt_of_ast (l, a) =
-  (<:patt< MLast.$lid:l$ >>, patt_of_ast a)
-;
+      <:patt< $anti:p$ >> ]
+and label_patt_of_ast (l, a) = (<:patt< MLast.$lid:l$ >>, patt_of_ast a);
 
 value apply_entry e =
   let f s = Grammar.Entry.parse e (Stream.of_string s) in
   let expr s = expr_of_ast (f s) in
-  let patt s = patt_of_ast (f s) in Quotation.ExAst (expr, patt)
+  let patt s = patt_of_ast (f s) in
+  Quotation.ExAst (expr, patt)
 ;
 
 let sig_item_eoi = Grammar.Entry.create gram "signature item" in
-do EXTEND
-     sig_item_eoi:
-       [ [ x = sig_item; EOI -> x ] ]
-     ;
-   END;
-   Quotation.add "sig_item" (apply_entry sig_item_eoi);
-return ();
+do {
+  EXTEND
+    sig_item_eoi:
+      [ [ x = sig_item; EOI -> x ] ]
+    ;
+  END;
+  Quotation.add "sig_item" (apply_entry sig_item_eoi)
+};
 
 let str_item_eoi = Grammar.Entry.create gram "structure item" in
-do EXTEND
-     str_item_eoi:
-       [ [ x = str_item; EOI -> x ] ]
-     ;
-   END;
-   Quotation.add "str_item" (apply_entry str_item_eoi);
-return ();
+do {
+  EXTEND
+    str_item_eoi:
+      [ [ x = str_item; EOI -> x ] ]
+    ;
+  END;
+  Quotation.add "str_item" (apply_entry str_item_eoi)
+};
 
 let ctyp_eoi = Grammar.Entry.create gram "type" in
-do EXTEND
-     ctyp_eoi:
-       [ [ x = ctyp; EOI -> x ] ]
-     ;
-   END;
-   Quotation.add "ctyp" (apply_entry ctyp_eoi);
-return ();
+do {
+  EXTEND
+    ctyp_eoi:
+      [ [ x = ctyp; EOI -> x ] ]
+    ;
+  END;
+  Quotation.add "ctyp" (apply_entry ctyp_eoi)
+};
 
 let patt_eoi = Grammar.Entry.create gram "pattern" in
-do EXTEND
-     patt_eoi:
-       [ [ x = patt; EOI -> x ] ]
-     ;
-   END;
-   Quotation.add "patt" (apply_entry patt_eoi);
-return ();
+do {
+  EXTEND
+    patt_eoi:
+      [ [ x = patt; EOI -> x ] ]
+    ;
+  END;
+  Quotation.add "patt" (apply_entry patt_eoi)
+};
 
 let expr_eoi = Grammar.Entry.create gram "expression" in
-do EXTEND
-     expr_eoi:
-       [ [ x = expr; EOI -> x ] ]
-     ;
-   END;
-   Quotation.add "expr" (apply_entry expr_eoi);
-return ();
+do {
+  EXTEND
+    expr_eoi:
+      [ [ x = expr; EOI -> x ] ]
+    ;
+  END;
+  Quotation.add "expr" (apply_entry expr_eoi)
+};
 
 let module_type_eoi = Grammar.Entry.create gram "module type" in
-do EXTEND
-     module_type_eoi:
-       [ [ x = module_type; EOI -> x ] ]
-     ;
-   END;
-   Quotation.add "module_type" (apply_entry module_type_eoi);
-return ();
+do {
+  EXTEND
+    module_type_eoi:
+      [ [ x = module_type; EOI -> x ] ]
+    ;
+  END;
+  Quotation.add "module_type" (apply_entry module_type_eoi)
+};
 
 let module_expr_eoi = Grammar.Entry.create gram "module expression" in
-do EXTEND
-     module_expr_eoi:
-       [ [ x = module_expr; EOI -> x ] ]
-     ;
-   END;
-   Quotation.add "module_expr" (apply_entry module_expr_eoi);
-return ();
+do {
+  EXTEND
+    module_expr_eoi:
+      [ [ x = module_expr; EOI -> x ] ]
+    ;
+  END;
+  Quotation.add "module_expr" (apply_entry module_expr_eoi)
+};
 
 let directive_eoi = Grammar.Entry.create gram "directive" in
-do EXTEND
-     directive_eoi:
-       [ [ x = directive; EOI -> x ] ]
-     ;
-   END;
-   Quotation.add "directive" (apply_entry directive_eoi);
-return ();
+do {
+  EXTEND
+    directive_eoi:
+      [ [ x = directive; EOI -> x ] ]
+    ;
+  END;
+  Quotation.add "directive" (apply_entry directive_eoi)
+};
 
 let class_type_eoi = Grammar.Entry.create gram "class_type" in
-do EXTEND
-     class_type_eoi:
-       [ [ x = class_type; EOI -> x ] ]
-     ;
-   END;
-   Quotation.add "class_type" (apply_entry class_type_eoi);
-return ();
+do {
+  EXTEND
+    class_type_eoi:
+      [ [ x = class_type; EOI -> x ] ]
+    ;
+  END;
+  Quotation.add "class_type" (apply_entry class_type_eoi)
+};
 
 let class_expr_eoi = Grammar.Entry.create gram "class_expr" in
-do EXTEND
-     class_expr_eoi:
-       [ [ x = class_expr; EOI -> x ] ]
-     ;
-   END;
-   Quotation.add "class_expr" (apply_entry class_expr_eoi);
-return ();
+do {
+  EXTEND
+    class_expr_eoi:
+      [ [ x = class_expr; EOI -> x ] ]
+    ;
+  END;
+  Quotation.add "class_expr" (apply_entry class_expr_eoi)
+};
 
 let class_sig_item_eoi = Grammar.Entry.create gram "class_sig_item" in
-do EXTEND
-     class_sig_item_eoi:
-       [ [ x = class_sig_item; EOI -> x ] ]
-     ;
-   END;
-   Quotation.add "class_sig_item" (apply_entry class_sig_item_eoi);
-return ();
+do {
+  EXTEND
+    class_sig_item_eoi:
+      [ [ x = class_sig_item; EOI -> x ] ]
+    ;
+  END;
+  Quotation.add "class_sig_item" (apply_entry class_sig_item_eoi)
+};
 
 let class_str_item_eoi = Grammar.Entry.create gram "class_str_item" in
-do EXTEND
-     class_str_item_eoi:
-       [ [ x = class_str_item; EOI -> x ] ]
-     ;
-   END;
-   Quotation.add "class_str_item" (apply_entry class_str_item_eoi);
-return ();
+do {
+  EXTEND
+    class_str_item_eoi:
+      [ [ x = class_str_item; EOI -> x ] ]
+    ;
+  END;
+  Quotation.add "class_str_item" (apply_entry class_str_item_eoi)
+};
