@@ -1,4 +1,4 @@
-(* camlp4r *)
+(* camlp4r q_MLast.cmo *)
 (* Id *)
 
 let action_arg s sl =
@@ -101,27 +101,61 @@ let loc_fmt =
 let print_location loc =
   if !(Pcaml.input_file) <> "-" then
     let (line, bp, ep) = line_of_loc !(Pcaml.input_file) loc in
-    Printf.eprintf loc_fmt !(Pcaml.input_file) line bp ep; ()
+    Printf.eprintf loc_fmt !(Pcaml.input_file) line bp ep
   else Printf.eprintf "At location %d-%d\n" (fst loc) (snd loc)
 ;;
 
-let print_warning loc s = print_location loc; Printf.eprintf "%s\n" s; ();;
+let print_warning loc s = print_location loc; Printf.eprintf "%s\n" s;;
 
-let process pa pr () =
+let process pa pr getdir =
   let name = !(Pcaml.input_file) in
   Pcaml.warning := print_warning;
   let ic = if name = "-" then stdin else open_in_bin name in
-  let clear () = if name = "-" then () else close_in ic in
   let cs = Stream.of_channel ic in
+  let clear () = if name = "-" then () else close_in ic in
   let phr =
-    try Grammar.Entry.parse pa cs with
+    try
+      let rec loop () =
+        let (pl, stopped_at_directive) = Grammar.Entry.parse pa cs in
+        if stopped_at_directive then
+          begin
+            begin match getdir (List.rev pl) with
+              Some x ->
+                begin match x with
+                  loc, "load", Some (MLast.ExStr (_, s)) ->
+                    Odyl_main.loadfile s
+                | loc, "directory", Some (MLast.ExStr (_, s)) ->
+                    Odyl_main.directory s
+                | loc, _, _ ->
+                    Stdpp.raise_with_loc loc (Stream.Error "bad directive")
+                end
+            | None -> ()
+            end;
+            pl @ loop ()
+          end
+        else pl
+      in
+      loop ()
+    with
       x -> clear (); raise x
   in
   clear (); pr phr
 ;;
 
-let process_intf () = process Pcaml.interf !(Pcaml.print_interf) ();;
-let process_impl () = process Pcaml.implem !(Pcaml.print_implem) ();;
+let gind =
+  function
+    (MLast.SgDir (loc, n, dp), _) :: _ -> Some (loc, n, dp)
+  | _ -> None
+;;
+
+let gimd =
+  function
+    (MLast.StDir (loc, n, dp), _) :: _ -> Some (loc, n, dp)
+  | _ -> None
+;;
+
+let process_intf () = process Pcaml.interf !(Pcaml.print_interf) gind;;
+let process_impl () = process Pcaml.implem !(Pcaml.print_implem) gimd;;
 
 type file_kind = Intf | Impl;;
 let file_kind = ref Intf;;
@@ -140,24 +174,27 @@ let usage =
 Usage: camlp4 [load-options] [--] [other-options]
 Load-options are:
   -I directory  Add directory in search patch for object files.
-  -where        Print camlp4's library directory and exit.
+  -where        Print camlp4 library directory and exit.
   -nolib        No automatic search for object files in library directory.
-  <object-file> Load this file in Camlp4's core.
+  <object-file> Load this file in Camlp4 core.
 Other-options are:
   <file>        Parse this file."
 ;;
 
 let initial_spec_list =
-  ["-intf",
-   Arg.String (fun x -> file_kind := Intf; Pcaml.input_file := x; ()),
+  ["-intf", Arg.String (fun x -> file_kind := Intf; Pcaml.input_file := x),
    "<file>  Parse <file> as an interface, whatever its extension.";
-   "-impl",
-   Arg.String (fun x -> file_kind := Impl; Pcaml.input_file := x; ()),
+   "-impl", Arg.String (fun x -> file_kind := Impl; Pcaml.input_file := x),
    "<file>  Parse <file> as an implementation, whatever its extension.";
    "-unsafe", Arg.Set Ast2pt.fast,
    "      Generate unsafe accesses to array and strings.";
    "-noassert", Arg.Set Pcaml.no_assert,
    "    Don't compile assertion checks.";
+   "-verbose", Arg.Set Grammar.error_verbose,
+   "     More verbose in parsing errors.";
+   "-loc", Arg.String (fun x -> Stdpp.loc_name := x),
+   "<name>   Name of the location variable (default: " ^ !(Stdpp.loc_name) ^
+     ")";
    "-QD", Arg.String (fun x -> Pcaml.quotation_dump_file := Some x),
    "<file>    Dump quotation expander result in case of syntax error.";
    "-o", Arg.String (fun x -> Pcaml.output_file := Some x),
@@ -165,7 +202,7 @@ let initial_spec_list =
    "-v", Arg.Unit print_version, "           Print Camlp4 version and exit."]
 ;;
 
-let anon_fun x = Pcaml.input_file := x; file_kind := file_kind_of_name x; ();;
+let anon_fun x = Pcaml.input_file := x; file_kind := file_kind_of_name x;;
 
 let parse spec_list anon_fun remaining_args =
   let spec_list =
@@ -205,14 +242,12 @@ let go () =
     exc ->
       Format.set_formatter_out_channel stderr;
       Format.open_vbox 0;
-      begin
-        let exc =
-          match exc with
-            Stdpp.Exc_located ((bp, ep), exc) -> print_location (bp, ep); exc
-          | _ -> exc
-        in
-        Pcaml.report_error exc
-      end;
+      let exc =
+        match exc with
+          Stdpp.Exc_located ((bp, ep), exc) -> print_location (bp, ep); exc
+        | _ -> exc
+      in
+      Pcaml.report_error exc;
       Format.close_box ();
       Format.print_newline ();
       exit 2
