@@ -5,7 +5,7 @@
 (*                                                                     *)
 (*        Daniel de Rauglaudre, projet Cristal, INRIA Rocquencourt     *)
 (*                                                                     *)
-(*  Copyright 1998 Institut National de Recherche en Informatique et   *)
+(*  Copyright 2001 Institut National de Recherche en Informatique et   *)
 (*  Automatique.  Distributed only by permission.                      *)
 (*                                                                     *)
 (***********************************************************************)
@@ -27,6 +27,16 @@ let get_tag x =
 
 let error loc str = raise_with_loc loc (Failure str);;
 
+let char_of_char_token loc s =
+  try Token.eval_char s with
+    Failure _ as exn -> raise_with_loc loc exn
+;;
+
+let string_of_string_token loc s =
+  try Token.eval_string s with
+    Failure _ as exn -> raise_with_loc loc exn
+;;
+
 let mkloc (bp, ep) =
   {Location.loc_start = bp; Location.loc_end = ep; Location.loc_ghost = false}
 ;;
@@ -47,7 +57,7 @@ value shd =
   let tab = Hashtbl.create 701 in
   fun s ->
     try Hashtbl.find tab s with
-    [ Not_found -> do Hashtbl.add tab s s; return s ]
+    [ Not_found -> do { Hashtbl.add tab s s; s } ]
 ;
 ... but no more implemented because semantic problems with Ocaml:
 let x = "bar" and y = "bar"; x.[0] <- 'c'; print_string y;;
@@ -176,8 +186,9 @@ and meth_list loc fl v =
 ;;
 
 let mktype loc tl cl tk tm =
-  {ptype_params = tl; ptype_cstrs = cl; ptype_kind = tk; ptype_manifest = tm;
-   ptype_loc = mkloc loc}
+  let (params, variance) = List.split tl in
+  {ptype_params = params; ptype_cstrs = cl; ptype_kind = tk;
+   ptype_manifest = tm; ptype_loc = mkloc loc; ptype_variance = variance}
 ;;
 let mkmutable m = if m then Mutable else Immutable;;
 let mkprivate m = if m then Private else Public;;
@@ -197,7 +208,7 @@ let type_decl tl cl =
   | t ->
       let m =
         match t with
-          TyQuo (_, s) -> if List.mem s tl then Some (ctyp t) else None
+          TyQuo (_, s) -> if List.mem_assoc s tl then Some (ctyp t) else None
         | _ -> Some (ctyp t)
       in
       mktype (loc_of_ctyp t) tl cl Ptype_abstract m
@@ -260,12 +271,13 @@ let rec module_expr_long_id =
 
 let mkwithc =
   function
-    WcTyp (loc, id, tl, ct) ->
+    WcTyp (loc, id, tpl, ct) ->
+      let (params, variance) = List.split tpl in
       long_id_of_string_list loc id,
       Pwith_type
-        {ptype_params = List.map shd tl; ptype_cstrs = [];
-         ptype_kind = Ptype_abstract; ptype_manifest = Some (ctyp ct);
-         ptype_loc = mkloc loc}
+        {ptype_params = params; ptype_cstrs = []; ptype_kind = Ptype_abstract;
+         ptype_manifest = Some (ctyp ct); ptype_loc = mkloc loc;
+         ptype_variance = variance}
   | WcMod (loc, id, m) ->
       long_id_of_string_list loc id, Pwith_module (module_type_long_id m)
 ;;
@@ -352,19 +364,24 @@ let rec patt =
             "this is not a constructor, it cannot be applied in a pattern"
       end
   | PaArr (loc, pl) -> mkpat loc (Ppat_array (List.map patt pl))
-  | PaChr (loc, s) -> mkpat loc (Ppat_constant (Const_char s))
+  | PaChr (loc, s) ->
+      mkpat loc (Ppat_constant (Const_char (char_of_char_token loc s)))
   | PaInt (loc, s) -> mkpat loc (Ppat_constant (Const_int (int_of_string s)))
+  | PaFlo (loc, s) -> mkpat loc (Ppat_constant (Const_float s))
   | PaLab (loc, _, _) -> error loc "labeled pattern not allowed here"
   | PaLid (loc, s) -> mkpat loc (Ppat_var (shd s))
   | PaOlb (loc, _, _, _) -> error loc "labeled pattern not allowed here"
   | PaOrp (loc, p1, p2) -> mkpat loc (Ppat_or (patt p1, patt p2))
   | PaRng (loc, p1, p2) ->
       begin match p1, p2 with
-        PaChr (_, c1), PaChr (_, c2) -> mkrangepat loc c1 c2
+        PaChr (loc1, c1), PaChr (loc2, c2) ->
+          let c1 = char_of_char_token loc1 c1 in
+          let c2 = char_of_char_token loc2 c2 in mkrangepat loc c1 c2
       | _ -> error loc "range pattern allowed only for characters"
       end
   | PaRec (loc, lpl) -> mkpat loc (Ppat_record (List.map mklabpat lpl))
-  | PaStr (loc, s) -> mkpat loc (Ppat_constant (Const_string (shd s)))
+  | PaStr (loc, s) ->
+      mkpat loc (Ppat_constant (Const_string (string_of_string_token loc s)))
   | PaTup (loc, pl) -> mkpat loc (Ppat_tuple (List.map patt pl))
   | PaTyc (loc, p, t) -> mkpat loc (Ppat_constraint (patt p, ctyp t))
   | PaUid (loc, s) ->
@@ -391,16 +408,20 @@ let rec sep_expr_acc l =
   | e -> (loc_of_expr e, [], e) :: l
 ;;
 
-let expr_label_long_id e =
+(*
+value expr_label_long_id e =
   match sep_expr_acc [] e with
-    [_, ml, ExLid (_, s)] -> mkli (conv_lab s) ml
-  | _ -> error (loc_of_expr e) "invalid label"
-;;
+  [ [(_, ml, ExLid _ s)] -> mkli (conv_lab s) ml
+  | _ -> error (loc_of_expr e) "invalid label" ]
+;
+*)
 
 let class_info class_expr ci =
+  let (params, variance) = List.split (snd ci.ciPrm) in
   {pci_virt = if ci.ciVir then Virtual else Concrete;
-   pci_params = snd ci.ciPrm, mkloc (fst ci.ciPrm); pci_name = ci.ciNam;
-   pci_expr = class_expr ci.ciExp; pci_loc = mkloc ci.ciLoc}
+   pci_params = params, mkloc (fst ci.ciPrm); pci_name = ci.ciNam;
+   pci_expr = class_expr ci.ciExp; pci_loc = mkloc ci.ciLoc;
+   pci_variance = variance}
 ;;
 
 let rec expr =
@@ -479,7 +500,8 @@ let rec expr =
         | _ -> error loc "bad left part of assignment"
       in
       mkexp loc e
-  | ExChr (loc, s) -> mkexp loc (Pexp_constant (Const_char s))
+  | ExChr (loc, s) ->
+      mkexp loc (Pexp_constant (Const_char (char_of_char_token loc s)))
   | ExCoe (loc, e, t) ->
       let (e, tc) =
         match e with
@@ -489,11 +511,7 @@ let rec expr =
       mkexp loc (Pexp_constraint (expr e, tc, Some (ctyp t)))
   | ExFlo (loc, s) -> mkexp loc (Pexp_constant (Const_float s))
   | ExFor (loc, i, e1, e2, df, el) ->
-      let e3 =
-        match List.rev el with
-          [] -> ExUid (loc, "()")
-        | e :: el -> ExSeq (loc, List.rev el, e)
-      in
+      let e3 = ExSeq (loc, el) in
       let df = if df then Upto else Downto in
       mkexp loc (Pexp_for (shd i, expr e1, expr e2, df, expr e3))
   | ExFun (loc, [PaLab (_, lab, p), w, e]) ->
@@ -523,16 +541,23 @@ let rec expr =
         | None -> None
       in
       mkexp loc (Pexp_record (List.map mklabexp lel, eo))
-  | ExSeq (loc, el, e) ->
-      List.fold_right (fun e1 e2 -> mkexp loc (Pexp_sequence (expr e1, e2)))
-        el (expr e)
+  | ExSeq (loc, el) ->
+      let el = List.rev el in
+      let rec loop =
+        function
+          [] -> expr (ExUid (loc, "()"))
+        | [e] -> expr e
+        | e :: el -> mkexp (loc_of_expr e) (Pexp_sequence (loop el, expr e))
+      in
+      loop el
   | ExSnd (loc, e, s) -> mkexp loc (Pexp_send (expr e, s))
   | ExSte (loc, e1, e2) ->
       mkexp loc
         (Pexp_apply
            (mkexp loc (Pexp_ident (array_function "String" "get")),
             ["", expr e1; "", expr e2]))
-  | ExStr (loc, s) -> mkexp loc (Pexp_constant (Const_string (shd s)))
+  | ExStr (loc, s) ->
+      mkexp loc (Pexp_constant (Const_string (string_of_string_token loc s)))
   | ExTry (loc, e, pel) -> mkexp loc (Pexp_try (expr e, List.map mkpwe pel))
   | ExTup (loc, el) -> mkexp loc (Pexp_tuple (List.map expr el))
   | ExTyc (loc, e, t) ->
@@ -541,12 +566,7 @@ let rec expr =
       mkexp loc (Pexp_construct (lident (conv_con s), None, true))
   | ExVrn (loc, s) -> mkexp loc (Pexp_variant (s, None))
   | ExWhi (loc, e1, el) ->
-      let e2 =
-        match List.rev el with
-          [] -> ExUid (loc, "()")
-        | e :: el -> ExSeq (loc, List.rev el, e)
-      in
-      mkexp loc (Pexp_while (expr e1, expr e2))
+      let e2 = ExSeq (loc, el) in mkexp loc (Pexp_while (expr e1, expr e2))
   | ExXnd (loc, c, _) ->
       error loc ("expression \"" ^ c ^ "\" (extension) not allowed here")
 and label_expr =
@@ -560,7 +580,7 @@ and when_expr e =
   function
     Some w -> mkexp (loc_of_expr e) (Pexp_when (expr w, expr e))
   | None -> expr e
-and mklabexp (lab, e) = expr_label_long_id lab, expr e
+and mklabexp (lab, e) = patt_label_long_id lab, expr e
 and mkideexp (ide, e) = shd ide, expr e
 and mktype_decl (c, tl, td, cl) =
   let cl =
@@ -590,6 +610,7 @@ and sig_item s l =
   | SgClt (loc, ctd) ->
       mksig loc (Psig_class_type (List.map (class_info class_type) ctd)) :: l
   | SgDcl (loc, sl) -> List.fold_right sig_item sl l
+  | SgDir (loc, _, _) -> l
   | SgExc (loc, n, tl) ->
       mksig loc (Psig_exception (shd n, List.map ctyp tl)) :: l
   | SgExt (loc, n, t, p) ->
@@ -622,11 +643,13 @@ and str_item s l =
   | StClt (loc, ctd) ->
       mkstr loc (Pstr_class_type (List.map (class_info class_type) ctd)) :: l
   | StDcl (loc, sl) -> List.fold_right str_item sl l
+  | StDir (loc, _, _) -> l
   | StExc (loc, n, tl) ->
       mkstr loc (Pstr_exception (shd n, List.map ctyp tl)) :: l
   | StExp (loc, e) -> mkstr loc (Pstr_eval (expr e)) :: l
   | StExt (loc, n, t, p) ->
       mkstr loc (Pstr_primitive (shd n, mkvalue_desc t p)) :: l
+  | StInc (loc, me) -> mkstr loc (Pstr_include (module_expr me)) :: l
   | StMod (loc, n, me) -> mkstr loc (Pstr_module (shd n, module_expr me)) :: l
   | StMty (loc, n, mt) ->
       mkstr loc (Pstr_modtype (shd n, module_type mt)) :: l
