@@ -5,7 +5,7 @@
 (*                                                                     *)
 (*        Daniel de Rauglaudre, projet Cristal, INRIA Rocquencourt     *)
 (*                                                                     *)
-(*  Copyright 1998 Institut National de Recherche en Informatique et   *)
+(*  Copyright 2001 Institut National de Recherche en Informatique et   *)
 (*  Automatique.  Distributed only by permission.                      *)
 (*                                                                     *)
 (***********************************************************************)
@@ -41,7 +41,7 @@ type ctyp =
 type 'a class_infos =
   { ciLoc : loc;
     ciVir : bool;
-    ciPrm : loc * string list;
+    ciPrm : loc * (string * (bool * bool)) list;
     ciNam : string;
     ciExp : 'a }
 ;;
@@ -53,8 +53,9 @@ type patt =
   | PaAny of loc
   | PaApp of loc * patt * patt
   | PaArr of loc * patt list
-  | PaChr of loc * char
+  | PaChr of loc * string
   | PaInt of loc * string
+  | PaFlo of loc * string
   | PaLab of loc * string * patt
   | PaLid of loc * string
   | PaOlb of loc * string * patt * expr option
@@ -74,7 +75,7 @@ and expr =
   | ExAre of loc * expr * expr
   | ExArr of loc * expr list
   | ExAss of loc * expr * expr
-  | ExChr of loc * char
+  | ExChr of loc * string
   | ExCoe of loc * expr * ctyp
   | ExFlo of loc * string
   | ExFor of loc * string * expr * expr * bool * expr list
@@ -89,8 +90,8 @@ and expr =
   | ExNew of loc * string list
   | ExOlb of loc * string * expr
   | ExOvr of loc * (string * expr) list
-  | ExRec of loc * (expr * expr) list * expr option
-  | ExSeq of loc * expr list * expr
+  | ExRec of loc * (patt * expr) list * expr option
+  | ExSeq of loc * expr list
   | ExSnd of loc * expr * string
   | ExSte of loc * expr * expr
   | ExStr of loc * string
@@ -113,16 +114,17 @@ and sig_item =
     SgCls of loc * class_type class_infos list
   | SgClt of loc * class_type class_infos list
   | SgDcl of loc * sig_item list
+  | SgDir of loc * string * expr option
   | SgExc of loc * string * ctyp list
   | SgExt of loc * string * ctyp * string list
   | SgInc of loc * module_type
   | SgMod of loc * string * module_type
   | SgMty of loc * string * module_type
   | SgOpn of loc * string list
-  | SgTyp of loc * (string * string list * ctyp * (ctyp * ctyp) list) list
+  | SgTyp of loc * type_decl list
   | SgVal of loc * string * ctyp
 and with_constr =
-    WcTyp of loc * string list * string list * ctyp
+    WcTyp of loc * string list * (string * (bool * bool)) list * ctyp
   | WcMod of loc * string list * module_type
 and module_expr =
     MeAcc of loc * module_expr * module_expr
@@ -135,14 +137,18 @@ and str_item =
     StCls of loc * class_expr class_infos list
   | StClt of loc * class_type class_infos list
   | StDcl of loc * str_item list
+  | StDir of loc * string * expr option
   | StExc of loc * string * ctyp list
   | StExp of loc * expr
   | StExt of loc * string * ctyp * string list
+  | StInc of loc * module_expr
   | StMod of loc * string * module_expr
   | StMty of loc * string * module_type
   | StOpn of loc * string list
-  | StTyp of loc * (string * string list * ctyp * (ctyp * ctyp) list) list
+  | StTyp of loc * type_decl list
   | StVal of loc * bool * (patt * expr) list
+and type_decl =
+  string * (string * (bool * bool)) list * ctyp * (ctyp * ctyp) list
 and class_type =
     CtCon of loc * string list * ctyp list
   | CtFun of loc * ctyp * class_type
@@ -171,14 +177,6 @@ and class_str_item =
   | CrVir of loc * string * bool * ctyp
 ;;
 
-type directive_param =
-  DpNon | DpStr of string | DpInt of string | DpIde of string list
-;;
-
-type phrase =
-  PhStr of loc * str_item | PhDir of loc * string * directive_param
-;;
-
 external loc_of_ctyp : ctyp -> loc = "%field0";;
 external loc_of_patt : patt -> loc = "%field0";;
 external loc_of_expr : expr -> loc = "%field0";;
@@ -186,4 +184,3 @@ external loc_of_module_type : module_type -> loc = "%field0";;
 external loc_of_module_expr : module_expr -> loc = "%field0";;
 external loc_of_sig_item : sig_item -> loc = "%field0";;
 external loc_of_str_item : str_item -> loc = "%field0";;
-external loc_of_phrase : phrase -> loc = "%field0";;
