@@ -5,14 +5,14 @@
 (*                                                                     *)
 (*        Daniel de Rauglaudre, projet Cristal, INRIA Rocquencourt     *)
 (*                                                                     *)
-(*  Copyright 1998 Institut National de Recherche en Informatique et   *)
+(*  Copyright 2001 Institut National de Recherche en Informatique et   *)
 (*  Automatique.  Distributed only by permission.                      *)
 (*                                                                     *)
 (***********************************************************************)
 
 (* Id *)
 
-let version = "3.00";;
+let version = "3.02";;
 
 let gram =
   Grammar.create
@@ -23,16 +23,16 @@ let gram =
      Token.text = fun _ -> ""}
 ;;
 
-let interf = Grammar.Entry.create gram "interface file";;
-let implem = Grammar.Entry.create gram "implementation file";;
-let top_phrase = Grammar.Entry.create gram "toplevel phrase";;
-let use_file = Grammar.Entry.create gram "source file";;
+let interf = Grammar.Entry.create gram "interf";;
+let implem = Grammar.Entry.create gram "implem";;
+let top_phrase = Grammar.Entry.create gram "top_phrase";;
+let use_file = Grammar.Entry.create gram "use_file";;
 let sig_item = Grammar.Entry.create gram "sig_item";;
 let str_item = Grammar.Entry.create gram "str_item";;
 let module_type = Grammar.Entry.create gram "module_type";;
 let module_expr = Grammar.Entry.create gram "module_expr";;
-let expr = Grammar.Entry.create gram "expression";;
-let patt = Grammar.Entry.create gram "pattern";;
+let expr = Grammar.Entry.create gram "expr";;
+let patt = Grammar.Entry.create gram "patt";;
 let ctyp = Grammar.Entry.create gram "type";;
 let let_binding = Grammar.Entry.create gram "let_binding";;
 
@@ -41,22 +41,26 @@ let class_str_item = Grammar.Entry.create gram "class_str_item";;
 let class_type = Grammar.Entry.create gram "class_type";;
 let class_expr = Grammar.Entry.create gram "class_expr";;
 
+let rec skip_to_eol cs =
+  match Stream.peek cs with
+    Some '\n' -> ()
+  | Some c -> Stream.junk cs; skip_to_eol cs
+  | _ -> ()
+;;
+let sync = ref skip_to_eol;;
+
 let input_file = ref "";;
 let output_file = ref None;;
 
-let undef x = ref (fun _ -> failwith x);;
-
-type 'a printer_t = ('a -> unit) ref;;
-let print_interf = undef "no printer";;
-let print_implem = undef "no printer";;
-
 List.iter (fun (n, f) -> Quotation.add n f)
   ["id", Quotation.ExStr (fun _ s -> "$0:" ^ s ^ "$");
    "string", Quotation.ExStr (fun _ s -> "\"" ^ String.escaped s ^ "\"")];;
 
 let quotation_dump_file = ref (None : string option);;
 
-type err_ctx = Expanding | ParsingResult of (int * int) * string | Locating;;
+type err_ctx =
+  Finding | Expanding | ParsingResult of (int * int) * string | Locating
+;;
 exception Qerror of string * err_ctx * exn;;
 
 let expand_quotation loc expander shift name str =
@@ -95,9 +99,9 @@ let handle_quotation loc proj in_expr entry reloc (name, str) =
   let shift = fst loc + shift in
   let expander =
     try Quotation.find name with
-      Not_found ->
-        let exc1 = Qerror (name, Expanding, Not_found) in
-        raise (Stdpp.Exc_located (loc, exc1))
+      exc ->
+        let exc1 = Qerror (name, Finding, exc) in
+        let loc = fst loc, shift in raise (Stdpp.Exc_located (loc, exc1))
   in
   let ast =
     match expander with
@@ -182,7 +186,8 @@ let report_quotation_error name ctx =
   Format.open_hovbox 2;
   Printf.eprintf "While %s \"%s\":"
     (match ctx with
-       Expanding -> "expanding quotation"
+       Finding -> "finding quotation"
+     | Expanding -> "expanding quotation"
      | ParsingResult (_, _) -> "parsing result of quotation"
      | Locating -> "parsing")
     name;
@@ -200,25 +205,22 @@ let report_quotation_error name ctx =
             flush oc;
             close_out oc;
             Printf.eprintf loc_fmt dump_file line c1 c2;
-            flush stderr;
-            ()
+            flush stderr
           with
             _ ->
               Printf.eprintf "Error while dumping result in file \"%s\""
                 dump_file;
               Printf.eprintf "; dump aborted.\n";
-              flush stderr;
-              ()
+              flush stderr
           end
       | None ->
           if !input_file = "" then
             Printf.eprintf
-              "\n(consider setting variable Pcaml_quotation_dump_file)\n"
+              "\n(consider setting variable Pcaml.quotation_dump_file)\n"
           else Printf.eprintf " (consider using option -QD)\n";
-          flush stderr;
-          ()
+          flush stderr
       end
-  | _ -> Printf.eprintf "\n"; flush stderr; ()
+  | _ -> Printf.eprintf "\n"; flush stderr
 ;;
 
 let print_format str =
@@ -250,16 +252,14 @@ let print_exn =
       Format.print_string ", chars ";
       Format.print_int first_char;
       Format.print_char '-';
-      Format.print_int last_char;
-      ()
+      Format.print_int last_char
   | Stream.Error str -> print_format ("Parse error: " ^ str)
   | Stream.Failure -> Format.print_string "Parse failure"
   | Token.Error str ->
-      Format.print_string "Lexing error: "; Format.print_string str; ()
-  | Failure str ->
-      Format.print_string "Failure: "; Format.print_string str; ()
+      Format.print_string "Lexing error: "; Format.print_string str
+  | Failure str -> Format.print_string "Failure: "; Format.print_string str
   | Invalid_argument str ->
-      Format.print_string "Invalid argument: "; Format.print_string str; ()
+      Format.print_string "Invalid argument: "; Format.print_string str
   | Sys_error msg ->
       Format.print_string "I/O error: "; Format.print_string msg
   | x ->
@@ -283,18 +283,23 @@ let print_exn =
             else Format.print_char '_'
           done;
           Format.print_char ')'
-        end;
-      ()
+        end
 ;;
 
 let report_error exn =
   match exn with
-    Qerror (name, ctx, exn) -> report_quotation_error name ctx; print_exn exn
+    Qerror (name, Finding, Not_found) ->
+      let name = if name = "" then !(Quotation.default) else name in
+      Format.print_flush ();
+      Format.open_hovbox 2;
+      Format.printf "Unbound quotation: \"%s\"" name;
+      Format.close_box ()
+  | Qerror (name, ctx, exn) -> report_quotation_error name ctx; print_exn exn
   | e -> print_exn exn
 ;;
 
 let warning_default_function (bp, ep) txt =
-  Printf.eprintf "<W> loc %d %d: %s\n" bp ep txt; flush stderr; ()
+  Printf.eprintf "<W> loc %d %d: %s\n" bp ep txt; flush stderr
 ;;
 
 let warning = ref warning_default_function;;
@@ -307,3 +312,91 @@ let arg_spec_list () = !arg_spec_list_ref;;
 let add_option name spec descr =
   arg_spec_list_ref := !arg_spec_list_ref @ [name, spec, descr]
 ;;
+
+(* Printers *)
+
+open Spretty;;
+
+type 'a printer_t =
+  { mutable pr_fun : string -> 'a -> string -> kont -> pretty;
+    mutable pr_levels : 'a pr_level list }
+and 'a pr_level =
+  { pr_label : string;
+    pr_box : 'a -> pretty Stream.t -> pretty;
+    mutable pr_rules : 'a pr_rule }
+and 'a pr_rule =
+  ('a, ('a curr -> 'a next -> string -> kont -> pretty Stream.t)) Extfun.t
+and 'a curr = 'a -> string -> kont -> pretty Stream.t
+and 'a next = 'a -> string -> kont -> pretty
+and kont = pretty Stream.t
+;;
+
+let pr_str_item =
+  {pr_fun =
+     (fun _ -> raise (Match_failure ("camlp4/pcaml.ml", 11503, 11509)));
+   pr_levels = []}
+;;
+let pr_sig_item =
+  {pr_fun =
+     (fun _ -> raise (Match_failure ("camlp4/pcaml.ml", 11558, 11564)));
+   pr_levels = []}
+;;
+let pr_expr =
+  {pr_fun =
+     (fun _ -> raise (Match_failure ("camlp4/pcaml.ml", 11609, 11615)));
+   pr_levels = []}
+;;
+let pr_patt =
+  {pr_fun =
+     (fun _ -> raise (Match_failure ("camlp4/pcaml.ml", 11660, 11666)));
+   pr_levels = []}
+;;
+let pr_expr_fun_args = ref Extfun.empty;;
+
+let not_impl name x =
+  let desc =
+    if Obj.is_block (Obj.repr x) then
+      "tag = " ^ string_of_int (Obj.tag (Obj.repr x))
+    else "int_val = " ^ string_of_int (Obj.magic x)
+  in
+  HVbox
+    (Stream.lsing
+       (fun _ -> S (NO, ("<pr_fun: not impl: " ^ name ^ "; " ^ desc ^ ">"))))
+;;
+
+let pr_fun name pr lab =
+  let rec loop app =
+    function
+      [] -> (fun x dg k -> not_impl name x)
+    | lev :: levl ->
+        if app || lev.pr_label = lab then
+          let next = loop true levl in
+          let rec curr x dg k = Extfun.apply lev.pr_rules x curr next dg k in
+          fun x dg k -> lev.pr_box x (curr x dg k)
+        else loop app levl
+  in
+  loop false pr.pr_levels
+;;
+
+pr_str_item.pr_fun <- pr_fun "str_item" pr_str_item;;
+pr_sig_item.pr_fun <- pr_fun "str_item" pr_sig_item;;
+pr_expr.pr_fun <- pr_fun "expr" pr_expr;;
+pr_patt.pr_fun <- pr_fun "patt" pr_patt;;
+
+let rec find_pr_level lab =
+  function
+    [] -> failwith ("level " ^ lab ^ " not found")
+  | lev :: levl -> if lev.pr_label = lab then lev else find_pr_level lab levl
+;;
+
+let undef x = ref (fun _ -> failwith x);;
+let print_interf = undef "no printer";;
+let print_implem = undef "no printer";;
+
+let top_printer pr x =
+  Format.force_newline ();
+  Spretty.print_pretty Format.print_char Format.print_string
+    Format.print_newline "<< " "   " 78 (fun _ -> ())
+    (pr.pr_fun "top" x "" Stream.sempty);
+  Format.print_string " >>"
+;;
