@@ -69,6 +69,7 @@ let rec patt floc sh =
     | PaArr (loc, x1) -> PaArr (floc loc, List.map self x1)
     | PaChr (loc, x1) -> PaChr (floc loc, x1)
     | PaInt (loc, x1) -> PaInt (floc loc, x1)
+    | PaFlo (loc, x1) -> PaFlo (floc loc, x1)
     | PaLab (loc, x1, x2) -> PaLab (floc loc, x1, self x2)
     | PaLid (loc, x1) -> PaLid (floc loc, x1)
     | PaOlb (loc, x1, x2, x3) ->
@@ -130,9 +131,9 @@ and expr floc sh =
         ExOvr (floc loc, List.map (fun (x1, x2) -> x1, self x2) x1)
     | ExRec (loc, x1, x2) ->
         ExRec
-          (floc loc, List.map (fun (x1, x2) -> self x1, self x2) x1,
+          (floc loc, List.map (fun (x1, x2) -> patt floc sh x1, self x2) x1,
            option_map self x2)
-    | ExSeq (loc, x1, x2) -> ExSeq (floc loc, List.map self x1, self x2)
+    | ExSeq (loc, x1) -> ExSeq (floc loc, List.map self x1)
     | ExSnd (loc, x1, x2) -> ExSnd (floc loc, self x1, x2)
     | ExSte (loc, x1, x2) -> ExSte (floc loc, self x1, self x2)
     | ExStr (loc, x1) -> ExStr (floc loc, x1)
@@ -172,6 +173,7 @@ and sig_item floc sh =
     | SgClt (loc, x1) ->
         SgClt (floc loc, List.map (class_infos class_type floc sh) x1)
     | SgDcl (loc, x1) -> SgDcl (floc loc, List.map self x1)
+    | SgDir (loc, x1, x2) -> SgDir (floc loc, x1, x2)
     | SgExc (loc, x1, x2) -> SgExc (floc loc, x1, List.map (ctyp floc sh) x2)
     | SgExt (loc, x1, x2, x3) -> SgExt (floc loc, x1, ctyp floc sh x2, x3)
     | SgInc (loc, x1) -> SgInc (floc loc, module_type floc sh x1)
@@ -217,9 +219,11 @@ and str_item floc sh =
     | StClt (loc, x1) ->
         StClt (floc loc, List.map (class_infos class_type floc sh) x1)
     | StDcl (loc, x1) -> StDcl (floc loc, List.map self x1)
+    | StDir (loc, x1, x2) -> StDir (floc loc, x1, x2)
     | StExc (loc, x1, x2) -> StExc (floc loc, x1, List.map (ctyp floc sh) x2)
     | StExp (loc, x1) -> StExp (floc loc, expr floc sh x1)
     | StExt (loc, x1, x2, x3) -> StExt (floc loc, x1, ctyp floc sh x2, x3)
+    | StInc (loc, x1) -> StInc (floc loc, module_expr floc sh x1)
     | StMod (loc, x1, x2) -> StMod (floc loc, x1, module_expr floc sh x2)
     | StMty (loc, x1, x2) -> StMty (floc loc, x1, module_type floc sh x2)
     | StOpn (loc, x1) -> StOpn (floc loc, x1)
