@@ -5,7 +5,7 @@
 (*                                                                     *)
 (*        Daniel de Rauglaudre, projet Cristal, INRIA Rocquencourt     *)
 (*                                                                     *)
-(*  Copyright 1998 Institut National de Recherche en Informatique et   *)
+(*  Copyright 2001 Institut National de Recherche en Informatique et   *)
 (*  Automatique.  Distributed only by permission.                      *)
 (*                                                                     *)
 (***********************************************************************)
@@ -29,32 +29,29 @@ let print_str s = print_string ("\"" ^ String.escaped s ^ "\"");;
 
 let rec print_symbol =
   function
-    Slist0 s -> print_string "LIST0"; print_string " "; print_symbol1 s; ()
+    Slist0 s -> print_string "LIST0"; print_string " "; print_symbol1 s
   | Slist0sep (s, t) ->
       print_string "LIST0";
       print_string " ";
       print_symbol1 s;
       print_string " SEP ";
-      print_symbol1 t;
-      ()
-  | Slist1 s -> print_string "LIST1"; print_string " "; print_symbol1 s; ()
+      print_symbol1 t
+  | Slist1 s -> print_string "LIST1"; print_string " "; print_symbol1 s
   | Slist1sep (s, t) ->
       print_string "LIST1";
       print_string " ";
       print_symbol1 s;
       print_string " SEP ";
-      print_symbol1 t;
-      ()
-  | Sopt s -> print_string "OPT "; print_symbol1 s; ()
+      print_symbol1 t
+  | Sopt s -> print_string "OPT "; print_symbol1 s
   | Stoken (con, prm) when con <> "" && prm <> "" ->
-      print_string con; print_space (); print_str prm; ()
+      print_string con; print_space (); print_str prm
   | Snterml (e, l) ->
       print_string e.ename;
       print_space ();
       print_string "LEVEL";
       print_space ();
-      print_str l;
-      ()
+      print_str l
   | s -> print_symbol1 s
 and print_symbol1 =
   function
@@ -64,7 +61,7 @@ and print_symbol1 =
   | Snext -> print_string "NEXT"
   | Stoken (con, "") -> print_string con
   | Stree t -> print_level print_space (flatten_tree t)
-  | s -> print_string "("; print_symbol s; print_string ")"; ()
+  | s -> print_string "("; print_symbol s; print_string ")"
 and print_rule symbols =
   open_hovbox 0;
   let _ =
@@ -82,12 +79,10 @@ and print_level print_space rules =
   let _ =
     List.fold_left
       (fun sep rule ->
-         sep ();
-         print_rule rule;
-         fun () -> print_space (); print_string "| "; ())
+         sep (); print_rule rule; fun () -> print_space (); print_string "| ")
       (fun () -> ()) rules
   in
-  print_string " ]"; close_box (); ()
+  print_string " ]"; close_box ()
 ;;
 
 let print_levels elev =
@@ -102,9 +97,7 @@ let print_levels elev =
          open_hovbox 2;
          begin match lev.lname with
            Some n ->
-             print_string ("\"" ^ String.escaped n ^ "\"");
-             print_break 1 2;
-             ()
+             print_string ("\"" ^ String.escaped n ^ "\""); print_break 1 2
          | _ -> ()
          end;
          begin match lev.assoc with
@@ -115,7 +108,7 @@ let print_levels elev =
          close_box ();
          print_break 1 2;
          print_level force_newline rules;
-         fun () -> print_cut (); print_string "| "; ())
+         fun () -> print_cut (); print_string "| ")
       (fun () -> ()) elev
   in
   ()
@@ -130,8 +123,7 @@ let print_entry e =
   end;
   print_string " ]";
   close_box ();
-  print_newline ();
-  ()
+  print_newline ()
 ;;
 
 type g = Gramext.grammar;;
@@ -184,6 +176,85 @@ and name_of_tree_failed entry =
   | DeadEnd | LocAct (_, _) -> "???"
 ;;
 
+let search_tree_in_entry prev_symb tree =
+  function
+    Dlevels levels ->
+      let rec search_levels =
+        function
+          [] -> tree
+        | level :: levels ->
+            match search_level level with
+              Some tree -> tree
+            | None -> search_levels levels
+      and search_level level =
+        match search_tree level.lsuffix with
+          Some t -> Some (Node {node = Sself; son = t; brother = DeadEnd})
+        | None -> search_tree level.lprefix
+      and search_tree t =
+        if tree <> DeadEnd && t == tree then Some t
+        else
+          match t with
+            Node n ->
+              begin match search_symbol n.node with
+                Some symb ->
+                  Some (Node {node = symb; son = n.son; brother = DeadEnd})
+              | None ->
+                  match search_tree n.son with
+                    Some t ->
+                      Some (Node {node = n.node; son = t; brother = DeadEnd})
+                  | None -> search_tree n.brother
+              end
+          | _ -> None
+      and search_symbol symb =
+        match symb with
+          Snterm _ | Snterml (_, _) | Slist0 _ | Slist0sep (_, _) | Slist1 _ |
+          Slist1sep (_, _) | Sopt _ | Stoken _ | Stree _
+          when symb == prev_symb ->
+            Some symb
+        | Slist0 symb ->
+            begin match search_symbol symb with
+              Some symb -> Some (Slist0 symb)
+            | None -> None
+            end
+        | Slist0sep (symb, sep) ->
+            begin match search_symbol symb with
+              Some symb -> Some (Slist0sep (symb, sep))
+            | None ->
+                match search_symbol sep with
+                  Some sep -> Some (Slist0sep (symb, sep))
+                | None -> None
+            end
+        | Slist1 symb ->
+            begin match search_symbol symb with
+              Some symb -> Some (Slist1 symb)
+            | None -> None
+            end
+        | Slist1sep (symb, sep) ->
+            begin match search_symbol symb with
+              Some symb -> Some (Slist1sep (symb, sep))
+            | None ->
+                match search_symbol sep with
+                  Some sep -> Some (Slist1sep (symb, sep))
+                | None -> None
+            end
+        | Sopt symb ->
+            begin match search_symbol symb with
+              Some symb -> Some (Sopt symb)
+            | None -> None
+            end
+        | Stree t ->
+            begin match search_tree t with
+              Some t -> Some (Stree t)
+            | None -> None
+            end
+        | _ -> None
+      in
+      search_levels levels
+  | _ -> tree
+;;
+
+let error_verbose = ref false;;
+
 let tree_failed entry prev_symb_result prev_symb tree =
   let txt = name_of_tree_failed entry tree in
   let txt =
@@ -215,6 +286,25 @@ let tree_failed entry prev_symb_result prev_symb tree =
     | Sopt _ | Stree _ -> txt ^ " expected"
     | _ -> txt ^ " expected after " ^ name_of_symbol entry prev_symb
   in
+  if !error_verbose then
+    begin
+      let tree = search_tree_in_entry prev_symb tree entry.edesc in
+      set_formatter_out_channel stderr;
+      open_vbox 0;
+      print_newline ();
+      print_string "----------------------------------";
+      print_newline ();
+      printf "Parse error in entry [%s], rule:" entry.ename;
+      print_break 0 2;
+      open_vbox 0;
+      print_level force_newline (flatten_tree tree);
+      close_box ();
+      print_newline ();
+      print_string "----------------------------------";
+      print_newline ();
+      close_box ();
+      print_newline ()
+    end;
   txt ^ " (in [" ^ entry.ename ^ "])"
 ;;
 
@@ -433,10 +523,10 @@ and parser_of_token_list p1 tokl =
   let rec loop n =
     function
       [p_con, ""] ->
-        let ps (strm__ : _ Stream.t) =
-          match peek_nth n strm__ with
-            Some (con, prm) when con = p_con ->
-              for i = 1 to n do Stream.junk strm__ done; Obj.repr prm
+        let ps strm =
+          match peek_nth n strm with
+            Some (con, prm) when p_con = "ANY" || con = p_con ->
+              for i = 1 to n do Stream.junk strm done; Obj.repr prm
           | _ -> raise Stream.Failure
         in
         (fun (strm__ : _ Stream.t) ->
@@ -448,10 +538,11 @@ and parser_of_token_list p1 tokl =
            in
            app act a)
     | [p_con, p_prm] ->
-        let ps (strm__ : _ Stream.t) =
-          match peek_nth n strm__ with
-            Some (con, prm) when con = p_con && prm = p_prm ->
-              for i = 1 to n do Stream.junk strm__ done; Obj.repr prm
+        let ps strm =
+          match peek_nth n strm with
+            Some (con, prm)
+            when (p_con = "ANY" || con = p_con) && prm = p_prm ->
+              for i = 1 to n do Stream.junk strm done; Obj.repr prm
           | _ -> raise Stream.Failure
         in
         (fun (strm__ : _ Stream.t) ->
@@ -463,18 +554,20 @@ and parser_of_token_list p1 tokl =
            in
            app act a)
     | (p_con, "") :: tokl ->
-        let ps (strm__ : _ Stream.t) =
-          match peek_nth n strm__ with
-            Some (con, prm) when con = p_con -> prm
+        let ps strm =
+          match peek_nth n strm with
+            Some (con, prm) when p_con = "ANY" || con = p_con -> prm
           | _ -> raise Stream.Failure
         in
         let p1 = loop (n + 1) tokl in
         (fun (strm__ : _ Stream.t) ->
            let a = ps strm__ in let act = p1 strm__ in app act a)
     | (p_con, p_prm) :: tokl ->
-        let ps (strm__ : _ Stream.t) =
-          match peek_nth n strm__ with
-            Some (con, prm) when con = p_con && prm = p_prm -> prm
+        let ps strm =
+          match peek_nth n strm with
+            Some (con, prm)
+            when (p_con = "ANY" || con = p_con) && prm = p_prm ->
+              prm
           | _ -> raise Stream.Failure
         in
         let p1 = loop (n + 1) tokl in
@@ -577,6 +670,17 @@ and parser_of_symbol entry nlevn =
       (fun (strm__ : _ Stream.t) -> e.estart (level_number e l) strm__)
   | Sself -> (fun (strm__ : _ Stream.t) -> entry.estart 0 strm__)
   | Snext -> (fun (strm__ : _ Stream.t) -> entry.estart nlevn strm__)
+  | Stoken ("ANY", v) ->
+      if v = "" then
+        fun (strm__ : _ Stream.t) ->
+          match Stream.peek strm__ with
+            Some (_, x) -> Stream.junk strm__; Obj.repr x
+          | _ -> raise Stream.Failure
+      else
+        (fun (strm__ : _ Stream.t) ->
+           match Stream.peek strm__ with
+             Some (_, x) when x = v -> Stream.junk strm__; Obj.repr x
+           | _ -> raise Stream.Failure)
   | Stoken tok ->
       match entry.egram.glexer.Token.tparse tok with
         Some f -> (Obj.magic f : Token.t Stream.t -> Obj.t)
@@ -659,9 +763,14 @@ let continue_parser_of_entry entry =
   | Dparser p -> fun levn bp a (strm__ : _ Stream.t) -> raise Stream.Failure
 ;;
 
+let empty_entry ename levn strm =
+  raise (Stream.Error ("entry [" ^ ename ^ "] is empty"))
+;;
+
 let rec start_parser_of_entry entry =
   match entry.edesc with
-    Dlevels elev -> start_parser_of_levels entry 0 elev
+    Dlevels [] -> empty_entry entry.ename
+  | Dlevels elev -> start_parser_of_levels entry 0 elev
   | Dparser p -> fun levn strm -> p strm
 ;;
 
@@ -669,19 +778,21 @@ let parse_parsable entry efun (cs, (ts, fun_loc)) =
   let restore = let old_floc = !floc in fun () -> floc := old_floc in
   floc := fun_loc;
   try let r = efun ts in restore (); r with
-    exc ->
-      let (loc, exc) =
-        match exc with
-          Stream.Failure ->
-            (try fun_loc (Stream.count ts) with
-               _ -> Stream.count cs, Stream.count cs + 1),
-            Stream.Error ("illegal begin of " ^ entry.ename)
-        | Stream.Error _ ->
-            (try fun_loc (Stream.count ts) with
-               _ -> Stream.count cs, Stream.count cs + 1),
-            exc
-        | exc -> (Stream.count cs, Stream.count cs + 1), exc
+    Stream.Failure ->
+      let loc =
+        try fun_loc (Stream.count ts) with
+          _ -> Stream.count cs, Stream.count cs + 1
       in
+      restore ();
+      raise_with_loc loc (Stream.Error ("illegal begin of " ^ entry.ename))
+  | Stream.Error _ as exc ->
+      let loc =
+        try fun_loc (Stream.count ts) with
+          _ -> Stream.count cs, Stream.count cs + 1
+      in
+      restore (); raise_with_loc loc exc
+  | exc ->
+      let loc = Stream.count cs, Stream.count cs + 1 in
       restore (); raise_with_loc loc exc
 ;;
 
@@ -704,10 +815,9 @@ let extend_entry entry position rules =
          let f = start_parser_of_entry entry in
          entry.estart <- f; f lev strm);
     entry.econtinue <-
-      (fun lev bp a strm ->
-         let f = continue_parser_of_entry entry in
-         entry.econtinue <- f; f lev bp a strm);
-    ()
+      fun lev bp a strm ->
+        let f = continue_parser_of_entry entry in
+        entry.econtinue <- f; f lev bp a strm
   with
     Token.Error s ->
       Printf.eprintf "Lexer initialization error.\n%s\n"
@@ -730,10 +840,8 @@ let extend entry_rules_list =
              end
        | None -> gram := Some entry.egram
        end;
-       extend_entry entry position rules;
-       ())
-    entry_rules_list;
-  ()
+       extend_entry entry position rules)
+    entry_rules_list
 ;;
 
 (* Deleting a rule *)
@@ -750,8 +858,7 @@ let delete_rule entry sl =
       entry.econtinue <-
         (fun lev bp a strm ->
            let f = continue_parser_of_entry entry in
-           entry.econtinue <- f; f lev bp a strm);
-      ()
+           entry.econtinue <- f; f lev bp a strm)
   | _ -> ()
 ;;
 
@@ -760,14 +867,12 @@ let delete_rule entry sl =
 let clear_entry e =
   e.estart <- (fun _ (strm__ : _ Stream.t) -> raise Stream.Failure);
   e.econtinue <- (fun _ _ _ (strm__ : _ Stream.t) -> raise Stream.Failure);
-  begin match e.edesc with
+  match e.edesc with
     Dlevels _ -> e.edesc <- Dlevels []
   | Dparser _ -> ()
-  end;
-  ()
 ;;
 
-let reinit_gram g lexer = Hashtbl.clear g.gtokens; g.glexer <- lexer; ();;
+let reinit_gram g lexer = Hashtbl.clear g.gtokens; g.glexer <- lexer;;
 
 module Unsafe =
   struct let clear_entry = clear_entry;; let reinit_gram = reinit_gram;; end
@@ -777,7 +882,7 @@ exception EntryFound of g_entry;;
 let find_entry e s =
   let rec find_levels levs =
     try
-      List.iter (fun lev -> find_tree lev.lsuffix; find_tree lev.lprefix; ())
+      List.iter (fun lev -> find_tree lev.lsuffix; find_tree lev.lprefix)
         levs;
       raise Not_found
     with
@@ -797,7 +902,7 @@ let find_entry e s =
   and find_tree =
     function
       Node {node = s; brother = bro; son = son} ->
-        find_symbol s; find_tree bro; find_tree son; ()
+        find_symbol s; find_tree bro; find_tree son
     | _ -> ()
   in
   match e.edesc with
@@ -811,8 +916,7 @@ module Entry =
   struct
     type 'a e = g_entry;;
     let create g n =
-      {egram = g; ename = n;
-       estart = (fun _ (strm__ : _ Stream.t) -> raise Stream.Failure);
+      {egram = g; ename = n; estart = empty_entry n;
        econtinue = (fun _ _ _ (strm__ : _ Stream.t) -> raise Stream.Failure);
        edesc = Dlevels []}
     ;;
@@ -892,8 +996,7 @@ module Make (L : LexerType) : S =
       struct
         type 'a e = g_entry;;
         let create n =
-          {egram = gram; ename = n;
-           estart = (fun _ (strm__ : _ Stream.t) -> raise Stream.Failure);
+          {egram = gram; ename = n; estart = empty_entry n;
            econtinue =
              (fun _ _ _ (strm__ : _ Stream.t) -> raise Stream.Failure);
            edesc = Dlevels []}
