@@ -14,6 +14,10 @@
 
 (* Module [Grammar]: extensible grammars *)
 
+(* This module implements the Camlp4 extensible grammars system.
+   Grammars entries can be extended using the [EXTEND] statement,
+   added by loading the Camlp4 [pa_extend.cmo] file. *)
+
 type g;;
     (* The type for grammars, holding entries. *)
 val create : Token.lexer -> g;;
@@ -60,6 +64,10 @@ module Entry :
 val of_entry : 'a Entry.e -> g;;
     (* Return the grammar associated with an entry. *)
 
+val error_verbose : bool ref;;
+    (* Flag for displaying more information in case of parsing error;
+       default = [False] *)
+
 val warning_verbose : bool ref;;
     (* Flag for displaying warnings while extension; default = [True] *)
 
