@@ -5,7 +5,7 @@
 (*                                                                     *)
 (*        Daniel de Rauglaudre, projet Cristal, INRIA Rocquencourt     *)
 (*                                                                     *)
-(*  Copyright 1998 Institut National de Recherche en Informatique et   *)
+(*  Copyright 2001 Institut National de Recherche en Informatique et   *)
 (*  Automatique.  Distributed only by permission.                      *)
 (*                                                                     *)
 (***********************************************************************)
@@ -34,13 +34,11 @@ let get_buff len = String.sub !buff 0 len;;
 
 (* The lexer *)
 
-let valch x = Char.code x - Char.code '0';;
-
 let rec ident len (strm__ : _ Stream.t) =
   match Stream.peek strm__ with
     Some
-      ('A'..'Z' | 'a'..'z' | ''..'' | ''..'' | ''..'' | '0'..'9' | '_' |
-       '\'' as c) ->
+      ('A'..'Z' | 'a'..'z' | '\192'..'\214' | '\216'..'\246' |
+       '\248'..'\255' | '0'..'9' | '_' | '\'' as c) ->
       Stream.junk strm__; ident (store len c) strm__
   | _ -> len
 and ident2 len (strm__ : _ Stream.t) =
@@ -53,9 +51,10 @@ and ident2 len (strm__ : _ Stream.t) =
 and ident3 len (strm__ : _ Stream.t) =
   match Stream.peek strm__ with
     Some
-      ('0'..'9' | 'A'..'Z' | 'a'..'z' | ''..'' | ''..'' | ''..'' | '_' |
-       '!' | '%' | '&' | '*' | '+' | '-' | '.' | '/' | ':' | '<' | '=' | '>' |
-       '?' | '@' | '^' | '|' | '~' | '\'' | '$' as c) ->
+      ('0'..'9' | 'A'..'Z' | 'a'..'z' | '\192'..'\214' | '\216'..'\246' |
+       '\248'..'\255' | '_' | '!' | '%' | '&' | '*' | '+' | '-' | '.' | '/' |
+       ':' | '<' | '=' | '>' | '?' | '@' | '^' | '|' | '~' | '\'' | '$' as c
+         ) ->
       Stream.junk strm__; ident3 (store len c) strm__
   | _ -> len
 and base_number len (strm__ : _ Stream.t) =
@@ -107,108 +106,72 @@ and end_exponent_part len (strm__ : _ Stream.t) =
   | _ -> "FLOAT", get_buff len
 ;;
 
-let rec escape (strm__ : _ Stream.t) =
-  match Stream.peek strm__ with
-    Some 'n' -> Stream.junk strm__; '\n'
-  | Some 'r' -> Stream.junk strm__; '\013'
-  | Some 't' -> Stream.junk strm__; '\t'
-  | Some 'b' -> Stream.junk strm__; '\008'
-  | Some '\\' -> Stream.junk strm__; '\\'
-  | Some ('0'..'9' as c) ->
-      Stream.junk strm__;
-      let cod =
-        try escape1 (valch c) strm__ with
-          Stream.Failure -> raise (Stream.Error "")
-      in
-      Char.chr cod
-  | _ -> raise Stream.Failure
-and escape1 cod (strm__ : _ Stream.t) =
-  match Stream.peek strm__ with
-    Some ('0'..'9' as c) ->
-      Stream.junk strm__;
-      begin try escape2 (10 * cod + valch c) strm__ with
-        Stream.Failure -> raise (Stream.Error "")
-      end
-  | _ -> cod
-and escape2 cod (strm__ : _ Stream.t) =
-  match Stream.peek strm__ with
-    Some ('0'..'9' as c) -> Stream.junk strm__; 10 * cod + valch c
-  | _ -> cod
-;;
-
-let escape_string (strm__ : _ Stream.t) =
-  try escape strm__ with
-    Stream.Failure ->
-      match Stream.peek strm__ with
-        Some '"' -> Stream.junk strm__; '"'
-      | _ -> raise Stream.Failure
-;;
-let escape_char (strm__ : _ Stream.t) =
-  try escape strm__ with
-    Stream.Failure ->
-      match Stream.peek strm__ with
-        Some '\'' -> Stream.junk strm__; '\''
-      | _ -> raise Stream.Failure
-;;
-
 let rec skip_spaces (strm__ : _ Stream.t) =
   match Stream.peek strm__ with
-    Some (' ' | '\n' | '\013' | '\t' | '\026' | '\012') ->
+    Some (' ' | '\010' | '\013' | '\t' | '\026' | '\012') ->
       Stream.junk strm__; skip_spaces strm__
   | _ -> ()
 ;;
 
-let rec skip_indent (strm__ : _ Stream.t) =
-  match Stream.peek strm__ with
-    Some (' ' | '\t') -> Stream.junk strm__; skip_indent strm__
-  | _ -> ()
-;;
-
-let skip_opt_linefeed (strm__ : _ Stream.t) =
-  match Stream.peek strm__ with
-    Some '\n' -> Stream.junk strm__; ()
-  | _ -> ()
-;;
-
 let error_on_unknown_keywords = ref false;;
+let err loc msg = raise_with_loc loc (Token.Error msg);;
 
-let next_token_fun find_id_kwd find_spe_kwd =
-  let err bp ep msg = raise_with_loc (bp, ep) (Token.Error msg) in
-  let keyword_or_error (bp, ep) s =
-    try "", find_spe_kwd s with
+let next_token_fun find_kwd =
+  let keyword_or_error loc s =
+    try ("", find_kwd s), loc with
       Not_found ->
-        if !error_on_unknown_keywords then err bp ep ("illegal token: " ^ s)
-        else "", s
+        if !error_on_unknown_keywords then err loc ("illegal token: " ^ s)
+        else ("", s), loc
   in
   let rec next_token (strm__ : _ Stream.t) =
     let bp = Stream.count strm__ in
     match Stream.peek strm__ with
-      Some ('A'..'Z' | ''..'' | ''..'' as c) ->
+      Some (' ' | '\010' | '\013' | '\t' | '\026' | '\012') ->
+        Stream.junk strm__; next_token strm__
+    | Some '(' -> Stream.junk strm__; left_paren bp strm__
+    | Some '#' ->
+        Stream.junk strm__; let s = strm__ in spaces_tabs s; linenum bp s
+    | Some ('A'..'Z' | '\192'..'\214' | '\216'..'\222' as c) ->
         Stream.junk strm__;
         let s = strm__ in
         let id = get_buff (ident (store 0 c) s) in
-        begin try "", find_id_kwd id with
-          Not_found -> "UIDENT", id
-        end
-    | Some ('a'..'z' | ''..'' | ''..'' | '_' as c) ->
+        let loc = bp, Stream.count s in
+        (try "", find_kwd id with
+           Not_found -> "UIDENT", id),
+        loc
+    | Some ('a'..'z' | '\223'..'\246' | '\248'..'\255' | '_' as c) ->
         Stream.junk strm__;
         let s = strm__ in
         let id = get_buff (ident (store 0 c) s) in
-        begin try "", find_id_kwd id with
-          Not_found -> "LIDENT", id
-        end
-    | Some ('1'..'9' as c) -> Stream.junk strm__; number (store 0 c) strm__
-    | Some '0' -> Stream.junk strm__; base_number (store 0 '0') strm__
+        let loc = bp, Stream.count s in
+        (try "", find_kwd id with
+           Not_found -> "LIDENT", id),
+        loc
+    | Some ('1'..'9' as c) ->
+        Stream.junk strm__;
+        let tok = number (store 0 c) strm__ in
+        let loc = bp, Stream.count strm__ in tok, loc
+    | Some '0' ->
+        Stream.junk strm__;
+        let tok = base_number (store 0 '0') strm__ in
+        let loc = bp, Stream.count strm__ in tok, loc
     | Some '\'' ->
         Stream.junk strm__;
         let s = strm__ in
-        let ep = Stream.count strm__ in
         begin match Stream.npeek 2 s with
-          [_; '\''] | ['\\'; _] -> "CHAR", String.make 1 (char bp s)
-        | _ -> keyword_or_error (bp, ep) "'"
+          [_; '\''] | ['\\'; _] ->
+            let tok = "CHAR", char bp 0 s in
+            let loc = bp, Stream.count s in tok, loc
+        | _ -> keyword_or_error (bp, Stream.count s) "'"
         end
-    | Some '"' -> Stream.junk strm__; "STRING", string bp 0 strm__
-    | Some '$' -> Stream.junk strm__; locate_or_antiquot bp 0 strm__
+    | Some '"' ->
+        Stream.junk strm__;
+        let tok = "STRING", string bp 0 strm__ in
+        let loc = bp, Stream.count strm__ in tok, loc
+    | Some '$' ->
+        Stream.junk strm__;
+        let tok = dollar bp 0 strm__ in
+        let loc = bp, Stream.count strm__ in tok, loc
     | Some ('!' | '=' | '@' | '^' | '&' | '+' | '-' | '*' | '/' | '%' as c) ->
         Stream.junk strm__;
         let id = get_buff (ident2 (store 0 c) strm__) in
@@ -231,7 +194,7 @@ let next_token_fun find_id_kwd find_spe_kwd =
                 with
                   Stream.Failure -> raise (Stream.Error "")
               in
-              t, get_buff len
+              let ep = Stream.count strm__ in (t, get_buff len), (bp, ep)
           | _ ->
               let id = get_buff (ident2 (store 0 c) strm__) in
               keyword_or_error (bp, Stream.count strm__) id
@@ -256,7 +219,7 @@ let next_token_fun find_id_kwd find_spe_kwd =
                 with
                   Stream.Failure -> raise (Stream.Error "")
               in
-              t, get_buff len
+              let ep = Stream.count strm__ in (t, get_buff len), (bp, ep)
           | _ ->
               let id = get_buff (ident2 (store 0 c) strm__) in
               keyword_or_error (bp, Stream.count strm__) id
@@ -327,17 +290,25 @@ let next_token_fun find_id_kwd find_spe_kwd =
             Stream.Failure -> raise (Stream.Error "")
         in
         let ep = Stream.count strm__ in keyword_or_error (bp, ep) id
-    | Some '\\' -> Stream.junk strm__; "LIDENT", get_buff (ident3 0 strm__)
+    | Some '\\' ->
+        Stream.junk strm__;
+        let ep = Stream.count strm__ in
+        ("LIDENT", get_buff (ident3 0 strm__)), (bp, ep)
     | Some c ->
         Stream.junk strm__;
         let ep = Stream.count strm__ in
         keyword_or_error (bp, ep) (String.make 1 c)
-    | _ -> raise Stream.Failure
+    | _ -> let _ = Stream.empty strm__ in ("EOI", ""), (bp, succ bp)
   and less bp (strm__ : _ Stream.t) =
     match Stream.peek strm__ with
       Some '<' ->
         Stream.junk strm__;
-        "QUOTATION", ":" ^ get_buff (quotation bp 0 strm__)
+        let len =
+          try quotation bp 0 strm__ with
+            Stream.Failure -> raise (Stream.Error "")
+        in
+        let ep = Stream.count strm__ in
+        ("QUOTATION", ":" ^ get_buff len), (bp, ep)
     | Some ':' ->
         Stream.junk strm__;
         let i =
@@ -347,75 +318,47 @@ let next_token_fun find_id_kwd find_spe_kwd =
         begin match Stream.peek strm__ with
           Some '<' ->
             Stream.junk strm__;
-            "QUOTATION", i ^ ":" ^ get_buff (quotation bp 0 strm__)
+            let len =
+              try quotation bp 0 strm__ with
+                Stream.Failure -> raise (Stream.Error "")
+            in
+            let ep = Stream.count strm__ in
+            ("QUOTATION", i ^ ":" ^ get_buff len), (bp, ep)
         | _ -> raise (Stream.Error "character '<' expected")
         end
     | _ ->
+        let len = ident2 (store 0 '<') strm__ in
         let ep = Stream.count strm__ in
-        let id = get_buff (ident2 (store 0 '<') strm__) in
-        keyword_or_error (bp, ep) id
+        let id = get_buff len in keyword_or_error (bp, ep) id
   and string bp len (strm__ : _ Stream.t) =
     match Stream.peek strm__ with
       Some '"' -> Stream.junk strm__; get_buff len
     | Some '\\' ->
         Stream.junk strm__;
-        let len =
-          try
-            match Stream.peek strm__ with
-              Some '\n' ->
-                Stream.junk strm__;
-                let _ =
-                  try skip_indent strm__ with
-                    Stream.Failure -> raise (Stream.Error "")
-                in
-                len
-            | Some '\013' ->
-                Stream.junk strm__;
-                let _ =
-                  try skip_opt_linefeed strm__ with
-                    Stream.Failure -> raise (Stream.Error "")
-                in
-                let _ =
-                  try skip_indent strm__ with
-                    Stream.Failure -> raise (Stream.Error "")
-                in
-                len
-            | _ ->
-                match
-                  try Some (escape_string strm__) with
-                    Stream.Failure -> None
-                with
-                  Some c -> store len c
-                | _ ->
-                    match Stream.peek strm__ with
-                      Some c -> Stream.junk strm__; store (store len '\\') c
-                    | _ -> raise Stream.Failure
-          with
-            Stream.Failure -> raise (Stream.Error "")
-        in
-        string bp len strm__
+        begin match Stream.peek strm__ with
+          Some c ->
+            Stream.junk strm__; string bp (store (store len '\\') c) strm__
+        | _ -> raise (Stream.Error "")
+        end
     | Some c -> Stream.junk strm__; string bp (store len c) strm__
-    | _ -> let ep = Stream.count strm__ in err bp ep "string not terminated"
-  and char bp (strm__ : _ Stream.t) =
+    | _ ->
+        let ep = Stream.count strm__ in err (bp, ep) "string not terminated"
+  and char bp len (strm__ : _ Stream.t) =
     match Stream.peek strm__ with
-      Some '\\' ->
+      Some '\'' ->
         Stream.junk strm__;
-        let c =
-          try escape_char strm__ with
-            Stream.Failure -> raise (Stream.Error "escape character expected")
-        in
-        begin match Stream.peek strm__ with
-          Some '\'' -> Stream.junk strm__; c
-        | _ -> raise (Stream.Error "quote expected")
-        end
-    | Some c ->
+        let s = strm__ in
+        if len = 0 then char bp (store len '\'') s else get_buff len
+    | Some '\\' ->
         Stream.junk strm__;
         begin match Stream.peek strm__ with
-          Some '\'' -> Stream.junk strm__; c
-        | _ -> raise (Stream.Error "quote expected")
+          Some c ->
+            Stream.junk strm__; char bp (store (store len '\\') c) strm__
+        | _ -> raise (Stream.Error "")
         end
-    | _ -> let ep = Stream.count strm__ in err bp ep "char not terminated"
-  and locate_or_antiquot bp len (strm__ : _ Stream.t) =
+    | Some c -> Stream.junk strm__; char bp (store len c) strm__
+    | _ -> let ep = Stream.count strm__ in err (bp, ep) "char not terminated"
+  and dollar bp len (strm__ : _ Stream.t) =
     match Stream.peek strm__ with
       Some '$' -> Stream.junk strm__; "ANTIQUOT", ":" ^ get_buff len
     | Some ('a'..'z' | 'A'..'Z' as c) ->
@@ -439,7 +382,7 @@ let next_token_fun find_id_kwd find_spe_kwd =
         "ANTIQUOT", ":" ^ locate_or_antiquot_rest bp (store len c) strm__
     | _ ->
         let ep = Stream.count strm__ in
-        err bp ep "antiquotation not terminated"
+        err (bp, ep) "antiquotation not terminated"
   and maybe_locate bp len (strm__ : _ Stream.t) =
     match Stream.peek strm__ with
       Some '$' -> Stream.junk strm__; "ANTIQUOT", ":" ^ get_buff len
@@ -461,7 +404,7 @@ let next_token_fun find_id_kwd find_spe_kwd =
         "ANTIQUOT", ":" ^ locate_or_antiquot_rest bp (store len c) strm__
     | _ ->
         let ep = Stream.count strm__ in
-        err bp ep "antiquotation not terminated"
+        err (bp, ep) "antiquotation not terminated"
   and antiquot bp len (strm__ : _ Stream.t) =
     match Stream.peek strm__ with
       Some '$' -> Stream.junk strm__; "ANTIQUOT", ":" ^ get_buff len
@@ -484,7 +427,7 @@ let next_token_fun find_id_kwd find_spe_kwd =
         "ANTIQUOT", ":" ^ locate_or_antiquot_rest bp (store len c) strm__
     | _ ->
         let ep = Stream.count strm__ in
-        err bp ep "antiquotation not terminated"
+        err (bp, ep) "antiquotation not terminated"
   and locate_or_antiquot_rest bp len (strm__ : _ Stream.t) =
     match Stream.peek strm__ with
       Some '$' -> Stream.junk strm__; get_buff len
@@ -500,7 +443,7 @@ let next_token_fun find_id_kwd find_spe_kwd =
         Stream.junk strm__; locate_or_antiquot_rest bp (store len c) strm__
     | _ ->
         let ep = Stream.count strm__ in
-        err bp ep "antiquotation not terminated"
+        err (bp, ep) "antiquotation not terminated"
   and quotation bp len (strm__ : _ Stream.t) =
     match Stream.peek strm__ with
       Some '>' -> Stream.junk strm__; maybe_end_quotation bp len strm__
@@ -520,7 +463,8 @@ let next_token_fun find_id_kwd find_spe_kwd =
         quotation bp len strm__
     | Some c -> Stream.junk strm__; quotation bp (store len c) strm__
     | _ ->
-        let ep = Stream.count strm__ in err bp ep "quotation not terminated"
+        let ep = Stream.count strm__ in
+        err (bp, ep) "quotation not terminated"
   and maybe_nested_quotation bp len (strm__ : _ Stream.t) =
     match Stream.peek strm__ with
       Some '<' ->
@@ -545,48 +489,31 @@ let next_token_fun find_id_kwd find_spe_kwd =
     match Stream.peek strm__ with
       Some '>' -> Stream.junk strm__; len
     | _ -> quotation bp (store len '>') strm__
-  in
-  let rec next_token_loc (strm__ : _ Stream.t) =
-    let bp = Stream.count strm__ in
-    match Stream.peek strm__ with
-      Some (' ' | '\n' | '\013' | '\t' | '\026' | '\012') ->
-        Stream.junk strm__; next_token_loc strm__
-    | Some '(' -> Stream.junk strm__; maybe_comment bp strm__
-    | Some '#' ->
-        Stream.junk strm__;
-        let _ =
-          try spaces_tabs strm__ with
-            Stream.Failure -> raise (Stream.Error "")
-        in
-        begin try linenum bp strm__ with
-          Stream.Failure -> raise (Stream.Error "")
-        end
-    | _ ->
-        match
-          try Some (next_token strm__) with
-            Stream.Failure -> None
-        with
-          Some tok -> let ep = Stream.count strm__ in tok, (bp, ep)
-        | _ -> let _ = Stream.empty strm__ in ("EOI", ""), (bp, succ bp)
-  and maybe_comment bp (strm__ : _ Stream.t) =
+  and left_paren bp (strm__ : _ Stream.t) =
     match Stream.peek strm__ with
       Some '*' ->
-        Stream.junk strm__; let s = strm__ in comment bp s; next_token_loc s
-    | _ ->
-        let ep = Stream.count strm__ in
-        let tok = keyword_or_error (bp, ep) "(" in tok, (bp, ep)
+        Stream.junk strm__;
+        let _ =
+          try comment bp strm__ with
+            Stream.Failure -> raise (Stream.Error "")
+        in
+        begin try next_token strm__ with
+          Stream.Failure -> raise (Stream.Error "")
+        end
+    | _ -> let ep = Stream.count strm__ in keyword_or_error (bp, ep) "("
   and comment bp (strm__ : _ Stream.t) =
     match Stream.peek strm__ with
-      Some '(' -> Stream.junk strm__; maybe_nested_comment bp strm__
-    | Some '*' -> Stream.junk strm__; maybe_end_comment bp strm__
+      Some '(' -> Stream.junk strm__; left_paren_in_comment bp strm__
+    | Some '*' -> Stream.junk strm__; star_in_comment bp strm__
     | Some c -> Stream.junk strm__; comment bp strm__
-    | _ -> let ep = Stream.count strm__ in err bp ep "comment not terminated"
-  and maybe_nested_comment bp (strm__ : _ Stream.t) =
+    | _ ->
+        let ep = Stream.count strm__ in err (bp, ep) "comment not terminated"
+  and left_paren_in_comment bp (strm__ : _ Stream.t) =
     match Stream.peek strm__ with
       Some '*' ->
         Stream.junk strm__; let s = strm__ in comment bp s; comment bp s
     | _ -> comment bp strm__
-  and maybe_end_comment bp (strm__ : _ Stream.t) =
+  and star_in_comment bp (strm__ : _ Stream.t) =
     match Stream.peek strm__ with
       Some ')' -> Stream.junk strm__; ()
     | _ -> comment bp strm__
@@ -609,10 +536,10 @@ let next_token_fun find_id_kwd find_spe_kwd =
               try any_to_nl strm__ with
                 Stream.Failure -> raise (Stream.Error "")
             in
-            next_token_loc strm__
+            next_token strm__
         | _ -> raise (Stream.Error "")
         end
-    | _ -> keyword_or_error (bp, bp + 1) "#", (bp, bp + 1)
+    | _ -> keyword_or_error (bp, bp + 1) "#"
   and spaces_tabs (strm__ : _ Stream.t) =
     match Stream.peek strm__ with
       Some (' ' | '\t') -> Stream.junk strm__; spaces_tabs strm__
@@ -623,51 +550,18 @@ let next_token_fun find_id_kwd find_spe_kwd =
     | _ -> ()
   and any_to_nl (strm__ : _ Stream.t) =
     match Stream.peek strm__ with
-      Some ('\013' | '\n') -> Stream.junk strm__; ()
+      Some ('\013' | '\010') -> Stream.junk strm__; ()
     | Some _ -> Stream.junk strm__; any_to_nl strm__
     | _ -> ()
   in
   fun cstrm ->
-    try next_token_loc cstrm with
-      Stream.Error str ->
-        err (Stream.count cstrm) (Stream.count cstrm + 1) str
-;;
-
-let locerr () = invalid_arg "Lexer: location function";;
-let loct_create () = ref (Array.create 1024 None);;
-let loct_func loct i =
-  match
-    if i < 0 || i >= Array.length !loct then None
-    else Array.unsafe_get !loct i
-  with
-    Some loc -> loc
-  | _ -> locerr ()
-;;
-let loct_add loct i loc =
-  if i >= Array.length !loct then
-    begin
-      let new_tmax = Array.length !loct * 2 in
-      let new_loct = Array.create new_tmax None in
-      Array.blit !loct 0 new_loct 0 (Array.length !loct); loct := new_loct; ()
-    end;
-  !loct.(i) <- Some loc;
-  ()
+    try next_token cstrm with
+      Stream.Error str -> err (Stream.count cstrm, Stream.count cstrm + 1) str
 ;;
 
 let func kwd_table =
   let find = Hashtbl.find kwd_table in
-  let lex cstrm =
-    let next_token_loc = next_token_fun find find in
-    let loct = loct_create () in
-    let ts =
-      Stream.from
-        (fun i ->
-           let (tok, loc) = next_token_loc cstrm in
-           loct_add loct i loc; Some tok)
-    in
-    let locf = loct_func loct in ts, locf
-  in
-  lex
+  Token.lexer_func_of_parser (next_token_fun find)
 ;;
 
 let rec check_keyword_stream (strm__ : _ Stream.t) =
@@ -679,7 +573,9 @@ let rec check_keyword_stream (strm__ : _ Stream.t) =
   true
 and check (strm__ : _ Stream.t) =
   match Stream.peek strm__ with
-    Some ('A'..'Z' | 'a'..'z' | ''..'' | ''..'' | ''..'') ->
+    Some
+      ('A'..'Z' | 'a'..'z' | '\192'..'\214' | '\216'..'\246' |
+       '\248'..'\255') ->
       Stream.junk strm__; check_ident strm__
   | Some
       ('!' | '?' | '~' | '=' | '@' | '^' | '&' | '+' | '-' | '*' | '/' | '%' |
@@ -741,8 +637,8 @@ and check (strm__ : _ Stream.t) =
 and check_ident (strm__ : _ Stream.t) =
   match Stream.peek strm__ with
     Some
-      ('A'..'Z' | 'a'..'z' | ''..'' | ''..'' | ''..'' | '0'..'9' | '_' |
-       '\'') ->
+      ('A'..'Z' | 'a'..'z' | '\192'..'\214' | '\216'..'\246' |
+       '\248'..'\255' | '0'..'9' | '_' | '\'') ->
       Stream.junk strm__; check_ident strm__
   | _ -> ()
 and check_ident2 (strm__ : _ Stream.t) =
@@ -768,7 +664,8 @@ let using_token kwd_table (p_con, p_prm) =
           else
             raise
               (Token.Error
-                 ("the token \"" ^ p_prm ^
+                 ("\
+the token \"" ^ p_prm ^
                     "\" does not respect Plexer rules"))
       end
   | "LIDENT" | "UIDENT" | "TILDEIDENT" | "TILDEIDENTCOLON" | "QUESTIONIDENT" |
@@ -778,7 +675,9 @@ let using_token kwd_table (p_con, p_prm) =
   | _ ->
       raise
         (Token.Error
-           ("the constructor \"" ^ p_con ^ "\" is not recognized by Plexer"))
+           ("\
+the constructor \"" ^ p_con ^
+              "\" is not recognized by Plexer"))
 ;;
 
 let removing_token kwd_table (p_con, p_prm) =
