@@ -5,7 +5,7 @@
 (*                                                                     *)
 (*        Daniel de Rauglaudre, projet Cristal, INRIA Rocquencourt     *)
 (*                                                                     *)
-(*  Copyright 1998 Institut National de Recherche en Informatique et   *)
+(*  Copyright 2001 Institut National de Recherche en Informatique et   *)
 (*  Automatique.  Distributed only by permission.                      *)
 (*                                                                     *)
 (***********************************************************************)
@@ -14,10 +14,13 @@
 
 open Stdpp;;
 
-let split_gext = ref false;;
+let split_ext = ref false;;
 
-Pcaml.add_option "-split_gext" (Arg.Set split_gext)
-  "  Split GEXTEND by functions to turn around a PowerPC problem.";;
+Pcaml.add_option "-split_ext" (Arg.Set split_ext)
+  "   Split EXTEND by functions to turn around a PowerPC problem.";;
+
+Pcaml.add_option "-split_gext" (Arg.Set split_ext)
+  "  Old name for the option -split_ext.";;
 
 type name = { expr : MLast.expr; tvar : string; loc : int * int };;
 
@@ -25,8 +28,10 @@ type ('e, 'p, 't) entry =
   { name : name; pos : 'e option; levels : ('e, 'p, 't) level list }
 and ('e, 'p, 't) level =
   { label : string option; assoc : 'e option; rules : ('e, 'p, 't) rule list }
-and ('e, 'p, 't) rule = { prod : ('e, 'p, 't) psymbol list; action : 'e }
-and ('e, 'p, 't) psymbol = { pattern : 'p; symbol : ('e, 'p, 't) symbol }
+and ('e, 'p, 't) rule =
+  { prod : ('e, 'p, 't) psymbol list; action : 'e option }
+and ('e, 'p, 't) psymbol =
+  { pattern : 'p option; symbol : ('e, 'p, 't) symbol }
 and ('e, 'p, 't) symbol =
   { used : name list; text : string -> string -> 'e; styp : string -> 't }
 ;;
@@ -38,8 +43,7 @@ let mark_used modif ht n =
     let rll = Hashtbl.find_all ht n.tvar in
     List.iter
       (fun (r, _) ->
-         if !r == Unused then
-           begin r := UsedNotScanned; modif := true; () end)
+         if !r == Unused then begin r := UsedNotScanned; modif := true end)
       rll
   with
     Not_found -> ()
@@ -93,8 +97,7 @@ let check_use nl el =
     (fun s (r, e) ->
        if !r = Unused then
          !(Pcaml.warning) e.name.loc ("Unused local entry \"" ^ s ^ "\""))
-    ht;
-  ()
+    ht
 ;;
 
 let locate n = let loc = n.loc in n.expr;;
@@ -109,6 +112,20 @@ let used_of_rule_list rl =
     rl
 ;;
 
+let retype_rule_list_without_patterns loc rl =
+  try
+    List.map
+      (function
+         {prod = [{pattern = None; symbol = s}]; action = None} ->
+           {prod = [{pattern = Some (MLast.PaLid (loc, "x")); symbol = s}];
+            action = Some (MLast.ExLid (loc, "x"))}
+       | {prod = []; action = Some _} as r -> r
+       | _ -> raise Exit)
+      rl
+  with
+    Exit -> rl
+;;
+
 let text_of_psymbol_list loc gmod psl tvar =
   List.fold_right
     (fun ps txt ->
@@ -118,11 +135,17 @@ let text_of_psymbol_list loc gmod psl tvar =
 ;;
 
 let text_of_action loc psl rtvar act tvar =
+  let locid = MLast.PaLid (loc, !(Stdpp.loc_name)) in
+  let act =
+    match act with
+      Some act -> act
+    | None -> MLast.ExUid (loc, "()")
+  in
   let e =
     MLast.ExFun
       (loc,
        [MLast.PaTyc
-          (loc, MLast.PaLid (loc, "loc"),
+          (loc, locid,
            MLast.TyTup
              (loc, [MLast.TyLid (loc, "int"); MLast.TyLid (loc, "int")])),
         None, MLast.ExTyc (loc, act, MLast.TyQuo (loc, rtvar))])
@@ -131,8 +154,8 @@ let text_of_action loc psl rtvar act tvar =
     List.fold_left
       (fun txt ps ->
          match ps.pattern with
-           MLast.PaAny _ -> MLast.ExFun (loc, [MLast.PaAny loc, None, txt])
-         | p ->
+           None -> MLast.ExFun (loc, [MLast.PaAny loc, None, txt])
+         | Some p ->
              let t = ps.symbol.styp tvar in
              MLast.ExFun (loc, [MLast.PaTyc (loc, p, t), None, txt]))
       e psl
@@ -308,29 +331,63 @@ let let_in_of_extend loc gmod functor_version gl el args =
 ;;
 
 let text_of_extend loc gmod gl el f =
-  let args =
-    List.fold_right
-      (fun e args ->
-         let (ent, pos, txt) = text_of_entry loc gmod e in
-         let ent =
+  if !split_ext then
+    let args =
+      List.map
+        (fun e ->
+           let (ent, pos, txt) = text_of_entry e.name.loc gmod e in
+           let ent =
+             MLast.ExApp
+               (loc,
+                MLast.ExAcc
+                  (loc,
+                   MLast.ExAcc
+                     (loc, MLast.ExUid (loc, gmod),
+                      MLast.ExUid (loc, "Entry")),
+                   MLast.ExLid (loc, "obj")),
+                ent)
+           in
+           let e = MLast.ExTup (loc, [ent; pos; txt]) in
+           MLast.ExLet
+             (loc, false,
+              [MLast.PaLid (loc, "aux"),
+               MLast.ExFun
+                 (loc,
+                  [MLast.PaUid (loc, "()"), None,
+                   MLast.ExApp
+                     (loc, f,
+                      MLast.ExApp
+                        (loc, MLast.ExApp (loc, MLast.ExUid (loc, "::"), e),
+                         MLast.ExUid (loc, "[]")))])],
+              MLast.ExApp
+                (loc, MLast.ExLid (loc, "aux"), MLast.ExUid (loc, "()"))))
+        el
+    in
+    let args = MLast.ExSeq (loc, args) in
+    let_in_of_extend loc gmod false gl el args
+  else
+    let args =
+      List.fold_right
+        (fun e el ->
+           let (ent, pos, txt) = text_of_entry e.name.loc gmod e in
+           let ent =
+             MLast.ExApp
+               (loc,
+                MLast.ExAcc
+                  (loc,
+                   MLast.ExAcc
+                     (loc, MLast.ExUid (loc, gmod),
+                      MLast.ExUid (loc, "Entry")),
+                   MLast.ExLid (loc, "obj")),
+                ent)
+           in
+           let e = MLast.ExTup (loc, [ent; pos; txt]) in
            MLast.ExApp
-             (loc,
-              MLast.ExAcc
-                (loc,
-                 MLast.ExAcc
-                   (loc, MLast.ExUid (loc, gmod), MLast.ExUid (loc, "Entry")),
-                 MLast.ExLid (loc, "obj")),
-              ent)
-         in
-         MLast.ExApp
-           (loc,
-            MLast.ExApp
-              (loc, MLast.ExUid (loc, "::"),
-               MLast.ExTup (loc, [ent; pos; txt])),
-            args))
-      el (MLast.ExUid (loc, "[]"))
-  in
-  MLast.ExApp (loc, f, let_in_of_extend loc gmod false gl el args)
+             (loc, MLast.ExApp (loc, MLast.ExUid (loc, "::"), e), el))
+        el (MLast.ExUid (loc, "[]"))
+    in
+    let args = let_in_of_extend loc gmod false gl el args in
+    MLast.ExApp (loc, f, args)
 ;;
 
 let text_of_functorial_extend loc gmod gl el =
@@ -338,7 +395,7 @@ let text_of_functorial_extend loc gmod gl el =
     let el =
       List.map
         (fun e ->
-           let (ent, pos, txt) = text_of_entry loc gmod e in
+           let (ent, pos, txt) = text_of_entry e.name.loc gmod e in
            let e =
              MLast.ExApp
                (loc,
@@ -353,7 +410,7 @@ let text_of_functorial_extend loc gmod gl el =
                    pos),
                 txt)
            in
-           if !split_gext then
+           if !split_ext then
              MLast.ExLet
                (loc, false,
                 [MLast.PaLid (loc, "aux"),
@@ -363,7 +420,7 @@ let text_of_functorial_extend loc gmod gl el =
            else e)
         el
     in
-    MLast.ExSeq (loc, el, MLast.ExUid (loc, "()"))
+    MLast.ExSeq (loc, el)
   in
   let_in_of_extend loc gmod true gl el args
 ;;
@@ -790,7 +847,7 @@ Grammar.extend
        Gramext.Stoken ("", "]")],
       Gramext.action
         (fun _ (rules : 'rule list) _ (loc : int * int) ->
-           (rules : 'rule_list));
+           (retype_rule_list_without_patterns loc rules : 'rule_list));
       [Gramext.Stoken ("", "["); Gramext.Stoken ("", "]")],
       Gramext.action (fun _ _ (loc : int * int) -> ([] : 'rule_list))]];
     Grammar.Entry.obj (rule : 'rule Grammar.Entry.e), None,
@@ -801,7 +858,7 @@ Grammar.extend
           Gramext.Stoken ("", ";"))],
       Gramext.action
         (fun (psl : 'psymbol list) (loc : int * int) ->
-           ({prod = psl; action = MLast.ExUid (loc, "()")} : 'rule));
+           ({prod = psl; action = None} : 'rule));
       [Gramext.Slist0sep
          (Gramext.Snterm
             (Grammar.Entry.obj (psymbol : 'psymbol Grammar.Entry.e)),
@@ -810,20 +867,20 @@ Grammar.extend
        Gramext.Snterm (Grammar.Entry.obj (expr : 'expr Grammar.Entry.e))],
       Gramext.action
         (fun (act : 'expr) _ (psl : 'psymbol list) (loc : int * int) ->
-           ({prod = psl; action = act} : 'rule))]];
+           ({prod = psl; action = Some act} : 'rule))]];
     Grammar.Entry.obj (psymbol : 'psymbol Grammar.Entry.e), None,
     [None, None,
      [[Gramext.Snterm (Grammar.Entry.obj (symbol : 'symbol Grammar.Entry.e))],
       Gramext.action
         (fun (s : 'symbol) (loc : int * int) ->
-           ({pattern = MLast.PaAny loc; symbol = s} : 'psymbol));
+           ({pattern = None; symbol = s} : 'psymbol));
       [Gramext.Snterm
          (Grammar.Entry.obj (pattern : 'pattern Grammar.Entry.e));
        Gramext.Stoken ("", "=");
        Gramext.Snterm (Grammar.Entry.obj (symbol : 'symbol Grammar.Entry.e))],
       Gramext.action
         (fun (s : 'symbol) _ (p : 'pattern) (loc : int * int) ->
-           ({pattern = p; symbol = s} : 'psymbol));
+           ({pattern = Some p; symbol = s} : 'psymbol));
       [Gramext.Stoken ("LIDENT", "");
        Gramext.Sopt
          (Gramext.srules
@@ -837,13 +894,14 @@ Grammar.extend
             let text = snterm loc name lev in
             let styp _ = MLast.TyQuo (loc, i) in
             let symb = {used = [name]; text = text; styp = styp} in
-            {pattern = MLast.PaAny loc; symbol = symb} :
+            {pattern = None; symbol = symb} :
             'psymbol));
       [Gramext.Stoken ("LIDENT", ""); Gramext.Stoken ("", "=");
        Gramext.Snterm (Grammar.Entry.obj (symbol : 'symbol Grammar.Entry.e))],
       Gramext.action
         (fun (s : 'symbol) _ (p : string) (loc : int * int) ->
-           ({pattern = MLast.PaLid (loc, p); symbol = s} : 'psymbol))]];
+           ({pattern = Some (MLast.PaLid (loc, p)); symbol = s} :
+            'psymbol))]];
     Grammar.Entry.obj (symbol : 'symbol Grammar.Entry.e), None,
     [Some "top", Some Gramext.NonA,
      [[Gramext.Stoken ("UIDENT", "OPT"); Gramext.Sself],
@@ -981,7 +1039,8 @@ Grammar.extend
        Gramext.Stoken ("", "]")],
       Gramext.action
         (fun _ (rl : 'rule list) _ (loc : int * int) ->
-           (let t = new_type_var () in
+           (let rl = retype_rule_list_without_patterns loc rl in
+            let t = new_type_var () in
             {used = used_of_rule_list rl; text = srules loc t rl;
              styp = fun _ -> MLast.TyQuo (loc, t)} :
             'symbol));
