@@ -126,8 +126,8 @@ let stream_pattern_component skont ckont =
          [MLast.PaApp (loc, MLast.PaUid (loc, "Some"), p), wo,
           MLast.ExSeq
             (loc,
-             [MLast.ExApp (loc, junk_fun loc, MLast.ExLid (loc, strm_n))],
-             skont);
+             [MLast.ExApp (loc, junk_fun loc, MLast.ExLid (loc, strm_n));
+              skont]);
           MLast.PaAny loc, None, ckont])
   | SpNtr (loc, p, e) ->
       let e =
@@ -262,8 +262,8 @@ let stream_patterns_term loc ekont tspel =
            let skont = stream_pattern loc epo e ekont spcl in
            MLast.ExSeq
              (loc,
-              [MLast.ExApp (loc, junk_fun loc, MLast.ExLid (loc, strm_n))],
-              skont)
+              [MLast.ExApp (loc, junk_fun loc, MLast.ExLid (loc, strm_n));
+               skont])
          in
          p, w, e)
       tspel
