@@ -10,27 +10,29 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: ocpp.ml,v 2.1 1999/11/30 11:00:35 doligez Exp $ *)
+(* $Id: ocpp.ml,v 3.1 2001/04/27 02:41:11 ddr Exp $ *)
 
 value buff = ref (String.create 80);
 value store len x =
-  do if len >= String.length buff.val then
-       buff.val := buff.val ^ String.create (String.length buff.val)
-     else ();
-     buff.val.[len] := x;
-  return succ len
+  do {
+    if len >= String.length buff.val then
+      buff.val := buff.val ^ String.create (String.length buff.val)
+    else ();
+    buff.val.[len] := x;
+    succ len
+  }
 ;
 value get_buff len = String.sub buff.val 0 len;
 
 value rec copy_strip_locate cs =
   match cs with parser
   [ [: `'$' :] -> maybe_locate cs
-  | [: `c :] -> do print_char c; return copy_strip_locate cs
+  | [: `c :] -> do { print_char c; copy_strip_locate cs }
   | [: :] -> () ]
 and maybe_locate cs =
   match cs with parser
   [ [: `'1'..'9' :] -> locate cs
-  | [: :] -> do print_char '$'; return copy_strip_locate cs ]
+  | [: :] -> do { print_char '$'; copy_strip_locate cs } ]
 and locate cs =
   match cs with parser
   [ [: `'0'..'9' :] -> locate cs
@@ -39,8 +41,8 @@ and locate cs =
 and inside_locate cs =
   match cs with parser
   [ [: `'$' :] -> copy_strip_locate cs
-  | [: `'\\'; `c :] -> do print_char c; return inside_locate cs
-  | [: `c :] -> do print_char c; return inside_locate cs
+  | [: `'\\'; `c :] -> do { print_char c; inside_locate cs }
+  | [: `c :] -> do { print_char c; inside_locate cs }
   | [: :] -> raise (Stream.Error "end of file in locate directive") ]
 ;
 
@@ -60,8 +62,7 @@ value quot name pos str =
         Stdpp.raise_with_loc (pos + p1, pos + p2) exc
     | exc -> Stdpp.raise_with_loc (pos, pos + String.length str) exc ]
   in
-  let cs = Stream.of_string new_str in
-  copy_strip_locate cs
+  let cs = Stream.of_string new_str in copy_strip_locate cs
 ;
 
 value rec ident len =
@@ -74,15 +75,15 @@ value rec ident len =
 value rec copy cs =
   match cs with parser
   [ [: `'<' :] -> maybe_quot cs
-  | [: `'"' :] -> do print_char '"'; return inside_string cs
-  | [: `c :] -> do print_char c; return copy cs
+  | [: `'"' :] -> do { print_char '"'; inside_string cs }
+  | [: `c :] -> do { print_char c; copy cs }
   | [: :] -> () ]
 and maybe_quot cs =
   match cs with parser
   [ [: `'<' :] ep -> inside_quot "" ep 0 cs
   | [: `':'; i = ident 0; `'<' ? "less char expected" :] ep ->
       inside_quot i ep 0 cs
-  | [: :] -> do print_char '<'; return copy cs ]
+  | [: :] -> do { print_char '<'; copy cs } ]
 and inside_quot name pos len cs =
   match cs with parser
   [ [: `'>' :] -> maybe_end_quot name pos len cs
@@ -90,18 +91,16 @@ and inside_quot name pos len cs =
   | [: :] -> raise (Stream.Error "end of file in quotation") ]
 and maybe_end_quot name pos len cs =
   match cs with parser
-  [ [: `'>' :] -> do quot name pos (get_buff len); return copy cs
+  [ [: `'>' :] -> do { quot name pos (get_buff len); copy cs }
   | [: :] -> inside_quot name pos (store len '>') cs ]
 and inside_string cs =
   match cs with parser
-  [ [: `'"' :] -> do print_char '"'; return copy cs
-  | [: `c :] -> do print_char c; return inside_string cs
+  [ [: `'"' :] -> do { print_char '"'; copy cs }
+  | [: `c :] -> do { print_char c; inside_string cs }
   | [: :] -> raise (Stream.Error "end of file in string") ]
 ;
 
-value copy_quot cs =
-  do copy cs; flush stdout; return ()
-;
+value copy_quot cs = do { copy cs; flush stdout; };
 
 value find_line (bp, ep) ic =
   find 0 1 0 where rec find i line col =
@@ -117,16 +116,13 @@ value loc_fmt =
   match Sys.os_type with
   [ "MacOS" ->
       ("File \"%s\"; line %d; characters %d to %d\n### " : format 'a 'b 'c)
-  | _ -> ("File \"%s\", line %d, characters %d-%d:\n" : format 'a 'b 'c)]
+  | _ -> ("File \"%s\", line %d, characters %d-%d:\n" : format 'a 'b 'c) ]
 ;
 
 value print_location loc file =
   let ic = open_in_bin file in
   let (line, c1, c2) = find_line loc ic in
-  do close_in ic;
-     Printf.eprintf loc_fmt file line c1 c2;
-     flush stderr;
-  return ()
+  do { close_in ic; Printf.eprintf loc_fmt file line c1 c2; flush stderr; }
 ;
 
 value file = ref "";
@@ -134,17 +130,20 @@ Arg.parse [] (fun x -> file.val := x) "ocpp <objects> <file>";
 
 value main () =
   try
-    if file.val <> "" then copy_quot (Stream.of_channel (open_in_bin file.val))
+    if file.val <> "" then
+      copy_quot (Stream.of_channel (open_in_bin file.val))
     else ()
   with exc ->
-    do print_newline (); flush stdout; return
-    let exc =
-      match exc with
-      [ Stdpp.Exc_located loc exc ->
-          do print_location loc file.val; return exc
-      | exc -> exc ]
-    in
-    raise exc
+    do {
+      print_newline ();
+      flush stdout;
+      let exc =
+        match exc with
+        [ Stdpp.Exc_located loc exc -> do { print_location loc file.val; exc }
+        | exc -> exc ]
+      in
+      raise exc
+    }
 ;
 
 Odyl_main.name.val := "ocpp";
